! -*- Mode: f90 -*-
PROGRAM allgrid
   !!
   !! Lit les fichiers de masques, grilles et surfaces des deux models,
   !! et reecrit dans un seul fichier
   !! Un simple 'cat' des deux fichiers dans un seul ne marche pas (pourquoi ?)
   !!
   !! Fichiers IEEE : 11,12,13,21,22,23,31,32,33
   !!
   USE declare
   USE dimensions
   USE modeles
   USE mod_locio
   USE mod_prih
   USE mod_inipar
   IMPLICIT NONE
   !!
   REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: xoclon, xoclat
   REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: xaclon, xaclat
   !!
   INTEGER (kind=il) :: nocegrid = 11, nocesrf = 12, nocemsk = 13
   INTEGER (kind=il) :: natmgrid = 21, natmsrf = 22, natmmsk = 23
   INTEGER (kind=il) :: ncplgrid = 31, ncplsrf = 32, ncplmsk = 33
   CHARACTER (len = 80) :: cfname
   CHARACTER (len=20) :: cores
   CHARACTER (len=1) :: c_i, c_r
   REAL    (kind=rl), ALLOCATABLE, DIMENSION (:, :) :: z2a, z2o
   INTEGER (kind=il), ALLOCATABLE, DIMENSION (:, :) :: i2a, i2o
   INTEGER (kind=il) :: il_grid, il_mask, il_area, il_o2a, il_a2o, nco2a
   !!
   INTEGER (kind=il) :: ierr = 0
   !!
   CALL inipar
   CALL alloc_modeles
   !!
   ALLOCATE (xoclon (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'xoclon', lreset = .TRUE., crout = 'allgrid')
   ALLOCATE (xoclat (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'xoclat')
   ALLOCATE (xaclon (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'xaclon')
   ALLOCATE (xaclat (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'xaclat')
   !!
   WRITE (nout, fmt = '("Dimension atmopshere : ", 3I8)') jpai, jpaj, jpan
   WRITE (nout, fmt = '("Dimension ocean      : ", 3I8)') jpoi, jpoj, jpon
   !!
   SELECT CASE (TRIM (cotyp))
   CASE ('orca4')
      cores   = 'orca4'
   CASE ('orca2')
      cores   = 'orca2'
   CASE ('orca2.0')
      cores   = 'orca2.0'
   CASE ('orca2.1')
      cores   = 'orca2.1'
   CASE ('orca05')
      cores   = 'orca05'
   END SELECT
   !!
   IF (locerev) THEN
      comod_t = 't'//TRIM (comod)
      comod_u = 'u'//TRIM (comod)
      comod_v = 'v'//TRIM (comod)
      comod_f = 'f'//TRIM (comod)
   ELSE
      comod_t = TRIM (comod) // 't' 
      comod_u = TRIM (comod) // 'u' 
      comod_v = TRIM (comod) // 'v' 
      comod_f = TRIM (comod) // 'f' 
   ENDIF
   camod_a = 'a' // TRIM (camod)
   camod_t = 't' // TRIM (camod)
   camod_u = 'u' // TRIM (camod)
   camod_v = 'v' // TRIM (camod)
   !!
   !!IF (TRIM(c_period) /= 'none') cores = TRIM (cores) // TRIM (c_period)
   !! 
   WRITE (unit = c_r, fmt = '(1I1)') rk_in
   cfname = TRIM (cores) // ".grids.r" // TRIM(c_r)
   WRITE(nout,*) 'Reading ', TRIM(cfname)
   OPEN (unit = nocegrid, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
   CALL locread (TRIM(comod_t) // '.lon', xolont, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 001
   CALL locread (TRIM(comod_t) // '.lat', xolatt, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 002
   CALL locread (TRIM(comod_u) // '.lon', xolonu, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 003
   CALL locread (TRIM(comod_u) // '.lat', xolatu, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 004
   CALL locread (TRIM(comod_v) // '.lon', xolonv, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 005
   CALL locread (TRIM(comod_v) // '.lat', xolatv, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 006
   CALL locread (TRIM(comod_f) // '.lon', xolonf, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 007
   CALL locread (TRIM(comod_f) // '.lat', xolatf, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 008
   CALL locread (TRIM(comod_t) // '.clo', xoclon, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 009
   CALL locread (TRIM(comod_t) // '.cla', xoclat, jpon, nocegrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 010
   !
   SELECT CASE (TRIM(catyp))
   CASE ('lmdz')
      WRITE (unit = cfname, fmt = '("lmd.grids.r", 1I1)') rk_in
      WRITE(nout,*) 'Reading ', TRIM(cfname)
      OPEN (unit = natmgrid, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
      CALL locread (TRIM(camod_t)//'.lon', xalont, jpan , natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 011
      CALL locread (TRIM(camod_t)//'.lat', xalatt, jpan , natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 012
      CALL locread (TRIM(camod_u)//'.lon', xalonu, jpan , natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 013
      CALL locread (TRIM(camod_u)//'.lat', xalatu, jpan , natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 014
      CALL locread (TRIM(camod_v)//'.lon', xalonv, jpanv, natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 015
      CALL locread (TRIM(camod_v)//'.lat', xalatv, jpanv, natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 016
      CALL locread (TRIM(camod_t)//'.clo', xaclon, jpan , natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 017
      CALL locread (TRIM(camod_t)//'.cla', xaclat, jpan , natmgrid, ierr, 6_il) ; IF (ierr /= 0_il) STOP 017
   CASE ('gaussian')
      cfname = "grids_orca_t106"
      WRITE(nout,*) 'Reading ', TRIM(cfname)
      OPEN (unit = natmgrid, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
      CALL locread (TRIM(camod_a)//'.lon', xalont, jpan , natmgrid, ierr, 6_il) ;   IF (ierr /= 0_il) STOP 011
      CALL locread (TRIM(camod_a)//'.lat', xalatt, jpan , natmgrid, ierr, 6_il) ;   IF (ierr /= 0_il) STOP 012
      xalonu = xalont
      xalatu = xalatt
      xalonv = xalont
      xalatv = xalatt
   END SELECT
   !
   WRITE (unit = cfname, fmt = '("grids.r", 1I1)') rk_in
   WRITE(nout,*) 'Writing ', TRIM(cfname)
   OPEN (unit = ncplgrid, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'write')
   WRITE (unit = ncplgrid) TRIM(comod_t) // '.lon' ; WRITE (unit = ncplgrid) REAL (xolont, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_t) // '.lat' ; WRITE (unit = ncplgrid) REAL (xolatt, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_u) // '.lon' ; WRITE (unit = ncplgrid) REAL (xolonu, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_u) // '.lat' ; WRITE (unit = ncplgrid) REAL (xolatu, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_v) // '.lon' ; WRITE (unit = ncplgrid) REAL (xolonv, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_v) // '.lat' ; WRITE (unit = ncplgrid) REAL (xolatv, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_f) // '.lon' ; WRITE (unit = ncplgrid) REAL (xolonf, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_f) // '.lat' ; WRITE (unit = ncplgrid) REAL (xolatf, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_t) // '.clo' ; WRITE (unit = ncplgrid) REAL (xoclon, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(comod_t) // '.cla' ; WRITE (unit = ncplgrid) REAL (xoclat, kind = rk_in)

   WRITE (unit = ncplgrid) TRIM(camod_t) //'.lon' ; WRITE (unit = ncplgrid) REAL (xalont, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(camod_t) //'.lat' ; WRITE (unit = ncplgrid) REAL (xalatt, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(camod_u) //'.lon' ; WRITE (unit = ncplgrid) REAL (xalonu, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(camod_u) //'.lat' ; WRITE (unit = ncplgrid) REAL (xalatu, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(camod_v) //'.lon' ; WRITE (unit = ncplgrid) REAL (xalonv, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(camod_v) //'.lat' ; WRITE (unit = ncplgrid) REAL (xalatv, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(camod_t) //'.clo' ; WRITE (unit = ncplgrid) REAL (xaclon, kind = rk_in)
   WRITE (unit = ncplgrid) TRIM(camod_t) //'.cla' ; WRITE (unit = ncplgrid) REAL (xaclat, kind = rk_in)
   !!
   WRITE (unit = c_i, fmt = '(1I1)') ik_in
   cfname = TRIM(cores) // ".masks.i" // TRIM(c_i)
   WRITE(nout,*) 'Reading ', TRIM(cfname)
   OPEN (unit = nocemsk, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
   CALL locrint (TRIM(comod_t) // '.msk', iomskt, jpon, nocemsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 021
   CALL locrint (TRIM(comod_u) // '.msk', iomsku, jpon, nocemsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 022
   CALL locrint (TRIM(comod_v) // '.msk', iomskv, jpon, nocemsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 023
   CALL locrint (TRIM(comod_f) // '.msk', iomskf, jpon, nocemsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 023
   !
   SELECT CASE (TRIM(catyp))
   CASE ('lmdz')
      WRITE (unit = cfname, fmt = '("lmd.masks.i", 1I1)') ik_in
      WRITE(nout,*) 'Reading ', TRIM(cfname)
      OPEN (unit = natmmsk, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
      CALL locrint (TRIM(camod_t) // '.msk', iamskt, jpan , natmmsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 031
      CALL locrint (TRIM(camod_u) // '.msk', iamsku, jpan , natmmsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 032
      CALL locrint (TRIM(camod_v) // '.msk', iamskv, jpanv, natmmsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 033
   CASE ('gaussian')
      cfname = "masks_orca_t106"
      WRITE(nout,*) 'Reading ', TRIM(cfname)
      OPEN (unit = natmmsk, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
      CALL locrint (TRIM(camod_a) // '.msk', iamskt, jpan , natmmsk, ierr, 6_il) ; IF (ierr /= 0_il) STOP 031
      iamsku = iamskt
      iamskv = iamskt
   END SELECT
   !!
   WRITE (unit = nout, fmt = *) 'Masque atm'
   CALL prihin (RESHAPE ( iamskt, (/ jpai, jpaj /)), 1_il, nout)
   WRITE (unit = nout, fmt = *) 'Masque ocean'
   CALL prihin (RESHAPE ( iomskt, (/ jpoi, jpoj /)), 1_il, nout)
   !!
   WRITE (unit = cfname, fmt = '("masks.i", 1I1)') i_4
   WRITE(nout,*) 'Writing ', TRIM(cfname)
   OPEN (unit = ncplmsk, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'write')
   WRITE(nout,*) 'Writing ', TRIM(comod_t) // '.msk'
   WRITE (unit = ncplmsk) TRIM(comod_t) // '.msk' ; WRITE (unit = ncplmsk) INT (iomskt, kind = i_4)
   WRITE (unit = ncplmsk) TRIM(comod_u) // '.msk' ; WRITE (unit = ncplmsk) INT (iomsku, kind = i_4)
   WRITE (unit = ncplmsk) TRIM(comod_v) // '.msk' ; WRITE (unit = ncplmsk) INT (iomskv, kind = i_4)
   WRITE (unit = ncplmsk) TRIM(comod_f) // '.msk' ; WRITE (unit = ncplmsk) INT (iomskf, kind = i_4)

   WRITE (unit = ncplmsk) TRIM(camod_t) // '.msk' ; WRITE (unit = ncplmsk) INT (iamskt, kind = i_4)
   WRITE (unit = ncplmsk) TRIM(camod_u) // '.msk' ; WRITE (unit = ncplmsk) INT (iamsku, kind = i_4)
   WRITE (unit = ncplmsk) TRIM(camod_v) // '.msk' ; WRITE (unit = ncplmsk) INT (iamskv, kind = i_4)
   CLOSE  (unit = ncplmsk)
   !
   WRITE (unit = cfname, fmt = '("masks.i", 1I1)') i_8
   WRITE(nout,*) 'Writing ', TRIM(cfname)
   OPEN (unit = ncplmsk, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'write')
   WRITE(nout,*) 'Writing ', TRIM(comod_t) // '.msk'
   WRITE (unit = ncplmsk) TRIM(comod_t) // '.msk' ; WRITE (unit = ncplmsk) INT (iomskt, kind = i_8)
   WRITE (unit = ncplmsk) TRIM(comod_u) // '.msk' ; WRITE (unit = ncplmsk) INT (iomsku, kind = i_8)
   WRITE (unit = ncplmsk) TRIM(comod_v) // '.msk' ; WRITE (unit = ncplmsk) INT (iomskv, kind = i_8)
   WRITE (unit = ncplmsk) TRIM(comod_f) // '.msk' ; WRITE (unit = ncplmsk) INT (iomskf, kind = i_8)

   WRITE(nout,*) 'Writing ', TRIM(camod_t)//'.msk' 
   WRITE (unit = ncplmsk) TRIM(camod_t) // '.msk' ; WRITE (unit = ncplmsk) INT (iamskt, kind = i_8)
   WRITE (unit = ncplmsk) TRIM(camod_t) // '.msk' ; WRITE (unit = ncplmsk) INT (iamsku, kind = i_8)
   WRITE (unit = ncplmsk) TRIM(camod_t) // '.msk' ; WRITE (unit = ncplmsk) INT (iamskv, kind = i_8)
   CLOSE  (unit = ncplmsk)
   !!
   WRITE (unit = c_r, fmt = '(1I1)') rk_in
   cfname = TRIM(cores) // ".areas.r" // TRIM(c_r)
   WRITE(nout,*) 'writing ', TRIM(cfname)
   OPEN (unit = nocesrf, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
   CALL locread (TRIM(comod_t) // '.srf', xosrft, jpon, nocesrf, ierr, 6_il) ; IF (ierr /= 0_il) STOP 041
   CALL locread (TRIM(comod_u) // '.srf', xosrfu, jpon, nocesrf, ierr, 6_il) ; IF (ierr /= 0_il) STOP 042
   CALL locread (TRIM(comod_v) // '.srf', xosrfv, jpon, nocesrf, ierr, 6_il) ; IF (ierr /= 0_il) STOP 043
   !
   SELECT CASE (TRIM(catyp))
   CASE ('lmdz')
      WRITE (unit = cfname, fmt = '("lmd.areas.r", 1I1)') rk_in
      WRITE(nout,*) 'Reading ', TRIM(cfname)
      OPEN (unit = natmsrf, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
      CALL locread (TRIM(camod_t) // '.srf', xasrft, jpan , natmsrf, ierr, 6_il) ; IF (ierr /= 0_il) STOP 051
      CALL locread (TRIM(camod_u) // '.srf', xasrfu, jpan , natmsrf, ierr, 6_il) ; IF (ierr /= 0_il) STOP 052
      CALL locread (TRIM(camod_v) // '.srf', xasrfv, jpanv, natmsrf, ierr, 6_il) ; IF (ierr /= 0_il) STOP 053
   CASE (' gaussian')
      cfname = "areas_orca_t106"
      WRITE(nout,*) 'reading ', TRIM(cfname)
      OPEN (unit = natmsrf, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read')
      CALL locread (TRIM(camod_a) // '.srf', xasrft, jpan , natmsrf, ierr, 6_il) ; IF (ierr /= 0_il) STOP 051
      xasrfu = xasrft
      xasrfv = xasrft
   END SELECT
   !
   WRITE (unit = cfname, fmt = '("areas.r", 1I1)') rk_in
   WRITE(nout,*) 'Writing ', TRIM(cfname)
   OPEN (unit = ncplsrf, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'write')
   WRITE (unit=ncplsrf) TRIM(comod_t) // '.srf' ; WRITE (unit = ncplsrf) REAL (xosrft, kind = rk_in)
   WRITE (unit=ncplsrf) TRIM(comod_u) // '.srf' ; WRITE (unit = ncplsrf) REAL (xosrfu, kind = rk_in)
   WRITE (unit=ncplsrf) TRIM(comod_v) // '.srf' ; WRITE (unit = ncplsrf) REAL (xosrfv, kind = rk_in)

   WRITE (unit = ncplsrf) TRIM(camod_t) // '.srf' ; WRITE (unit = ncplsrf) REAL (xasrft, kind = rk_in)
   WRITE (unit = ncplsrf) TRIM(camod_u) // '.srf' ; WRITE (unit = ncplsrf) REAL (xasrfu, kind = rk_in)
   WRITE (unit = ncplsrf) TRIM(camod_v) // '.srf' ; WRITE (unit = ncplsrf) REAL (xasrfv, kind = rk_in)
   !!
   STOP
END PROGRAM allgrid
