! -*- Mode: f90 -*-
MODULE bords
   USE modeles
   USE formula
   PRIVATE
   PUBLIC bord_a, bord_o, bord_oa
CONTAINS
   !!
   SUBROUTINE bord_a 
      IMPLICIT NONE
      INTEGER (kind=il) :: jai, jaj
      REAL (kind=rl) :: xs_n, xs_s
      
      WRITE (nout,*) 'Debut bord_a'
      
      SELECT CASE (TRIM(catyp))
      CASE ('lmd5')
         CALL bord_lmd5 ( xa_ed, ya_ed )
      CASE ('lmdz')
         CALL bord_lmdz ( xa_ed, ya_ed )
      CASE ('gaussian')
         CALL bord_gaussian ( xa_ed, ya_ed )
      END SELECT

!-$$      xa_ed ( :, jpae) = xalont ( : )
!-$$      ya_ed ( :, jpae) = xalatt ( : )
      !
      !! Periodicity masks
      iamskp = 0_il
      
      IF ( la_pole ) THEN 
         DO jai = 2, jpai
            iamskp (m1a (jai, 1_il)) = 1_il
            iamskp (m1a (jai, jpaj)) = 1_il
         END DO
         xs_n = 0.0_rl ; xs_s = 0.0_rl
         DO jai = 1_il, jpai
            xs_n = xs_n + xasrft ( m1a (jai, jpaj))
            xs_s = xs_n + xasrft ( m1a (jai,  1  ))
         END DO
         DO jai = 2_il, jpai
            xasrft ( m1a (jai, jpaj)) = 0.0_rl
            xasrft ( m1a (jai,  1  )) = 0.0_rl
         END DO
         xasrft ( m1a (1_il, jpaj)) = xs_s
         xasrft ( m1a (1_il, 1_il)) = xs_n  
      END IF
      
      RETURN
      
   END SUBROUTINE bord_a
   
   SUBROUTINE bord_o
      IMPLICIT NONE
      INTEGER (kind=il) :: joi, joj

      WRITE (nout,*) 'Debut bord_o'      
            
      SELECT CASE ( TRIM(comod) )
      CASE ('opa')
         CALL bord_opa ( xo_ed, yo_ed )
      CASE ('orc')
         CALL bord_opa ( xo_ed, yo_ed )
      CASE ('grn')
         CALL bord_grisli ( xo_ed, yo_ed )
      CASE ('grs')
         CALL bord_grisli ( xo_ed, yo_ed )
      END SELECT

      xo_ed ( :, jpoe) = xolont ( : )
      yo_ed ( :, jpoe) = xolatt ( : )

      iomskp = 0_il

      ! Periodical edge set to zero to suppress double points for ocean
      SELECT CASE ( noperio)
      CASE ( 1, 4, 5 ,6 )
         DO joj = 1, jpoj 
            iomskp (m1o (1_il, joj)) = 1_il
            iomskp (m1o (jpoi, joj)) = 1_il
         END DO
      END SELECT

      IF (noperio == -4_il ) THEN 
         DO joi = jpoi/2+2, jpoi
            iomskp (m1o (joi, jpoj) ) = 1_il
         END DO
      ENDIF

      ! Supress duplicate points due to the North fold
      SELECT CASE (noperio)
      CASE ( 3, 4)
         DO joi = 1, jpoi
            iomskp (m1o (joi, jpoj  ) ) = 1_il
         END DO
         DO joi = jpoi/2+2, jpoi
            iomskp (m1o (joi, jpoj-1) ) = 1_il
         END DO
      CASE ( 6)
         DO joi = 1, jpoi
            iomskp (m1o (joi, jpoj  ) ) = 1_il
         END DO
      CASE ( -1)
         DO joi = 2, jpoi
            iomskp (m1o (joi, 1_il)) = 1_il
            iomskp (m1o (joi, jpoj)) = 1_il
         END DO
      CASE (-4) 
         DO joi = jpoi/2+2, jpoi
            iomskp (m1o (joi, jpoj) ) = 1_il
         END DO
      END SELECT

      RETURN

   END SUBROUTINE bord_o
      
   SUBROUTINE bord_oa
      IMPLICIT NONE
      INTEGER :: jn, jo
      REAL (kind=rl) :: xmin, xmax

      WRITE (nout,*) 'Debut bord_oa'
            
      !!
      !! All ocean longitudes are set within the limits of the atmospheric grid
      !!
      xinf = MINVAL (xa_ed) ; xsup = MAXVAL (xa_ed)
      !!
      DO jn = 1, 3
         WHERE ( xo_ed (:,jpoe) <  xinf ) xo_ed (:,jpoe) = xo_ed (:,jpoe) + 360.0_rl
         WHERE ( xo_ed (:,jpoe) >  xsup ) xo_ed (:,jpoe) = xo_ed (:,jpoe) - 360.0_rl
      END DO
      !!
      !! In longitude, takes the closest lon, referring to the center of the box
      DO jn = 1, jpoe-1
         DO jo = 1, jpon
            xo_ed (jo, jn) = clo_lon ( xo_ed (jo, jn), xo_ed (jo, jpoe) )
         END DO
      END DO

      xa_inf = MINVAL (xa_ed) ; xa_sup = MAXVAL (xa_ed)
      ya_inf = MINVAL (ya_ed) ; ya_sup = MAXVAL (ya_ed)
      xo_inf = MINVAL (xo_ed) ; xo_sup = MAXVAL (xo_ed)
      yo_inf = MINVAL (yo_ed) ; yo_sup = MAXVAL (yo_ed)

      IF ( yo_sup > 85.0_rl ) yo_sup = 90.0_rl

      WRITE (nout,*) 'Extrema atm : ', xa_inf, xa_sup, ya_inf, ya_sup
      WRITE (nout,*) 'Extrema oce : ', xo_inf, xo_sup, yo_inf, yo_sup

      WRITE (nout,*) 'Bord_oa'
      WRITE (nout,*) 'xae : ', MINVAL (xae), MAXVAL (xae)
      WRITE (nout,*) 'xaw : ', MINVAL (xaw), MAXVAL (xaw)
      WRITE (nout,*) 'yan : ', MINVAL (yan), MAXVAL (yan)
      WRITE (nout,*) 'yas : ', MINVAL (yas), MAXVAL (yas)

   END SUBROUTINE bord_oa
   
   SUBROUTINE bord_lmd5 ( xx_ed, yy_ed )
      !! 
      !! Find limits of each atm grid box
      !!
      !! For LMD 5
      !!
      !! C grid in Arakawa classification
      !! One uses u and v points to find grid box limits
      !!  La limite Est   (xae) est le point u ( i-1, j) 
      !!  La limite Ouest (xaw) est le point u ( i  , j) 
      !!  La limite Nord  (xan) est le point v ( i, j+1)
      !!  La limite Sud   (xas) est le point v ( i, j  )
      !!  Don't forget periodicity for u point 
      !!
      !!
      IMPLICIT NONE
      !!
      REAL (kind=rl), DIMENSION (:,:), INTENT (out) :: xx_ed, yy_ed
      INTEGER (kind=il) :: ja, jai, jaj, jn
      WRITE (nout,*) 'Debut bord_lmd5'
      !!
      DO ja = 1, jpan
         jai = MOD ( ja - 1, jpai) + 1 ; jaj = ( ja - 1) / jpai + 1   ! 2D index of atm grid box
         !!
         SELECT CASE ( jai) 
         CASE (   1_il)  ; xaw ( ja) = xalonu ( jpai  )  ; xae ( ja) = xalonu ( ja)
         CASE default ; xaw ( ja) = xalonu ( ja - 1)  ; xae ( ja) = xalonu ( ja)
         END SELECT
         IF ( xae ( ja) < xalont ( ja) ) xae ( ja) = xae ( ja) + 360.0
         IF ( xaw ( ja) > xalont ( ja) ) xaw ( ja) = xaw ( ja) - 360.0
         yas ( ja) = xalatv ( ja) ; yan ( ja) = xalatv ( ja + jpai)
      END DO
      
      xx_ed (:,1) = xaw    (:)
      xx_ed (:,2) = xalont (:)
      xx_ed (:,3) = xae    (:)
      xx_ed (:,4) = xae    (:)
      xx_ed (:,5) = xae    (:)
      xx_ed (:,6) = xalont (:)
      xx_ed (:,7) = xaw    (:)
      xx_ed (:,8) = xaw    (:)
      xx_ed (:,9) = xalont (:)
      !
      yy_ed (:,1) = yan    (:)
      yy_ed (:,2) = yan    (:)
      yy_ed (:,3) = yan    (:)
      yy_ed (:,4) = xalatt (:)
      yy_ed (:,5) = yas    (:)
      yy_ed (:,6) = yas    (:)
      yy_ed (:,7) = yas    (:)
      yy_ed (:,8) = xalatt (:)
      yy_ed (:,9) = xalatt (:)
  
      !
   END SUBROUTINE bord_lmd5
   !!
   SUBROUTINE bord_lmdz ( xx_ed, yy_ed )
      !!
      !! Calculer les bords d'une maille pour LMDZ pour une grille reguliere
      !!
      !! Tous les points sont au point physique. On se sert d'eux pour
      !! determiner les limites des mailles.
      !!
      IMPLICIT NONE
      !!
      REAL (kind=rl), DIMENSION (:,:), INTENT (out) :: xx_ed, yy_ed
      INTEGER (kind=il) :: ja, jai, jaj, jn
      !
      xx_ed (:,1) = xaw    (:)
      xx_ed (:,2) = xalont (:)
      xx_ed (:,3) = xae    (:)
      xx_ed (:,4) = xae    (:)
      xx_ed (:,5) = xae    (:)
      xx_ed (:,6) = xalont (:)
      xx_ed (:,7) = xaw    (:)
      xx_ed (:,8) = xaw    (:)
      xx_ed (:,9) = xalont (:)
      !
      yy_ed (:,1) = yan    (:)
      yy_ed (:,2) = yan    (:)
      yy_ed (:,3) = yan    (:)
      yy_ed (:,4) = xalatt (:)
      yy_ed (:,5) = yas    (:)
      yy_ed (:,6) = yas    (:)
      yy_ed (:,7) = yas    (:)
      yy_ed (:,8) = xalatt (:)
      yy_ed (:,9) = xalatt (:)

      !! In longitude, takes the closest lon, refering to the center
      DO jai = 1, jpan
         DO jn = 1, 8 
            xx_ed(jai,jn) = clo_lon ( xx_ed(jai,jn), xx_ed(jai,9) )
         END DO
      END DO
      
      ! remet les points 'au milieu' pour avoir une projection plus precise
!-$$      xx_ed(:,9) = 0.5 * ( MINVAL(xx_ed(:,1:8),DIM=2) + MAXVAL(xx_ed(:,1:8),DIM=2) )
!-$$      yy_ed(:,9) = 0.5 * ( MINVAL(yy_ed(:,1:8),DIM=2) + MAXVAL(yy_ed(:,1:8),DIM=2) )
!-$$
       xx_ed(:,9) = 0.5 * ( xae (:) + xaw (:) )
       yy_ed(:,9) = 0.5 * ( yas (:) + yan (:) )

       WRITE (nout,*) 'Bord LMDZ'
       WRITE (nout,*) 'xae : ', MINVAL (xae), MAXVAL (xae)
       WRITE (nout,*) 'xaw : ', MINVAL (xaw), MAXVAL (xaw)
       WRITE (nout,*) 'yan : ', MINVAL (yan), MAXVAL (yan)
       WRITE (nout,*) 'yas : ', MINVAL (yas), MAXVAL (yas)

      !
      RETURN
   END SUBROUTINE bord_lmdz
   !
   SUBROUTINE bord_gaussian ( xx_ed, yy_ed )
      !!
      !! Calcule les bords d'une maille Gaussian
      !! Grille Gaussienne (repris tel que de lmdz)
      !!
      IMPLICIT NONE
      !!
      REAL (kind=rl), DIMENSION (:,:) :: xx_ed, yy_ed
      INTEGER (kind=il) :: ja, jai, jaj
      REAL (kind=rl) :: xdel, ydel, yinf, ysup
      !!
      WRITE (nout,*) 'Debut bord_gaussian'
      !
      xinf =  1.0e+20 ; xsup = -1.0e+20
      !
      DO ja = 1, jpan
         jai = m2ai ( ja) ; jaj = m2aj ( ja)
         IF ( jai == 1 ) THEN
            xdel = ( xalont ( ja + 1) - xalont ( ja)) * 0.5
            xaw ( ja) = xalont ( ja) - xdel ; xae ( ja) = xalont ( ja) + xdel
         ELSE IF ( jai == jpai ) THEN
            xdel = ( xalont ( ja) - xalont ( ja-1))/2.0
            xaw ( ja) = xalont ( ja) - xdel ; xae ( ja) = xalont ( ja) + xdel
         ELSE
            xaw ( ja) = ( xalont ( ja) + xalont ( ja - 1)) * 0.5
            xae ( ja) = ( xalont ( ja) + xalont ( ja + 1)) * 0.5
         ENDIF
         IF ( xae ( ja) < xaw ( ja) ) xae ( ja) = xae ( ja) + 360.0
         xinf = min ( xinf, xaw ( ja), xae ( ja))
         xsup = max ( xsup, xaw ( ja), xae ( ja))
      ENDDO
      !!
      PRINT*, "XINF=", xinf
      PRINT*, "XSUP=", xsup
      !!
      !   xaw (     1          : jpai) = xinf 
      !   xae (     1          : jpai) = xsup
      !   xaw ( jpan - jpai + 1: jpan) = xinf
      !   xae ( jpan - jpai + 1: jpan) = xsup
      !
      yinf = 1.0e+20
      ysup = -1.0e+20
      DO ja = 1, jpan
         jai = m2ai ( ja) ; jaj = m2aj ( ja)
         IF ( jaj == 1) THEN
            ydel = ( xalatt ( ja + jpai) - xalatt ( ja)) / 2.0
            yas ( ja) = xalatt ( ja) - ydel
            yan ( ja) = xalatt ( ja) + ydel
         ELSE IF ( jaj == jpaj) THEN
            ydel = ( xalatt ( ja) - xalatt ( ja - jpai))/2.0
            yas ( ja) = xalatt ( ja) - ydel
            yan ( ja) = xalatt ( ja) + ydel
         ELSE
            yas ( ja) = ( xalatt ( ja) + xalatt ( ja - jpai)) / 2.0
            yan ( ja) = ( xalatt ( ja) + xalatt ( ja + jpai)) / 2.0
         ENDIF
         !     IF ( yas ( ja) < yinf) yinf = yas ( ja)

         yinf = min ( yinf, yas ( ja), yan ( ja))
         ysup = max ( ysup, yas ( ja), yan ( ja))
      ENDDO
      PRINT*, "YINF=", xinf
      PRINT*, "YSUP=", xsup
      !!
      RETURN
   END SUBROUTINE bord_gaussian
   !!
   SUBROUTINE bord_grisli ( xx_ed, yy_ed ) 
      !!
      IMPLICIT NONE
      REAL (kind=rl), DIMENSION (:,:), INTENT (inout) :: xx_ed, yy_ed
      INTEGER (kind=il) :: jn, jo, ji, jj, jpe
      !!
      WRITE (nout,*) 'Debut bord_grisli'
      !! Corner are directly read in lecdata
      jpe = SIZE ( xx_ed, DIM=2 )
      !!
   END SUBROUTINE bord_grisli
   !!
   SUBROUTINE bord_opa ( xx_ed, yy_ed )
      !!
      IMPLICIT NONE
      REAL (kind=rl), DIMENSION (:,:), INTENT (out) :: xx_ed, yy_ed
      INTEGER (kind=il) :: jn, jo, ji, jj
      REAL :: ylim_south
      !!
      WRITE (nout,*) 'Debut bord_opa'
      !
      ylim_south = MIN ( MINVAL(xolatt), MINVAL(xolatu), MINVAL(xolatv), MINVAL(xolatf))
      !!
      !! Set edges (upper-left, upper-middle, upper-right, middle-right,lower-right,middle-lower,lower-left,middle-left) of each grid box
      !! Highly depending of the model
      DO jj = 2, jpoj
         DO ji = 2, jpoi
            xx_ed (m1o(ji,jj),1) = xolonf ( m1o(ji-1,jj  ))
            xx_ed (m1o(ji,jj),2) = xolonv ( m1o(ji  ,jj  ))
            xx_ed (m1o(ji,jj),3) = xolonf ( m1o(ji  ,jj  ))
            xx_ed (m1o(ji,jj),4) = xolonu ( m1o(ji  ,jj  ))
            xx_ed (m1o(ji,jj),5) = xolonf ( m1o(ji  ,jj-1))
            xx_ed (m1o(ji,jj),6) = xolonv ( m1o(ji  ,jj-1))
            xx_ed (m1o(ji,jj),7) = xolonf ( m1o(ji-1,jj-1))
            xx_ed (m1o(ji,jj),8) = xolonu ( m1o(ji-1,jj  ))
            xx_ed (m1o(ji,jj),9) = xolont ( m1o(ji  ,jj  ))
            !!
            yy_ed (m1o(ji,jj),1) = xolatf ( m1o(ji-1,jj  ))
            yy_ed (m1o(ji,jj),2) = xolatv ( m1o(ji  ,jj  ))
            yy_ed (m1o(ji,jj),3) = xolatf ( m1o(ji  ,jj  ))
            yy_ed (m1o(ji,jj),4) = xolatu ( m1o(ji  ,jj  ))
            yy_ed (m1o(ji,jj),5) = xolatf ( m1o(ji  ,jj-1))
            yy_ed (m1o(ji,jj),6) = xolatv ( m1o(ji  ,jj-1))
            yy_ed (m1o(ji,jj),7) = xolatf ( m1o(ji-1,jj-1))
            yy_ed (m1o(ji,jj),8) = xolatu ( m1o(ji-1,jj  ))
            yy_ed (m1o(ji,jj),9) = xolatt ( m1o(ji  ,jj  ))
         END DO
      END DO
      !! Boundary South
      DO jj = 1_il, 1_il
         DO ji = 2, jpoi
            xx_ed (m1o(ji, 1_il),1) = xolonf ( m1o(ji-1, 1_il  ))
            xx_ed (m1o(ji, 1_il),2) = xolonv ( m1o(ji  , 1_il  ))
            xx_ed (m1o(ji, 1_il),3) = xolonf ( m1o(ji  , 1_il  ))
            xx_ed (m1o(ji, 1_il),4) = xolonu ( m1o(ji  , 1_il  ))
            xx_ed (m1o(ji, 1_il),5) = xolonf ( m1o(ji  , 1_il  ))
            xx_ed (m1o(ji, 1_il),6) = xolonv ( m1o(ji  , 1_il  ))
            xx_ed (m1o(ji, 1_il),7) = xolonf ( m1o(ji-1, 1_il  ))
            xx_ed (m1o(ji, 1_il),8) = xolonu ( m1o(ji-1, 1_il  ))
            xx_ed (m1o(ji, 1_il),9) = xolont ( m1o(ji  , 1_il  ))
            !!
            yy_ed (m1o(ji, 1_il),1) = xolatf ( m1o(ji-1, 1_il  ))
            yy_ed (m1o(ji, 1_il),2) = xolatv ( m1o(ji  , 1_il  ))
            yy_ed (m1o(ji, 1_il),3) = xolatf ( m1o(ji  , 1_il  ))
            yy_ed (m1o(ji, 1_il),4) = xolatu ( m1o(ji  , 1_il  ))
            yy_ed (m1o(ji, 1_il),5) = ylim_south
            yy_ed (m1o(ji, 1_il),6) = ylim_south
            yy_ed (m1o(ji, 1_il),7) = ylim_south
            yy_ed (m1o(ji, 1_il),8) = xolatu ( m1o(ji-1, 1_il  ))
            yy_ed (m1o(ji, 1_il),9) = xolatt ( m1o(ji  , 1_il  ))
         END DO
      END DO
      !!
      !! Boundaries (west) 

      DO jj = 2, jpoj
         DO ji = 1, 1
            xx_ed (m1o( 1_il,jj),1) = xolonf ( m1o(jpoi- 1,jj  ))
            xx_ed (m1o( 1_il,jj),2) = xolonv ( m1o( 1_il  ,jj  ))
            xx_ed (m1o( 1_il,jj),3) = xolonf ( m1o( 1_il  ,jj  ))
            xx_ed (m1o( 1_il,jj),4) = xolonu ( m1o( 1_il  ,jj  ))
            xx_ed (m1o( 1_il,jj),5) = xolonf ( m1o( 1_il  ,jj-1))
            xx_ed (m1o( 1_il,jj),6) = xolonv ( m1o( 1_il  ,jj-1))
            xx_ed (m1o( 1_il,jj),7) = xolonf ( m1o(jpoi- 1,jj-1))
            xx_ed (m1o( 1_il,jj),8) = xolonu ( m1o(jpoi- 1,jj  ))
            xx_ed (m1o( 1_il,jj),9) = xolont ( m1o( 1_il  ,jj  ))
            !!
            yy_ed (m1o( 1_il,jj),1) = xolatf ( m1o(jpoi- 1,jj  ))
            yy_ed (m1o( 1_il,jj),2) = xolatv ( m1o( 1_il  ,jj  ))
            yy_ed (m1o( 1_il,jj),3) = xolatf ( m1o( 1_il  ,jj  ))
            yy_ed (m1o( 1_il,jj),4) = xolatu ( m1o( 1_il  ,jj  ))
            yy_ed (m1o( 1_il,jj),5) = xolatf ( m1o( 1_il  ,jj-1))
            yy_ed (m1o( 1_il,jj),6) = xolatv ( m1o( 1_il  ,jj-1))
            yy_ed (m1o( 1_il,jj),7) = xolatf ( m1o(jpoi- 1,jj-1))
            yy_ed (m1o( 1_il,jj),8) = xolatu ( m1o(jpoi- 1,jj  ))
            yy_ed (m1o( 1_il,jj),9) = xolatt ( m1o( 1_il  ,jj  ))
         END DO
      END DO
      DO jj = 1, 1
         DO ji = 1, 1
            xx_ed (m1o( 1_il, 1_il),1) = xolonf ( m1o(jpoi- 1, 1_il ))
            xx_ed (m1o( 1_il, 1_il),2) = xolonv ( m1o( 1_il  , 1_il ))
            xx_ed (m1o( 1_il, 1_il),3) = xolonf ( m1o( 1_il  , 1_il ))
            xx_ed (m1o( 1_il, 1_il),4) = xolonu ( m1o( 1_il  , 1_il ))
            xx_ed (m1o( 1_il, 1_il),5) = xolonf ( m1o( 1_il  , 1_il ))
            xx_ed (m1o( 1_il, 1_il),6) = xolonv ( m1o( 1_il  , 1_il ))
            xx_ed (m1o( 1_il, 1_il),7) = xolonf ( m1o(jpoi- 1, 1_il ))
            xx_ed (m1o( 1_il, 1_il),8) = xolonu ( m1o(jpoi- 1, 1_il ))
            xx_ed (m1o( 1_il, 1_il),9) = xolont ( m1o( 1_il  , 1_il ))
            !!
            yy_ed (m1o( 1_il, 1_il),1) = xolatf ( m1o(jpoi- 1, 1_il ))
            yy_ed (m1o( 1_il, 1_il),2) = xolatv ( m1o( 1_il  , 1_il ))
            yy_ed (m1o( 1_il, 1_il),3) = xolatf ( m1o( 1_il  , 1_il ))
            yy_ed (m1o( 1_il, 1_il),4) = xolatu ( m1o( 1_il  , 1_il ))
            yy_ed (m1o( 1_il, 1_il),5) = xolatf ( m1o( 1_il  , 1_il ))
            yy_ed (m1o( 1_il, 1_il),6) = xolatv ( m1o( 1_il  , 1_il ))
            yy_ed (m1o( 1_il, 1_il),7) = xolatf ( m1o(jpoi- 1, 1_il ))
            yy_ed (m1o( 1_il, 1_il),8) = xolatu ( m1o(jpoi- 1, 1_il ))
            yy_ed (m1o( 1_il, 1_il),9) = xolatt ( m1o( 1_il  , 1_il ))
         END DO
      END DO
      !!
      !! All ocean longitudes are set within the limits of the atmospheric grid
      !!
      xinf = MINVAL (xx_ed) ; xsup = MAXVAL (xx_ed)
      !!
      DO jn = 1, 3
         WHERE ( xx_ed (:,9) <  xinf ) xx_ed (:,9) = xx_ed (:,9) + 360.0_rl
         WHERE ( xx_ed (:,9) >  xsup ) xx_ed (:,9) = xx_ed (:,9) - 360.0_rl
      END DO
      !!
      !! In longitude, takes the closest lon, refering to the center of the box
      DO jo = 1, jpon
         DO jn = 1, 8 
            xx_ed ( jo, jn) = clo_lon ( xx_ed ( jo, jn), xx_ed ( jo, 9) )
         END DO
      END DO
      !!
      xow (:) = MINVAL (xx_ed(:,:), DIM=2)
      xoe (:) = MAXVAL (xx_ed(:,:), DIM=2)
      yos (:) = MINVAL (yy_ed(:,:), DIM=2)
      yon (:) = MAXVAL (yy_ed(:,:), DIM=2)
      !!
   END SUBROUTINE bord_opa
END MODULE bords
