! -*- Mode: f90 -*-
PROGRAM icestream
   USE declare
   USE modeles
   USE mod_lecdata
   USE formula
   USE math
   USE mod_rectifie
   USE mod_lbc
   USE bords
   USE mod_prih
   USE mod_wri_wei
   USE fliocom
   USE errioipsl
   USE getincom
   USE mod_inipar
   USE mod_inter
!!!
   IMPLICIT NONE
!!!
!!!   A partir d'un fichier de poids, complete avec les poids pour l'eau qui
!!!   vient des calottes

   REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: ynorth, ysouth
   REAL (kind=rl), PARAMETER :: zm_frac = 0.0_rl ! Fraction criterium
   !!
   INTEGER (kind=il) :: ja, jo, jn, joi, joj, jai, jaj, jr, kn ! indices de boucle
   INTEGER (kind=il), DIMENSION (:, :), ALLOCATABLE :: imask ! Masque
   REAL (kind=rl)    :: zosrf, zasrf ! Surface totale
   REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: zasrf_lat ! Surface atmosphere par bande de latitude
   INTEGER (kind=il), DIMENSION (:), ALLOCATABLE :: ktemp
   REAL(kind=rl), DIMENSION (:), ALLOCATABLE :: wtemp
   REAL(kind=rl), DIMENSION (:, :), ALLOCATABLE  :: z2o
   CHARACTER (len=80) :: cfname4, cfname8
   INTEGER (kind=il), DIMENSION (:,:), ALLOCATABLE  :: i2a
   INTEGER (kind=il), DIMENSION (:,:), ALLOCATABLE  :: i2o
   CHARACTER (len=50) :: cldiag_a2o, clw_a2o, clw_a2o_cdf, clw_a2o_mct
   INTEGER (kind=il) :: ierr
   LOGICAL :: l_no_pac
   !!
   INTEGER (kind=il), DIMENSION (2) :: jind
   INTEGER (kind=i_4) :: il_ncid
!   CHARACTER (len=40) :: cl_atl, cl_pac, cl_noclo, cl_nomed, c_basins
   REAL(kind=rl), DIMENSION (:), ALLOCATABLE :: tmask_atl, tmask_pac, tmask_noclose, tmask_nomed
   !!
   CHARACTER (len = 25) :: mo_name, a2o_name
   CHARACTER (len =  1) :: c_i, c_r
   CHARACTER (len=80) :: clarg
   INTEGER :: narg
   !!
   !! Lecture de la ligne de comande
   narg = iargc()
   IF ( narg > 0 ) THEN
      CALL getarg ( 1, clarg)
      SELECT CASE (TRIM(clarg))
      CASE Default
         IF ( INDEX ( TRIM(clarg), '.def' ) /= 0 ) THEN
            WRITE (nout,*) 'Lecture des parametres dans ', TRIM(clarg)
            CALL getin_name ( TRIM (clarg) )
         ELSE
            WRITE (nout,*) 'Lecture des parametres dans run.def'
         ENDIF
      END SELECT
   END IF
   !!
   CALL inipar
   CALL alloc_modeles
   !!
   CALL ipsldbg (l_ipsldbg)
   !!
   !! Initialisation
   !!

   ALLOCATE (imask (jpon, jp_calv), STAT=ierr)  ; CALL chk_allo (ierr, 'imask', lreset=.TRUE., crout='icestream')
   ALLOCATE (zasrf_lat      (jpaj), STAT=ierr)  ; CALL chk_allo (ierr, 'zasrf_lat')
   ALLOCATE (wtemp         (jpa2o), STAT=ierr)  ; CALL chk_allo (ierr, 'wtemp')
   ALLOCATE (ktemp         (jpa2o), STAT=ierr)  ; CALL chk_allo (ierr, 'ktemp')
   ALLOCATE (z2o      (jpoi, jpoj), STAT=ierr)  ; CALL chk_allo (ierr, 'z2o')
   ALLOCATE (i2a      (jpai, jpaj), STAT=ierr)  ; CALL chk_allo (ierr, 'i2a')
   ALLOCATE (i2o      (jpoi, jpoj), STAT=ierr)  ; CALL chk_allo (ierr, 'i2o')
   ALLOCATE (tmask_atl      (jpon), STAT=ierr)  ; CALL chk_allo (ierr, 'tmask_atl')
   ALLOCATE (tmask_pac      (jpon), STAT=ierr)  ; CALL chk_allo (ierr, 'tmask_pac')
   ALLOCATE (tmask_noclose  (jpon), STAT=ierr)  ; CALL chk_allo (ierr, 'tmask_noclose')
   ALLOCATE (tmask_nomed    (jpon), STAT=ierr)  ; CALL chk_allo (ierr, 'tmasl_nomed')
   !!
   ALLOCATE (ynorth (1:jp_calv), STAT=ierr) ; CALL chk_allo (ierr, 'ynorth')
   ALLOCATE (ysouth (1:jp_calv), STAT=ierr) ; CALL chk_allo (ierr, 'ynorth')
   !!
   ysouth (1:jp_calv) = ylimits (1:jp_calv)
   ynorth (1:jp_calv) = ylimits (2:jp_calv+1)
   !!
   !! Initialisation
   ngrd = 11 ; nsrf = 12 ; nmsk = 13 ; nchk = 14 ; ndeb = 9 ; nbug1 = 10
   nwei4o2a = 21 ; nwei8o2a = 22 ; nwei4a2o = 23 ; nwei8a2o = 24
   !
   nout = 0
   IF (nout /= 6 .AND. nout /= 0 ) THEN
      OPEN (unit = nout, file = 'icestream.out', action = 'write', status = 'unknown', position = 'rewind' )
   END IF
   !
   jma2o = 0_il ; jmo2a = 0_il ; nvo = 0_il
   !!
   !! Diagnostics files
   !!
!$$$  OPEN (unit = nout, file = 'icestreams.out', position = 'REWIND', action = 'write', status = 'replace' )
!$$$  !!
   !! Read coordinates of all models, and weights
   !!
   CALL lecdata (lecweights = .FALSE., leco2amask = .TRUE. )
   !!
   wa2o = zero ; ka2o = 0_il
   !
   CALL ipsldbg (l_ipsldbg)
   !
   CALL lbc (xolont, noperio, 'T', jpoi, jpoj)
   CALL lbc (xolonu, noperio, 'U', jpoi, jpoj)
   CALL lbc (xolonv, noperio, 'V', jpoi, jpoj)
   CALL lbc (xolonf, noperio, 'F', jpoi, jpoj)
   CALL lbc (xolatt, noperio, 'T', jpoi, jpoj)
   CALL lbc (xolatu, noperio, 'U', jpoi, jpoj)
   CALL lbc (xolatv, noperio, 'V', jpoi, jpoj)
   CALL lbc (xolatf, noperio, 'F', jpoi, jpoj)
   CALL lbc (xosrft, noperio, 'T', jpoi, jpoj)
   CALL lbc (iomskt, noperio, 'T', jpoi, jpoj)
   !
   CALL lbc (xalont, naperio, 'T', jpai, jpaj)
   CALL lbc (xalonu, naperio, 'U', jpai, jpaj)
   CALL lbc (xalonv, naperio, 'V', jpai, jpaj)
   CALL lbc (xalatt, naperio, 'T', jpai, jpaj)
   CALL lbc (xalatu, naperio, 'U', jpai, jpaj)
   CALL lbc (xalatv, naperio, 'V', jpai, jpaj)
   CALL lbc (xasrft, naperio, 'T', jpai, jpaj)
   CALL lbc (iamskt, naperio, 'T', jpai, jpaj)
   !!
   CALL rectifie
   !!
   !! Calcul de la surface par bande de latitude atmosphere
   DO jaj = 1_il, jpaj
      zasrf_lat(jaj) = 0.0_rl
      DO jai = 1_il, jpai
         ja = m1a(1_il, jaj)
         zasrf_lat(jaj) = zasrf_lat(jaj) + xasrft(ja)
      END DO
   ENDDO
   DO jaj = 1, jpaj
      WRITE(UNIT=nout,FMT='(1F7.1,1PE15.3)' ) xalatt ( m1a (1_il, jaj)), zasrf_lat (jaj)
   END DO
   WRITE(nout,*) SUM (zasrf_lat)
   !!
   CALL bord_a
   !!
   mo_name  = "mozaic" ; a2o_name = "a2o"
   !!
   CALL bord_o
   CALL bord_oa
   !!
   WRITE (nout,*) 'o2amask ', COUNT (o2amask > 0 ), COUNT (o2amask <=  0 )
   i2a = 0_il
   WHERE (RESHAPE(o2amask,(/jpai,jpaj/)) .GT. 0.0_rl ) i2a = 1_il
   !CALL prihin (i2a , 1_il, nout )
   !!
   !! Lecture masques de bassin
   !IF ( TRIM (cotyp) == 'orca2' .OR. TRIM (cotyp) == 'orca2.0' .OR. TRIM (cotyp) == 'orca2.1' .OR. TRIM (cotyp) == 'orca4' ) THEN
   !!IF ( l_calving_noatl .OR. l_calving_nopac ) THEN
!-$$   c_basins = 'eORCA1.2.nc'
!-$$   cl_atl   = 'mask_atl'
!-$$   cl_pac   = 'mask_pac'
!-$$   cl_nomed = 'mask_nomed'
!-$$   cl_noclo = 'mask_noclose'

   CALL flioopfd (TRIM (c_basins), il_ncid )
   CALL fliogetv (il_ncid, TRIM (cl_atl)  , z2o ) ;  tmask_atl     = RESHAPE (z2o, (/ jpon /) )
   CALL fliogetv (il_ncid, TRIM (cl_pac)  , z2o ) ;  tmask_pac     = RESHAPE (z2o, (/ jpon /) )
   CALL fliogetv (il_ncid, TRIM (cl_noclo), z2o ) ;  tmask_noclose = RESHAPE (z2o, (/ jpon /) )
   CALL fliogetv (il_ncid, TRIM (cl_nomed), z2o ) ;  tmask_nomed   = RESHAPE (z2o, (/ jpon /) )
   CALL flioclo (il_ncid)


!ELSE
!      tmask_atl     = REAL (1-iomskt, rl)
!      tmask_noclose = REAL (1-iomskt, rl)
!      tmask_nomed   = REAL (1-iomskt, rl)
   !   END IF
   WRITE (unit=nout, fmt=*) 'iomskt'
   CALL prihin (RESHAPE(iomskt             ,(/jpoi,jpoj/)), 1_il, nout)
   WRITE (unit=nout, fmt=*) 'tmask_atl'
   CALL prihin (RESHAPE(NINT(tmask_atl)    ,(/jpoi,jpoj/)), 1_il, nout)
   WRITE (unit=nout, fmt=*) 'tmask_pac'
   CALL prihin (RESHAPE(NINT(tmask_pac)    ,(/jpoi,jpoj/)), 1_il, nout)
   WRITE (unit=nout, fmt=*) 'tmask_nomed'
   CALL prihin (RESHAPE(NINT(tmask_nomed)  ,(/jpoi,jpoj/)), 1_il, nout)
   WRITE (unit=nout, fmt=*) 'tmask_noclose'
   CALL prihin (RESHAPE(NINT(tmask_noclose),(/jpoi,jpoj/)), 1_il, nout)

   !
   Bassin: DO jr = 1_il, jp_calv !! Boucle sur les zones
      !!
      !! Creation du masque
      !!
      imask (:,jr) = 1_il
      !!
      ! Limite nord incluse, limite sud exclue. Mers fermees exclues
      IF (l_calving_nomed ) THEN
         WHERE (tmask_nomed  (:) > 0.5_r_4 .AND. (xolatt > ysouth (jr) .AND. xolatt <= ynorth (jr)) ) imask(:,jr) = iomskt
      ELSE
         WHERE (tmask_noclose(:) > 0.5_r_4 .AND. (xolatt > ysouth (jr) .AND. xolatt <= ynorth (jr)) ) imask(:,jr) = iomskt
      END IF
      !!
      IF ( jr > 1 ) THEN 
         IF (l_calving_noatl) THEN
            WHERE (tmask_atl (:) >  0.5_rl)  imask (:,jr) = 1_il
            ! Mise a zero sur l'Arctique
            WHERE (xolatt (:)    > 65.0_rl)  imask (:,jr) = 1_il
         END IF
         IF (l_calving_nopac) THEN
            WHERE (tmask_pac (:) >  0.5_rl)  imask (:,jr) = 1_il
         ENDIF
      END IF
      !!
      WRITE(nout,*) 'Bassin ', jr
      CALL prihin (RESHAPE ( imask(:,jr), (/jpoi, jpoj/)), 1_il, nout)
      !!
      !! Calcul de la surface totale de la zone ocean
      zosrf = DOT_PRODUCT (xosrft, REAL(1-imask(:,jr),KIND=rl)*REAL(1-iomskp,KIND=rl) )
      WRITE(nout,*) 'Surface de la zone ', jr, zosrf
      !!
      !! Calcul de la surface totale de la zone atmosphere
      zasrf = 0.0_rl
      DO ja = 1, jpan
         IF ( xalatt(ja) > ysouth(jr) .AND. xalatt(ja) <= ynorth(jr)) zasrf = zasrf + xasrft(ja)
      END DO
      !!
      !! Calcul des poids
      WRITE (nout,*) 'Calcul des poids'
      DO jo = 1, jpon ! Boucle sur tous les points ocean
         IF (imask(jo,jr) == 0_il ) THEN ! Vrai si le point NEMO est un point ocean dans la bande de latitude
            DO jaj = 1_il, jpaj ! Boucle sur les latitudes atmosphere
               ja = m1a(1_il, jaj) ! Indice 1D du point le plus à l'ouest
               IF ( xalatt(ja) > ysouth(jr) .AND. xalatt(ja) <= ynorth(jr)) THEN ! Test si le point atmosphere est dans la bande de latitude
                  nvo (jo) = nvo (jo) + 1 ! Increment du nombre de voisins
                  IF (nvo(jo) .GT. jpa2o ) THEN ! Test sur le nombre maximum de voisin
                     WRITE (nout,*) 'Nombre de voisins insuffisant ', jpa2o
                     WRITE (nout,*) jr, jo, ja, xalatt(ja), ysouth(jr), ynorth(jr)
                     STOP
                  END IF
                  ka2o (nvo(jo), jo) = ja
                  wa2o (nvo(jo), jo) = zasrf_lat(jaj) / zosrf ! Division par la surface totale de la zone océanique
               ENDIF
            ENDDO
         END IF
      END DO
      !!
      jma2o = MAXVAL (nvo)
      !!
      WRITE (nout,*) 'Fin bassin ' , jr
   END DO Bassin
   !!
   WRITE(nout, *) 'jma2o : ', jma2o
   !!
   ! Tests bassins
   WRITE(nout,*) 'Somme des bassins '
   i2o =  RESHAPE (SUM(1-imask,DIM=2)*(1-iomskp), (/jpoi, jpoj/) )
   CALL prihin (i2o, 1_il, nout)
   !
   WRITE(nout,*) 'Somme des bassins '
   i2o =  RESHAPE ((1-iomskt)-SUM(1-imask,DIM=2)*(1-iomskp), (/jpoi, jpoj/) )
   CALL prihin (i2o, 1_il, nout)
   !
   WRITE(nout,*) 'Somme des bassins '
   i2o =  RESHAPE (SUM(imask,DIM=2), (/jpoi, jpoj/))
   CALL prihin (i2o, 1_il, nout)
   !!
   !! Normalisation
   !!
!$$$      WRITE (nout, *) 'wa2o ', MINVAL (wa2o), MAXVAL (wa2o), COUNT (wa2o > 0.0_rl)
!$$$      WRITE (nout, *) 'ka2o ', MINVAL (ka2o), MAXVAL (ka2o), COUNT (ka2o > 0 )
!$$$      WRITE (nout, *) 'xasrft ', MINVAL (xasrft), MAXVAL (xasrft)
!$$$      WRITE (nout, *) 'o2amask ', MINVAL (o2amask), MAXVAL (o2amask), SUM (o2amask)
!$$$      WRITE (nout, *) 'xosrft ', MINVAL (xosrft), MAXVAL (xosrft)
!$$$      WRITE (nout, *) 'iomskt ', MINVAL (iomskt), MAXVAL (iomskt), COUNT (iomskt == 0)
!$$$      WRITE (nout, *) 'iomskp ', MINVAL (iomskp), MAXVAL (iomskp), COUNT (iomskp == 0)
!$$$      WRITE (nout, *) 'iamskt ', MINVAL (iamskt), MAXVAL (iamskt), COUNT (iamskt == 0)
!$$$      WRITE (nout, *) 'locot ', COUNT (locot)
   !!
   CALL gather_wei
   !!
   !! Checking weights
   WRITE (unit = nout, fmt = '("Neighbors a2o     : ", 3I9  )' ) MINVAL (nvo, nvo /=0), MAXVAL (nvo), jma2o
   WRITE (unit = nout, fmt = '("Index a2o         : ", 2I9  )' ) MINVAL (ka2o, ka2o /= 0), MAXVAL (ka2o)
   jind = MINLOC(wa2o, wa2o /= 0.0_rl) ; jo = jind(2) ; jn = jind(1); joi = m2oi(jo) ; joj = m2oj(jo)
   WRITE (unit = nout, fmt = '("Weights a2o mini : ", 4I6, 1E13.4, 3I6)' ) &
      & jo, jn, joi, joj, wa2o(jn,jo), ka2o(jn,jo), m2ai(ka2o(jn,jo)), m2aj(ka2o(jn,jo))
   jind = MAXLOC(wa2o, wa2o /= 0.0_rl) ; jo = jind(2) ; jn = jind(1); joi = m2oi(jo) ; joj = m2oj(jo)
   WRITE (unit = nout, fmt = '("Weights a2o maxi : ", 4I6, 1E13.4, 3I6)' ) &
      & jo, jn, joi, joj, wa2o(jn,jo), ka2o(jn,jo), m2ai(ka2o(jn,jo)), m2aj(ka2o(jn,jo))
   !
   jind(1:1) = INT(MAXLOC (nvo),il) ; jo = jind(1)
   WRITE (unit = nout, fmt = *) jo, m2oi(jo), m2oj(jo), ka2o(1:nvo(jo),jo), &
      & (m2ai(ka2o(jn,jo)), m2aj(ka2o(jn,jo)), jn = 1, nvo(jo))
   !!
   ! Calcul du masque
   CALL inter_a2o (REAL (1-iamskt, KIND = rl), a2omask)
   !!
!-$$   za = 1.0_rl
!-$$   CALL inter_a2o (za, zo)
!-$$   !
!-$$   zsuma = DOT_PRODUCT (za, xasrft*o2amask)
   
   !!
   !! Verif nombre de voisins
   !!
   IF ( jma2oi /= jma2o ) THEN 
      WRITE (nout, *) 'Erreur nombre de voisins dans run.def : '
      WRITE (nout, *) ' jma2o calcule : ', jma2o
      WRITE (nout, *) ' jma2o lu      : ', jma2oi
      STOP
   ENDIF
   !!
   !! Ecriture des poids
   !!
   WRITE (unit = c_r, fmt = '(1i1)' ) rk_out

   IF (l_wei_i4) THEN 
      WRITE (unit = c_i, fmt = '(1i1)' ) i_4
      cfname4 = TRIM(mo_name) // ".wa2o.calvin" // TRIM(c_suffix) // ".i" // TRIM(c_i) // ".r" // TRIM(c_r)
      IF ( l_calving_noatl ) cfname4 = TRIM (cfname4) // '_noatl'
      IF ( l_calving_nopac ) cfname4 = TRIM (cfname4) // '_nopac'
      OPEN (unit = nwei4a2o, file = TRIM(cfname4), form = 'unformatted', status = 'replace', action = 'write')
      WRITE (nout, *) 'Poids i_4 a -> o : ', TRIM(cfname4)
   END IF

   IF (l_wei_i8) THEN
      WRITE (unit = c_i, fmt = '(1i1)' ) i_8
      cfname8 = TRIM(mo_name) // ".wa2o.calvin" // TRIM(c_suffix) // ".i" // TRIM(c_i) // ".r" // TRIM(c_r)
      IF ( l_calving_noatl ) cfname8 = TRIM (cfname8) // '_noatl'
      IF ( l_calving_nopac ) cfname8 = TRIM (cfname8) // '_nopac'
      OPEN (unit = nwei8a2o, file = TRIM(cfname8), form = 'unformatted', status = 'replace', action = 'write')
      WRITE (nout, *) 'Poids i_8 a -> o : ', TRIM(cfname8)
   END IF
   
   cldiag_a2o  = TRIM(a2o_name) // '.calvin.diag'
   clw_a2o     = 'mozaic.wa2o.calvin'
   clw_a2o_mct = "rmp_"//TRIM(camod_t)//"_to_"//TRIM(comod_t)//"_MOSAIC_calvin"

   IF ( l_calving_noatl ) THEN
      cldiag_a2o  = TRIM (cldiag_a2o  ) // '_noatl'
      clw_a2o     = TRIM (   clw_a2o  ) // '_noatl'
      clw_a2o_mct = TRIM (clw_a2o_mct ) // '_noatl'
   ENDIF
   IF ( l_calving_nopac ) THEN
      cldiag_a2o  = TRIM (cldiag_a2o  ) // '_nopac'
      clw_a2o     = TRIM (   clw_a2o  ) // '_nopac'
      clw_a2o_mct = TRIM (clw_a2o_mct ) // '_nopac'
   ENDIF
   cldiag_a2o  = TRIM ( cldiag_a2o)
   clw_a2o     = TRIM ( clw_a2o   )
   clw_a2o_mct = TRIM (clw_a2o_mct)

   
   WRITE (nout, *) 'Calling wri_weights_a2o ', cldiag_a2o, clw_a2o, "5"
   CALL wri_weights_a2o (cldiag_a2o, clw_a2o, clw_a2o_mct, "5" )         

   IF (l_wei_i4) CLOSE (unit = nwei8a2o)
   IF (l_wei_i8) CLOSE (unit = nwei4a2o)
   !
   STOP
   !!
CONTAINS
   !!
   SUBROUTINE gather_wei
      !!
      WRITE (nout,*) 'Debut gather wei : perio'
      jma2o = MAXVAL (nvo)
      !! Periodicity
      DO jn = 1, jma2o
         CALL lbc (wa2o(jn,:) , noperio, 'T', jpoi, jpoj)
         CALL lbc (ka2o(jn,:) , noperio, 'T', jpoi, jpoj)
      END DO
      !!
      !! Gather weights
      WRITE (nout,*) 'gather wei : gather'
      DO jo = 1, jpon
         ktemp = ka2o(:,jo) ; wtemp = wa2o(:,jo)
         ka2o(:,jo) = 0_il  ; wa2o(:,jo) = 0.0_rl
         kn = 1_il
         DO jn = 1, jma2o
            IF (ktemp(jn) /= 0_il ) THEN
               ka2o(kn,jo) =  ktemp(jn)
               wa2o(kn,jo) =  wtemp(jn)
               kn = kn + 1_il
            END IF
         END DO
      END DO
      nvo = COUNT (ka2o /= 0_il, DIM = 1)
      jma2o = MAXVAL (nvo)
      !!
   END SUBROUTINE gather_wei
   !!
END PROGRAM icestream

