! -*- Mode: f90 -*-
!!! 
!!! lecture d'un fichier NetCDF et interpolation type Mosaic
!!! Suppose un calcul pralable des poids
!!!
!!! Modifiez le module 'modele' selon votre cas de figure
!!!
MODULE declare_1
   !!
   USE defprec
   IMPLICIT NONE
   !! Data types
   INTEGER, PARAMETER :: r4 = r_4, r8 = r_8
   INTEGER, PARAMETER :: i1=1, i2 = 2, i4 = i_4, i8 = i_8
   INTEGER, PARAMETER :: rl = r_std, il = i_std
   INTEGER (kind=il), PARAMETER :: nout = 6 ! Fichier de sortie des messages
   !!
END MODULE declare_1
!!
MODULE modele_1
  USE declare_1
  IMPLICIT NONE
  CHARACTER (len=80) :: cfile_in = 'routing.nc' ! Fichier entree
  CHARACTER (len=80) :: cfile_ou = 'runoff_oce.nc' ! Fichier sortie
  ! OPA -> LMD
!$$$  INTEGER (kind=il), PARAMETER :: jpi1 = 92, jpj1 = 76, jpk1 = 1      ! Dimension modele entree
!$$$  INTEGER (kind=il), PARAMETER :: jpi2 = 72, jpj2 = 46, jpk2 = 1      ! Dimension modele sortie
!$$$  CHARACTER (len=8) :: clweight =  "WEIGHTS1" , cladress = "ADRESSE1" ! Reference des poids et adresses  utiliser
!$$$  INTEGER (kind=il), PARAMETER :: jpn   = 15_il ! Nombre maxi de voisins
  ! LMD -> OPA
  INTEGER (kind=il), PARAMETER :: jpi1 = 72_il, jpj1 = 46_il, jpk1 = 1_il ! Dimension modele entree
  INTEGER (kind=il), PARAMETER :: jpi2 = 92_il, jpj2 = 76_il, jpk2 = 1_il ! Dimension modele sortie
  CHARACTER (len=8) :: clweight =  "WEIGHTS4" , cladress = "ADRESSE4" ! Reference des poids et adresses  utiliser
  INTEGER (kind=il), PARAMETER :: jpn   = 6_il ! Nombre maxi de voisins
  CHARACTER (len=80) :: cfile_ma = '/home/users/marti/GRAF/DATA/orca4.nc' ! Fichier geo sortie
  !
  INTEGER (kind=il), PARAMETER :: jpdim =  4_il ! Nombre maxi de dimensions
  INTEGER (kind=il), PARAMETER :: jpvar = 30_il ! Nombre maxi de variables
  CHARACTER (len=8) :: ctaux = 'FRTU', ctauy = 'FRTV' ! Nom des composantes du stress
  REAL (kind=r8) :: zfreeze = -1.77_r8 ! Point de conglation
  !
  REAL    (kind=rl), DIMENSION ( jpn,jpi2*jpj2) :: weight
  INTEGER (kind=il), DIMENSION ( jpn,jpi2*jpj2) :: kadd
  REAL    (kind=rl), DIMENSION ( jpi2*jpj2) :: glon, glat
END MODULE modele_1
!!
PROGRAM interpol
   USE declare_1
   USE modele_1
   USE fliocom
   USE errioipsl
   IMPLICIT NONE
   INTEGER (kind=i4),PARAMETER :: in = i4
   !! 
   REAL (kind=r8), DIMENSION (jpi1*jpj1) :: zvar_in8
   REAL (kind=r8), DIMENSION (jpi2*jpj2) :: zvar_ou8
   INTEGER (kind=i4), DIMENSION (flio_max_dims) :: dim_len1, dim_len2
   CHARACTER (len=20), DIMENSION (flio_max_dims) :: cldim
   INTEGER (kind=i4) :: jdim, jvar
   INTEGER (kind=i4), DIMENSION (jpvar) :: xtype, ndims, natts
   REAL (kind=r8), DIMENSION (jpvar) :: spval = 0.0_r8
   CHARACTER (len=20), DIMENSION (jpvar) :: clvar
   INTEGER (kind=i4), DIMENSION (jpvar, flio_max_dims) :: dimsid
   INTEGER (kind=in) :: jd, jv, jn, jo 
   INTEGER (kind=il) :: ji, jj, jt, jnt, tid, xid, yid, tid_v, kerr
   INTEGER (kind=i4) :: ncid_in, ncid_ou, ncid_ma, ja
   INTEGER (kind=il) :: len1, len2, nwei = 13
   CHARACTER (len=8) :: c8
   INTEGER (kind=i4), DIMENSION ( jpdim) :: istart1, istart2, icount1, icount2
   CHARACTER (len=80) :: cname
   REAL (kind=r8), DIMENSION ( :), ALLOCATABLE :: zt
   INTEGER (kind=il)       , DIMENSION ( :), ALLOCATABLE :: it
   CHARACTER (len=80) :: cfname
   ! For flio
   CHARACTER (len=80), DIMENSION(1) :: catt
   LOGICAL :: l_ex
   !!
   !! Lecture poids et adresses
   WRITE ( unit = cfname, fmt = '("/home/climdata/ocean/CPL/ORCA4xLMD7245/mozaic.w.runoff.i", 1I1, ".r", 1I1)' ) il, rl
   OPEN ( unit = nwei, file = TRIM(cfname), form = 'unformatted', status = 'old', action = 'read')
   ALLOCATE ( it ( jpi2*jpj2*jpn) )
   CALL locrint (cladress, it, jpi2*jpj2*jpn, nwei, kerr, nout) ;  IF ( kerr /= 0 ) STOP 055
   DO jo = 1, jpi2*jpj2
      DO jn = 1, jpn
         kadd  (jn, jo) = it ( jn + (jo-1)*jpn)
      END DO
   ENDDO
   DEALLOCATE ( it)
   !
   ALLOCATE ( zt ( jpi2*jpj2*jpn) )
   CALL locread (clweight, zt, jpi2*jpj2*jpn, nwei, kerr, nout) ;  IF ( kerr /= 0 ) STOP 056
   DO jo = 1, jpi2*jpj2
      DO jn = 1, jpn
         weight (jn, jo) = zt (jn + (jo-1)*jpn)
      END DO
   ENDDO
   DEALLOCATE ( zt)
   !! Lecture geo
   CALL flioopfd (TRIM(cfile_ma), ncid_ma)
   CALL fliogetv (ncid_ma, 'nav_lon', glon)
   CALL fliogetv (ncid_ma, 'nav_lat', glat)
   CALL flioclo  (ncid_ma)
   !! Ouverture fichier entree
   CALL flioopfd (TRIM(cfile_in), ncid_in, nb_dim=jdim, nb_gat=natts(1))
   !! Dimensions
   CALL flioinqf (ncid_in, ln_dim=dim_len1(1:jdim), nb_var=jvar )
   CALL flioinqn (ncid_in, cn_dim=cldim(1:jdim), cn_var=clvar(1:jvar))
   SeekDim: DO jd = 1, jdim
      WRITE ( unit = c8, fmt = '(1I8)' ) jd
      IF      ( dim_len1 (jd) == jpi1 ) THEN 
         dim_len2 (jd) = jpi2
      ELSE IF ( dim_len1 (jd) == jpj1 ) THEN 
         dim_len2 (jd) = jpj2
      ELSE IF ( dim_len1 (jd) == jpk1 ) THEN 
         dim_len2 (jd) = jpk2
      ELSE 
         dim_len2 (jd) = dim_len1 (jd)
      END IF
      ! Determination du nombre de pas de temps
      jnt = 0
      WRITE(unit=c8,fmt='(2I4)') jd
      IF ( INDEX ( cldim(jd), 'time') /= 0 .OR. INDEX ( cldim(jd), 'TIME') /= 0 .OR. INDEX ( cldim(jd), 'Time') /= 0) THEN 
         jnt = dim_len1(jd) ; tid = jd ; dim_len2 (jd) = -1 ! Unlimited dimension
      END IF
      IF ( INDEX ( cldim(jd), 'X') /= 0 .OR. INDEX ( cldim(jd), 'x') /= 0 ) xid = jd 
      IF ( INDEX ( cldim(jd), 'Y') /= 0 .OR. INDEX ( cldim(jd), 'y') /= 0 ) yid = jd 
   END DO SeekDim
   WRITE ( nout, *) 'Nombre de dimensions : ', jdim
   WRITE ( nout, *) 'Longueur temps       : ', jnt
   WRITE ( nout, *) 'Dimension temps : ', tid, cldim(tid)
   !! Ouverture fichier sortie
   CALL fliocrfd (TRIM(cfile_ou), cldim(1:jdim), dim_len2(1:jdim), ncid_in )
   CALL flioputa (ncid_in, '?', 'Comment', TRIM(c_comment) )
   !! Attributs globaux
   DO ja = 1, natts (1) 
      WRITE ( unit = c8, fmt = '(2I4)' ) ja
      CALL flioinqn (ncid_in, ia_start=ja, ia_count=1, cn_gat=catt) ; cname=catt(1)
      CALL fliocpya (ncid_in, '?', cname, ncid_ou, '?' )
   END DO
   !! Definition variables
   SeekVar: DO jv = 1, jvar
      CALL flioinqv (ncid_in, TRIM(clvar(jv)), l_ex, nb_atts=natts(jv), v_t=xtype(jv), nb_dims=ndims(jv))
      CALL flioinqv (ncid_in, TRIM(clvar(jv)), l_ex, id_dims=dimsid(jv,1:ndims(jv)))
      WRITE ( nout, FMT='(1A20,8I5)' ) TRIM(clvar(jv)), xtype(jv), dimsid(jv,1:ndims(jv)), natts(jv), jv
      IF ( INDEX ( clvar(jv), 'time') /= 0 .OR. INDEX ( clvar(jv), 'TIME') /= 0 .OR. &
         & INDEX ( clvar(jv), 'Time') /= 0) tid_v = jv
      CALL fliodefv (ncid_ou, TRIM(clvar(jv)), dimsid(jv,1:ndims(jv)), v_t=xtype(jv))
      !! Traitements des attributs
      DO ja = 1, natts(jv)
         WRITE ( unit = c8, fmt = '(2I4)' ) jv, ja
         CALL flioinqv (ncid_in, TRIM(clvar(jv)), l_ex, ia_start=ja, ia_count=1, cn_atts=catt) ; cname=catt(1)
         CALL fliocpya (ncid_in, TRIM(clvar(jv)), cname, ncid_ou, TRIM(clvar(jv)) )
      END DO
      IF ( TRIM(cname) == 'missing_value' .OR. TRIM(cname) == 'MISSING_VALUE' ) THEN 
         CALL fliogeta ( ncid_in, TRIM(clvar(jv)), cname, spval(jv))
         WRITE ( unit = nout, fmt = *) 'Missing value ' // TRIM (clvar(jv)) // " : ", spval(jv)
      ENDIF
   END DO SeekVar
   !!
   Temps: DO jt = 1_il, 50 !jnt
      Var: DO jv = 1_il, jvar
         WRITE(unit=c8,fmt='(2I4)') jt, jv
         istart1 = 1_il ; icount1 = 1_il
         istart2 = 1_il ; icount2 = 1_il
         SELECT CASE ( ndims(jv) )
         CASE (1)
            IF      ( jv == tid_v ) THEN 
               istart1 (1_il)= jt ; icount1 (1_il) = 1_il
               istart2 (1_il)= jt ; icount2 (1_il) = 1_il
            ELSE IF ( jv == xid ) THEN
               istart1 (1_il) = 1_il ; icount1 (1_il) = jpi1
               istart2 (1_il) = 1_il ; icount2 (1_il) = jpi2
               CALL flioputv (ncid_ou, TRIM(clvar(jv)),  (/ (REAL(ji,kind=r8),ji=1,jpi2) /) ) ! Ecrit Y
               CYCLE var
            ELSE IF ( jv == yid ) THEN 
               istart1 (1_il) = 1_il ; icount1 (1) = jpj1
               istart2 (1_il) = 1_il ; icount2 (1) = jpj2
               CALL flioputv (ncid_ou, TRIM(clvar(jv)),  (/ (REAL(jj,kind=r8),jj=1,jpj2) /) ) ! Ecrit Y
               CYCLE var
            ELSE
               IF ( jt > 1_il) CYCLE var
            END IF
         CASE ( 2) 
            IF ( jt > 1 ) CYCLE var
            istart1 (1:2) = (/  1_il, 1_il /) ; icount1 (1:2) = (/jpi1, jpj1 /)
            istart2 (1:2) = (/  1_il, 1_il /) ; icount2 (1:2) = (/jpi2, jpj2 /)
         CASE ( 3) 
            istart1 (1:3) = (/  1_il, 1_il, jt /)
            icount1 (1:3) = (/jpi1, jpj1, 1_il /)
            istart2 (1:3) = (/  1_il, 1_il, jt /)
            icount2 (1:3) = (/jpi2, jpj2, 1_il /)
         CASE ( 4) 
            istart1 (1:4) = (/  1_il, 1_il, 1_il, jt /) ; icount1 (1:4)  = (/jpi1, jpj1, 1_il, 1_il /)
            istart2 (1:4) = (/  1_il, 1_il, 1_il, jt /) ; icount2 (1:4)  = (/jpi2, jpj2, 1_il, 1_il /)
         END SELECT
         !!
         len1 = product ( icount1 ) ; len2 = product ( icount2 )
         WRITE ( nout, FMT = '("No ", 1I3, ", Nom : ", 1A20, ", Time : ", 1I3, ", Lens : ", 2I6, ", Type : ", 1I4)' ) &
            &  jv, TRIM(clvar(jv)), jt, len1, len2, xtype(jv)
         !
         ! Lecture
         CALL fliogetv ( ncid_in, TRIM(clvar(jv)), zvar_in8(1:len1), istart1, icount1)
         zvar_in8(1:len1) = zvar_in8(1:len1) + 0.0_r8
         WHERE ( zvar_in8(1:len1) == spval ( jv) ) zvar_in8(1:len1) = 0.0_r8
!$$$      WRITE (nout,*) 'extrema entree : ', MINVAL ( zvar_in8(1:len1)) , MAXVAL ( zvar_in8(1:len1))
         ! Interpolation eventuelle
         IF ( ndims(jv) == 1 ) THEN 
            zvar_ou8(1:len1) = zvar_in8(1:len1)
         ELSE
!$$$          IF ( jv == 6 ) THEN 
!$$$              WRITE(nout,*) 'Attention : traitement glace '
!$$$              DO ji = 1, len1
!$$$                IF ( zvar_in8(ji) <= zfreeze ) THEN
!$$$                     zvar_in8(ji) = 1.0_rl
!$$$                 ELSE
!$$$                     zvar_in8(ji) = 0.0_rl
!$$$                 END IF
!$$$               END DO
!$$$          ENDIF
            IF      ( TRIM(clvar(jv)) == 'nav_lon' .OR. TRIM(clvar(jv)) == 'longitude' .OR. & 
               &      TRIM(clvar(jv)) == 'NAV_LON' .OR. TRIM(clvar(jv)) == 'LONGITUDE' ) THEN
               zvar_ou8(1:len2) = glon
            ELSE IF ( TRIM(clvar(jv)) == 'nav_lat' .OR. TRIM(clvar(jv)) == 'latitude' .OR. &
               &      TRIM(clvar(jv)) == 'NAV_LAT' .OR. TRIM(clvar(jv)) == 'LATITUDE' ) THEN
               zvar_ou8(1:len2) = glat
            ELSE
               CALL inter ( zvar_in8(1:len1), zvar_ou8(1:len2))
            ENDIF
         END IF
!$$$      WRITE (nout,*) 'extrema sortie : ', MINVAL ( zvar_ou8(1:len2)) , MAXVAL ( zvar_ou8(1:len2))
         WRITE (nout,FMT='("extrema : ",4E12.3)' ) MINVAL ( zvar_in8(1:len1)), MINVAL ( zvar_ou8(1:len2)) &
            &  , MAXVAL ( zvar_in8(1:len1)), MAXVAL ( zvar_ou8(1:len2))
         ! Ecriture
         CALL flioputv ( ncid_ou, TRIM(clvar(jv)), zvar_ou8(1:len2))
      END DO Var
   END DO Temps

   !!
   CALL flioclo ( ncid_in)
   CALL flioclo ( ncid_ou)

   STOP
CONTAINS
   SUBROUTINE inter ( pin, pout)
      USE declare_1
      USE modele_1
      IMPLICIT NONE
      REAL (kind=r8), INTENT ( in), DIMENSION ( jpi1*jpj1) :: pin
      REAL (kind=r8), INTENT (out), DIMENSION ( jpi2*jpj2) :: pout
      INTEGER (kind=il) :: ji, jn, in
      !!
      pout = 0.0_rl
      !
      DO ji = 1, jpi2*jpj2
         DO jn = 1, jpn
            in  = kadd ( jn, ji)
            IF ( in > 0 ) pout ( ji) = pout ( ji) + weight ( jn, ji) * pin ( in )
         END DO
      END DO
      !!
   END SUBROUTINE inter
   !!
   SUBROUTINE locread ( cdfldn, pfield, kdimax, knulre, kflgre, kout)
      USE declare_1
      !!
      IMPLICIT NONE
      !
      CHARACTER (LEN = 8), INTENT ( in) :: cdfldn ! Name of field to search for
      INTEGER (kind=il), INTENT (  in) :: kdimax, knulre    ! Field dimension, unit to read
      INTEGER (kind=il), INTENT (  in) :: kout              ! Standard output
      INTEGER (kind=il), INTENT ( out) :: kflgre            ! Error status code
      REAL (kind=r8)  , INTENT ( out), DIMENSION ( :) :: pfield ! Field
      !
      CHARACTER (len = 8) clecfl
      INTEGER (kind=il) :: ji, kerr
      !
      !*    1. Initialization
      !        --------------
      !
      WRITE (UNIT = kout,FMT = '( "Locread :  Read field ", A8, " in unit = ", 1I6)' ) cdfldn, knulre
      !
      !     2. Find field in file
      !        ------------------
      !
      REWIND ( UNIT = knulre)
200   CONTINUE
      !* Find string
      READ (UNIT = knulre, ERR = 200, END = 210, IOSTAT = kerr) clecfl
      IF ( clecfl /= cdfldn ) GO TO  200
      !* Read associated field
      READ (UNIT = knulre, IOSTAT = kerr) ( pfield (ji),ji = 1, kdimax)
      IF ( kerr /= 0 ) WRITE ( unit = nout, fmt = *) 'Problem reading field'
      kflgre = 0
      GO TO 220
      !* Problem in reading
210   kflgre = 1
      WRITE ( unit = nout, fmt = *) 'Problem in reading'
220   CONTINUE
      !
      !*    3. End of routine
      !        --------------
      !
      RETURN
   END SUBROUTINE locread
   !
   SUBROUTINE locrint ( cdfldn, kfield, kdimax, knulre, kflgre, kout)
      !
      USE declare_1
      !!
      IMPLICIT none
      !
      CHARACTER (LEN = 8), INTENT ( in) :: cdfldn ! Name of field to search for
      INTEGER (kind=il), INTENT (  in) :: kdimax, knulre    ! Field dimension, unit to read
      INTEGER (kind=il), INTENT (  in) :: kout              ! Standard output
      INTEGER (kind=il), INTENT ( out) :: kflgre            ! Error status code
      INTEGER (kind=il), INTENT ( inout), DIMENSION (:) :: kfield ! Field
      !
      CHARACTER (len = 8) :: clecfl
      INTEGER (kind=il) :: ji, kerr
      !
      !*    1. Initialization
      !        --------------
      !
      WRITE (UNIT = kout, FMT = '( "Locrint :  Read field ", A8, " in unit = ", 1I6, " Dim : ", 1I9)' ) cdfldn, knulre, kdimax
      !
      !     2. Find field in file
      !        ------------------
      !
      REWIND ( UNIT = knulre )
200   CONTINUE
      !
      !* Find string
      READ (UNIT = knulre, ERR = 200, END = 210) clecfl
!$$$    WRITE ( unit = kout, fmt = *) clecfl
      !
      IF ( clecfl /= cdfldn ) GO TO  200
      !* Read associated field
      READ (UNIT = knulre, IOSTAT = kerr) ( kfield (ji),ji = 1, kdimax)
      IF ( kerr /= 0 ) WRITE ( unit = nout, fmt = *) 'Problem reading field'
      !* Reading done and ok
      kflgre = 0
      GO TO 220
      !* Problem in reading
210   kflgre = 1
      WRITE ( unit = nout, fmt = *) 'Problem in reading. ji : ', ji, kfield(1), kfield(kdimax)
      !
220   CONTINUE
      !
      !*    3. End of routine
      !        --------------
      !
      RETURN
   END SUBROUTINE locrint
   !
END PROGRAM interpol
