! -*- Mode: f90 -*-
PROGRAM isf_icb
   USE declare
   USE modeles
   USE mod_lecdata
   USE formula
   USE math
   USE mod_rectifie
   USE mod_lbc
   USE bords
   USE mod_prih
   USE mod_wri_wei
   USE fliocom
   USE errioipsl
   USE getincom
   USE mod_inipar
   USE mod_inter
!!!
   IMPLICIT NONE
!!!
!!! Calcul pour poids pour le calving qui va aller dans les parametrisations isf et icb de NEMO.
!!!
   REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: ynorth, ysouth
   REAL (kind=rl), PARAMETER :: zm_frac = 0.0_rl ! Fraction criterium
   !!
   INTEGER (kind=il) :: ja, jo, jn, joi, joj, jai, jaj, jr, kn ! indices de boucle
   INTEGER (kind=il), DIMENSION (:, :), ALLOCATABLE :: imask ! Masque
   REAL (kind=rl)    :: zosrf, zasrf ! Surface totale
   REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: zasrf_lat ! Surface atmosphere par bande de latitude
   INTEGER (kind=il), DIMENSION (:), ALLOCATABLE :: ktemp
   REAL(kind=rl), DIMENSION (:), ALLOCATABLE :: wtemp, rnf_key
   REAL(kind=rl), DIMENSION (:, :), ALLOCATABLE  :: z2o
   CHARACTER (len=80) :: cfname4, cfname8, cfname_cdf
   INTEGER (kind=il), DIMENSION (:,:), ALLOCATABLE  :: i2a
   INTEGER (kind=il), DIMENSION (:,:), ALLOCATABLE  :: i2o
   CHARACTER (len=50) :: cldiag_a2o, clw_a2o, clw_a2o_cdf, clw_a2o_mct, c_poids, calving_name
   INTEGER (kind=il) :: ierr, il_icb
   LOGICAL :: l_no_pac, l_compute, l_iceshelf, l_iceberg, l_standard
   REAL (kind=rl), DIMENSION (:,:)  , ALLOCATABLE  :: w2d
   REAL (kind=rl), DIMENSION (:,:,:), ALLOCATABLE  :: w3d
   REAL(kind=rl) :: rnf_key_sum
   !!
   INTEGER (kind=il), DIMENSION (2) :: jind
   INTEGER (kind=i_4) :: il_ncid
   CHARACTER (len=40) :: cl_bassin, cpoids
   REAL(kind=rl), DIMENSION (:), ALLOCATABLE :: tmask_atl, tmask_noclose, tmask_nomed, tmask_med
   !!
   CHARACTER (len = 25) :: mo_name, a2o_name
   CHARACTER (len =  1) :: c_i, c_r
   CHARACTER (len=80) :: clarg
   INTEGER :: narg
   !!
   !! Lecture de la ligne de comande
   narg = iargc()
   IF ( narg > 1 ) THEN
      CALL getarg ( 1, clarg)
      SELECT CASE (TRIM(clarg))
      CASE Default
         IF ( INDEX ( TRIM(clarg), '.def' ) /= 0 ) THEN
            WRITE (nout,*) 'Lecture des parametres dans ', TRIM(clarg)
            CALL getin_name ( TRIM (clarg) )
         ELSE
            WRITE (nout,*) 'Lecture des parametres dans run.def'
         ENDIF
      END SELECT
   END IF
   !!
   CALL inipar
   CALL alloc_modeles
   !!
   !! Initialisation
   !!
   ! Lecture sur la ligne de commande du calcul à faire
   CALL getarg ( 2, c_poids)
   WRITE (nout,*) 'Calcul a faire : ', TRIM(c_poids)
   SELECT CASE ( TRIM(c_poids))
   CASE ( 'iceshelf'        ) ; calving_name = 'calving_iceshelf'
   CASE ( 'iceberg'         ) ; calving_name = 'calving_iceberg'
   CASE ( 'calving_nosouth' ) ; calving_name = 'calving_nosouth'
   CASE ( 'calving_full'    ) ; calving_name = 'calving_full'
   CASE Default
      WRITE (nout,*) 'Cas non reconnu'
      STOP
   END SELECT
   
   ALLOCATE (imask (jpon, jp_calv), STAT=ierr)    ; CALL chk_allo (ierr, 'imask', lreset=.TRUE., crout='icestream')
   ALLOCATE (zasrf_lat (jpaj), STAT=ierr)     ; CALL chk_allo (ierr, 'zasrf_lat')
   ALLOCATE (wtemp (jpa2o), STAT=ierr)        ; CALL chk_allo (ierr, 'wtemp')
   ALLOCATE (ktemp (jpa2o), STAT=ierr)        ; CALL chk_allo (ierr, 'ktemp')
   ALLOCATE (z2o (jpoi, jpoj), STAT=ierr)     ; CALL chk_allo (ierr, 'z2o')
   ALLOCATE (i2a (jpai, jpaj), STAT=ierr)     ; CALL chk_allo (ierr, 'i2a')
   ALLOCATE (i2o (jpoi, jpoj), STAT=ierr)     ; CALL chk_allo (ierr, 'i2o')
   ALLOCATE (tmask_atl     (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'tmask_atl')
   ALLOCATE (tmask_noclose (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'tmask_noclose')
   ALLOCATE (tmask_nomed   (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'tmasl_nomed')
   ALLOCATE (tmask_med     (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'tmask_med')
   ALLOCATE (rnf_key       (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'rnf_key')
   !!
   ALLOCATE (ynorth (1:jp_calv), STAT=ierr) ; CALL chk_allo (ierr, 'ynorth')
   ALLOCATE (ysouth (1:jp_calv), STAT=ierr) ; CALL chk_allo (ierr, 'ynorth')
   !!
   ysouth (1:jp_calv) = ylimits (1:jp_calv)
   ynorth (1:jp_calv) = ylimits (2:jp_calv+1)
   !!
   !! Initialisation
   ngrd = 11 ; nsrf = 12 ; nmsk = 13 ; nchk = 14 ; ndeb = 9 ; nbug1 = 10
   nwei4o2a = 21 ; nwei8o2a = 22 ; nwei4a2o = 23 ; nwei8a2o = 24
   !
   nout = 0
   IF (nout /= 6 .AND. nout /= 0 ) THEN
      OPEN (unit = nout, file = 'icestream' // TRIM(c_suffix) // '.out', action = 'write', status = 'unknown', position = 'rewind' )
   END IF
   !
   jma2o = 0_il
   jmo2a = 0_il
   nvo = 0_il
   !!
   !! Read coordinates of all models, and weights
   !!
   CALL lecdata (lecweights = .FALSE., leco2amask = .TRUE. )
   !!
   wa2o = zero
   ka2o = 0_il
   !
!    CALL ipsldbg (.TRUE. )
   !
   WRITE (nout,*) 'Periodicite'
   CALL lbc (xolont, noperio, 'T', jpoi, jpoj)
   CALL lbc (xolonu, noperio, 'U', jpoi, jpoj)
   CALL lbc (xolonv, noperio, 'V', jpoi, jpoj)
   CALL lbc (xolonf, noperio, 'F', jpoi, jpoj)
   CALL lbc (xolatt, noperio, 'T', jpoi, jpoj)
   CALL lbc (xolatu, noperio, 'U', jpoi, jpoj)
   CALL lbc (xolatv, noperio, 'V', jpoi, jpoj)
   CALL lbc (xolatf, noperio, 'F', jpoi, jpoj)
   CALL lbc (xosrft, noperio, 'T', jpoi, jpoj)
   CALL lbc (iomskt, noperio, 'T', jpoi, jpoj)
   !
   CALL lbc (xalont, naperio, 'T', jpai, jpaj)
   CALL lbc (xalonu, naperio, 'U', jpai, jpaj)
   CALL lbc (xalonv, naperio, 'V', jpai, jpaj)
   CALL lbc (xalatt, naperio, 'T', jpai, jpaj)
   CALL lbc (xalatu, naperio, 'U', jpai, jpaj)
   CALL lbc (xalatv, naperio, 'V', jpai, jpaj)
   CALL lbc (xasrft, naperio, 'T', jpai, jpaj)
   CALL lbc (iamskt, naperio, 'T', jpai, jpaj)
   !!
   WRITE (nout,*) 'Rectifie'
   CALL rectifie
   !!
   WRITE (nout,*) 'Calcul de la surface par bande de latitude atmosphere'
   zasrf_lat(:) = 0.0_rl
   DO jaj = 1_il, jpaj
      DO jai = 1_il, jpai
         ja = m1a(jai, jaj)
         zasrf_lat(jaj) = zasrf_lat(jaj) + xasrft(ja)
      END DO
   ENDDO
   DO jaj = 1, jpaj
      WRITE(UNIT=nout,FMT='(1F7.1,1PE15.3)' ) xalatt ( m1a (1_il, jaj)), zasrf_lat (jaj)
   END DO
   WRITE(nout,*) SUM (zasrf_lat)
   !!
   CALL bord_a
   !!
   mo_name  = "mozaic"
   a2o_name = "a2o"
   !!
   WRITE (nout,*) 'bord_o'
   CALL bord_o
   WRITE (nout,*) 'bord_a'
   CALL bord_oa
   !!
   WRITE (nout,*) 'o2amask ', COUNT (o2amask > 0 ), COUNT (o2amask <=  0 )
   i2a = 0_il
   WHERE (RESHAPE(o2amask,(/jpai,jpaj/)) .GT. 0.0_rl ) i2a = 1_il
   !CALL prihin (i2a , 1_il, nout )
   !!
   !! Lecture masques de bassin
   IF ( TRIM (cotyp) == 'orca2' .OR. TRIM (cotyp) == 'orca2.0' .OR. TRIM (cotyp) == 'orca2.1' .OR. TRIM (cotyp) == 'orca4' ) THEN 
      cl_bassin = TRIM (cotyp)
!-$$      IF (c_period /= 'none' ) cl_bassin = TRIM (cl_bassin) // TRIM (c_period)
      cl_bassin = TRIM (cl_bassin) // '.nc'
      CALL flioopfd (TRIM (cl_bassin), il_ncid )
      CALL fliogetv (il_ncid, 'mask_atl'    , z2o )
      tmask_atl = RESHAPE (z2o, (/ jpon /) )
      CALL fliogetv (il_ncid, 'mask_noclose', z2o )
      tmask_noclose = RESHAPE (z2o, (/ jpon /) )
      CALL fliogetv (il_ncid, 'mask_nomed'  , z2o )
      tmask_nomed = RESHAPE (z2o, (/ jpon /) )
      CALL flioclo (il_ncid)
   ELSE
      tmask_atl     = REAL (1-iomskt, rl)
      tmask_noclose = REAL (1-iomskt, rl)
      tmask_nomed   = REAL (1-iomskt, rl)
   END IF
   !
   WRITE (unit=nout, fmt=*) 'iomskt'
   !CALL prihin (RESHAPE(iomskt             ,(/jpoi,jpoj/)), 1_il, nout)
   WRITE (unit=nout, fmt=*) 'tmask_atl'
   !CALL prihin (RESHAPE(NINT(tmask_atl)    ,(/jpoi,jpoj/)), 1_il, nout)
   WRITE (unit=nout, fmt=*) 'tmask_med'
   !CALL prihin (RESHAPE(NINT(tmask_med)    ,(/jpoi,jpoj/)), 1_il, nout)
   WRITE (unit=nout, fmt=*) 'tmask_noclose'
   !CALL prihin (RESHAPE(NINT(tmask_noclose),(/jpoi,jpoj/)), 1_il, nout)
   !
   Bassin: DO jr = 1_il, jp_calv !! Boucle sur les zones
      WRITE (nout,*) '==================================================================='
      WRITE (nout,*) 'Zone : ', jr, ' sud: ', ysouth(jr), '  nord : ', ynorth(jr)
      !!
      !! Creation du masque
      !!
      imask (:,jr) = 1_il
      !!
      tmask_med = 0.0_r_4
      WHERE (tmask_atl > 0.5_r_4 .AND. tmask_nomed < 0.5_r_4 ) tmask_med = 1.0_r_4

      ! Limite nord incluse, limite sud exclue
      ! Ne pas faire pour la bande sud dans le cas iceberg, car les icebergs vont plus loin au nord
      IF ( jr == 1 .AND.  TRIM (c_poids) == 'iceberg' ) THEN
         WHERE (tmask_noclose > 0.5_r_4 .AND. xolatt < -10.0_rl ) imask(:,jr) = iomskt
      ELSE
         WHERE (tmask_noclose > 0.5_r_4 .AND. (xolatt > ysouth (jr) .AND. xolatt <= ynorth (jr)) ) imask(:,jr) = iomskt
         WHERE (tmask_med (:) > 0.5_rl ) imask (:,jr) = 1_il
         IF (l_no_pac) WHERE (tmask_atl (:) < 0.5_rl .AND. xolatt > ysouth (jp_calv) )  imask (:,jr) = 1_il
      ENDIF
      !!
      WRITE(nout,*) 'Bassin ', jr
      i2o =  RESHAPE ( imask(:,jr), (/jpoi, jpoj/))
      !CALL prihin (i2o, 1_il, nout)
      !!
      WRITE (nout,*) 'Calcul de la surface totale de la zone ocean'
      zosrf = DOT_PRODUCT (xosrft, REAL(1-imask(:,jr),KIND=rl)*REAL(1-iomskp,KIND=rl) )
      WRITE(nout,*) 'Surface de la zone ', jr, zosrf
      !!
      !! Calcul de la surface totale de la zone atmosphere
      zasrf = 0.0_rl
      DO ja = 1, jpan
         IF ( xalatt(ja) > ysouth(jr) .AND. xalatt(ja) <= ynorth(jr)) zasrf = zasrf + xasrft(ja)
      END DO
      !!
      l_compute = .FALSE. ; l_iceshelf = .FALSE. ; l_iceberg = .FALSE. ; l_standard = .FALSE.
      IF ( ysouth(jr) .LE. -89.0_rl ) THEN
         IF (      TRIM(c_poids) == 'iceshelf'        ) THEN
            l_iceshelf = .TRUE. ; l_compute = .TRUE.
         ELSE IF ( TRIM(c_poids) == 'iceberg'         ) THEN
            l_iceberg  = .TRUE. ; l_compute = .TRUE.
         ELSE IF ( TRIM(c_poids) == 'calving_nosouth' ) THEN
            l_compute = .FALSE.
         ELSE
            l_compute = .TRUE. ; l_standard = .TRUE. 
         END IF
      ELSE
         IF (      TRIM(c_poids) == 'iceshelf'  .OR. TRIM (c_poids) == 'iceberg'         ) THEN
            l_compute = .FALSE.
         ELSE
            l_compute = .TRUE. ; l_standard = .TRUE. 
         END IF
      END IF
      
      !!
      !! Calcul des poids
      Compute: IF ( l_compute ) THEN
         Sud: IF ( l_iceberg .OR. l_iceshelf ) THEN
            WRITE (nout,*) 'Calcul des poids : Antarctique : ', TRIM(c_poids)
            ! Lecture des données de de fonte des ice shelf autour de l'Antarctique et de fonte d'iceberg
            CALL flioopfd ('runoff-icb_DaiTrenberth_Depoorter_eORCA1_JD.nc', il_icb)
            ALLOCATE ( w3d (jpoi, jpoj, 12))
            ALLOCATE ( w2d (jpoi, jpoj))
            IF ( l_iceshelf ) CALL fliogetv (il_icb, 'sornfisf', w3d(:,:,:))
            IF ( l_iceberg  ) CALL fliogetv (il_icb, 'Icb_flux', w3d(:,:,:))
            w2d (:,:) = SUM ( w3d(:,:,:), DIM=3)
            !
            !WRITE (nout,*) 'champ avant rescale ' 
            !CALL prihre (w2d (:,:), 0.0, nout )
            !
            rnf_key (:) = RESHAPE ( w2d, (/ jpon /))
            DEALLOCATE (w2d) ; DEALLOCATE (w3d)
            CALL flioclo ( il_icb)
            !
            ! Integrale du flux
            rnf_key_sum = SUM ( rnf_key(:)*xosrft(:)*REAL(1- imask(:,jr),KIND=rl)*REAL(1-iomskp,KIND=rl) )
            WRITE (nout,*) 'Somme ',  TRIM(c_poids), ' : ', rnf_key_sum
            ! Normalisation
            rnf_key (:) = rnf_key (:) * zosrf / rnf_key_sum * REAL(1- imask(:,jr),KIND=rl)
            CALL lbc (rnf_key (:) , noperio, 'T', jpoi, jpoj)
            !
            !WRITE (nout,*) 'champ apres rescale ' 
            !CALL prihre ( RESHAPE(rnf_key (:), (/jpoi,jpoj/)), 0.0, nout)
            !
            WRITE (nout,*) 'Verif somme   : ', SUM ( rnf_key(:) )
            WRITE (nout,*) 'Verif min/max : ', MINVAL ( rnf_key(:)), MAXVAL ( rnf_key(:))
            WRITE (nout,*) 'Verif somme   : ', SUM ( xosrft(:)*REAL(1- imask(:,jr),KIND=rl)*REAL(1-iomskp,KIND=rl) ), SUM ( rnf_key(:) * xosrft(:)*REAL(1- imask(:,jr),KIND=rl)*REAL(1-iomskp,KIND=rl) )
            !
            DO jo = 1, jpon ! Boucle sur tous les points ocean
               IF (imask(jo,jr) == 0_il ) THEN ! Vrai si le point NEMO est un point ocean dans la bande de latitude
                  DO jaj = 1_il, jpaj ! Boucle sur les latitudes atmosphere
                     ja = m1a(1_il, jaj) ! Indice 1D du point le plus à l'ouest
                     IF ( xalatt(ja) > ysouth(jr) .AND. xalatt(ja) <= ynorth(jr)) THEN ! Test si le point atmosphere est dans la bande de latitude
                        nvo (jo) = nvo (jo) + 1 ! Increment du nombre de voisins
                        IF (nvo(jo) .GT. jpa2o ) THEN ! Test sur le nombre maximum de voisin
                           WRITE (nout,*) 'Nombre de voisins insuffisant ', jpa2o
                           WRITE (nout,*) jr, jo, ja, xalatt(ja), ysouth(jr), ynorth(jr)
                           STOP
                        END IF
                        ka2o (nvo(jo), jo) = ja
                        wa2o (nvo(jo), jo) = zasrf_lat(jaj) / zosrf * rnf_key(jo)
                     END IF
                  END DO
               END IF
            END DO
         ELSE IF ( l_standard) THEN
            WRITE (nout,*) 'Cas standard'
            WRITE (nout,*) 'Calcul des poids ',  ysouth(jr), ynorth(jr)
            DO jo = 1, jpon ! Boucle sur tous les points ocean
               IF (imask(jo,jr) == 0_il ) THEN ! Vrai si le point NEMO est un point ocean dans la bande de latitude
                  DO jaj = 1_il, jpaj ! Boucle sur les latitudes atmosphere
                     ja = m1a(1_il, jaj) ! Indice 1D du point le plus à l'ouest
                     IF ( xalatt(ja) > ysouth(jr) .AND. xalatt(ja) <= ynorth(jr)) THEN ! Test si le point atmosphere est dans la bande de latitude
                        nvo (jo) = nvo (jo) + 1 ! Increment du nombre de voisins
                        IF (nvo(jo) .GT. jpa2o ) THEN ! Test sur le nombre maximum de voisin
                           WRITE (nout,*) 'Nombre de voisins insuffisant ', jpa2o
                           WRITE (nout,*) jr, jo, ja, xalatt(ja), ysouth(jr), ynorth(jr)
                           STOP
                        END IF
                        ka2o (nvo(jo), jo) = ja
                        wa2o (nvo(jo), jo) = zasrf_lat(jaj) / zosrf ! Division par la surface totale de la zone océanique
                     ENDIF
                  ENDDO
               END IF
            END DO
            !!
         ELSE
            WRITE (nout,*) 'On ne fait rien, et on ne devrait pas passer par la ... !'
            STOP
         END IF Sud
      ELSE
         WRITE (nout,*) 'On ne fait rien'
      END IF Compute
      jma2o = MAXVAL (nvo)
      WRITE (nout,*) 'jma2o, first guess : ', jma2o
      !!
   END DO Bassin
   !!
   WRITE (nout,*) '==================================================================='
   !!
   WRITE(nout, *) 'jma2o : ', jma2o
   !!
   ! Tests bassins
!-$$   WRITE(nout,*) 'Somme des bassins '
!-$$   i2o =  RESHAPE (SUM(1-imask,DIM=2)*(1-iomskp), (/jpoi, jpoj/) )
!-$$   CALL prihin (i2o, 1_il, nout)
!-$$   !
!-$$   WRITE(nout,*) 'Somme des bassins '
!-$$   i2o =  RESHAPE ((1-iomskt)-SUM(1-imask,DIM=2)*(1-iomskp), (/jpoi, jpoj/) )
!-$$   CALL prihin (i2o, 1_il, nout)
!-$$   !
!-$$   WRITE(nout,*) 'Somme des bassins '
!-$$   i2o =  RESHAPE (SUM(imask,DIM=2), (/jpoi, jpoj/))
!-$$   CALL prihin (i2o, 1_il, nout)
   !!
   !! Normalisation
   !!
!$$$      WRITE (nout, *) 'wa2o ', MINVAL (wa2o), MAXVAL (wa2o), COUNT (wa2o > 0.0_rl)
!$$$      WRITE (nout, *) 'ka2o ', MINVAL (ka2o), MAXVAL (ka2o), COUNT (ka2o > 0 )
!$$$      WRITE (nout, *) 'xasrft ', MINVAL (xasrft), MAXVAL (xasrft)
!$$$      WRITE (nout, *) 'o2amask ', MINVAL (o2amask), MAXVAL (o2amask), SUM (o2amask)
!$$$      WRITE (nout, *) 'xosrft ', MINVAL (xosrft), MAXVAL (xosrft)
!$$$      WRITE (nout, *) 'iomskt ', MINVAL (iomskt), MAXVAL (iomskt), COUNT (iomskt == 0)
!$$$      WRITE (nout, *) 'iomskp ', MINVAL (iomskp), MAXVAL (iomskp), COUNT (iomskp == 0)
!$$$      WRITE (nout, *) 'iamskt ', MINVAL (iamskt), MAXVAL (iamskt), COUNT (iamskt == 0)
!$$$      WRITE (nout, *) 'locot ', COUNT (locot)
   !!
   WRITE(nout,*) 'gather weights'
   !CALL gather_wei
   !!
   WRITE(nout,*) 'Checking weights'
   WRITE (unit = nout, fmt = '("Neighbors a2o     : ", 3I9  )' ) MINVAL (nvo, nvo /=0), MAXVAL (nvo), jma2o
   WRITE (unit = nout, fmt = '("Index a2o         : ", 2I9  )' ) MINVAL (ka2o, ka2o /= 0), MAXVAL (ka2o)
   jind = MINLOC(wa2o, wa2o /= 0.0_rl) ; jo = jind(2) ; jn = jind(1); joi = m2oi(jo) ; joj = m2oj(jo)
   WRITE (unit = nout, fmt = '("Weights a2o mini : ", 4I6, 1E13.4, 3I6)' ) &
      & jo, jn, joi, joj, wa2o(jn,jo), ka2o(jn,jo), m2ai(ka2o(jn,jo)), m2aj(ka2o(jn,jo))
   jind = MAXLOC(wa2o, wa2o /= 0.0_rl) ; jo = jind(2) ; jn = jind(1); joi = m2oi(jo) ; joj = m2oj(jo)
   WRITE (unit = nout, fmt = '("Weights a2o maxi : ", 4I6, 1E13.4, 3I6)' ) &
      & jo, jn, joi, joj, wa2o(jn,jo), ka2o(jn,jo), m2ai(ka2o(jn,jo)), m2aj(ka2o(jn,jo))
   !
   jind(1:1) = INT(MAXLOC (nvo),il) ; jo = jind(1)
   WRITE (unit = nout, fmt = *) jo, m2oi(jo), m2oj(jo), ka2o(1:nvo(jo),jo), &
      & (m2ai(ka2o(jn,jo)), m2aj(ka2o(jn,jo)), jn = 1, nvo(jo))
   !!
   WRITE(nout,*) 'Calcul du masque'
   CALL inter_a2o (REAL (1-iamskt, KIND = rl), a2omask)
   !!
!-$$   za = 1.0_rl
!-$$   CALL inter_a2o (za, zo)
!-$$   !
!-$$   zsuma = DOT_PRODUCT (za, xasrft*o2amask)
   
   !!
!-$$   WRITE(nout,*) 'Verif nombre de voisins'
!-$$   !!
!-$$   IF ( jma2oi /= jma2o ) THEN 
!-$$      WRITE (nout, *) 'Erreur nombre de voisins dans run.def : '
!-$$      WRITE (nout, *) ' jma2o calcule : ', jma2o
!-$$      WRITE (nout, *) ' jma2o lu      : ', jma2oi
!-$$      STOP
!-$$   ENDIF
   !!
   WRITE(nout,*) 'Ecriture des poids'
   !!
   WRITE (unit = c_r, fmt = '(1i1)' ) rk_out

   SELECT CASE ( TRIM(c_poids))
   CASE ( 'iceshelf'        ) ; calving_name = 'calving_iceshelf'
   CASE ( 'iceberg'         ) ; calving_name = 'calving_iceberg'
   CASE ( 'calving_nosouth' ) ; calving_name = 'calving_nosouth'
   CASE ( 'calving_full'    ) ; calving_name = 'calving_full'
   END SELECT
   
   IF (l_wei_i4) THEN 
      WRITE (unit = c_i, fmt = '(1i1)' ) i_4
      cfname4 = TRIM(mo_name) // ".wa2o." // TRIM(calving_name) // TRIM(c_suffix)  // ".i" // TRIM(c_i) // ".r" // TRIM(c_r) 
      OPEN (unit = nwei4a2o, file = TRIM(cfname4), form = 'unformatted', status = 'replace', action = 'write')
      WRITE (nout, *) 'Poids i_4 a -> o : ', TRIM(cfname4)
   END IF

   IF (l_wei_i8) THEN
      WRITE (unit = c_i, fmt = '(1i1)' ) i_8
      cfname8 = TRIM(mo_name) // ".wa2o." // TRIM(calving_name) // TRIM(c_suffix) // ".i" // TRIM(c_i)  // ".r" // TRIM(c_r)
      OPEN (unit = nwei8a2o, file = TRIM(cfname8), form = 'unformatted', status = 'replace', action = 'write')
      WRITE (nout, *) 'Poids i_8 a -> o : ', TRIM(cfname8)
   END IF

   cldiag_a2o  = TRIM(a2o_name) // "." // TRIM(calving_name)  // ".diag" ! // TRIM(c_suffix) // ".nc"
   clw_a2o     = "mozaic.wa2o." // TRIM(calving_name) // TRIM(c_suffix)
   clw_a2o_mct = "rmp_" // TRIM(camod_t) // "_to_" // TRIM(comod_t) // "_MOSAIC_" // TRIM(calving_name) ! // TRIM(c_suffix) // ".nc"

   WRITE (nout, *) 'Calling wri_weights_a2o ', cldiag_a2o, clw_a2o, "5"
   CALL wri_weights_a2o (cldiag_a2o, clw_a2o, clw_a2o_mct, "5" )      

   IF (l_wei_i4) CLOSE (unit = nwei8a2o)
   IF (l_wei_i8) CLOSE (unit = nwei4a2o)
   !
   STOP
   !!
CONTAINS
   !!
   SUBROUTINE gather_wei
      WRITE(nout,*) 'Gather weight : Periodicity'
      DO jn = 1, jpa2o
         CALL lbc (wa2o(jn,:) , noperio, 'T', jpoi, jpoj)
         CALL lbc (ka2o(jn,:) , noperio, 'T', jpoi, jpoj)
      END DO
      !!
      WRITE(nout,*) 'Gather weights : gather'
      DO jo = 1, jpon
         ktemp = ka2o(:,jo) ; wtemp = wa2o(:,jo)
         ka2o(:,jo) = 0_il  ; wa2o(:,jo) = 0.0_rl
         kn = 1_il
         DO jn = 1, jma2o
            IF (ktemp(jn) /= 0_il ) THEN
               ka2o(kn,jo) =  ktemp(jn)
               wa2o(kn,jo) =  wtemp(jn)
               kn = kn + 1_il
            END IF
         END DO
      END DO
      WRITE(nout,*) 'Gather weights : stat'
      nvo = COUNT (ka2o /= 0_il, DIM = 1)
      jma2o = MAXVAL (nvo)
      !!
   END SUBROUTINE gather_wei
   !!
END PROGRAM isf_icb

