! -*- Mode: f90 -*-
MODULE mod_lbc
   USE declare
   !> Periodicite des champs
   !
   !> Multiply 2D vectors
   INTERFACE lbc
      MODULE PROCEDURE lbc_r1, lbc_i1, lbc_l1, lbc_r2, lbc_i2, lbc_l2, lbc_r3
   END INTERFACE lbc
CONTAINS
   !
   !> Periodicity of real fields
   SUBROUTINE lbc_r1 (ptab, npolj, cd_type, ni, nj, sgn)
      IMPLICIT NONE
      !
      INTEGER (kind=il), INTENT (in) :: npolj !< Type of periodicity
      INTEGER (kind=il), INTENT (in) :: ni !< Array dimension
      INTEGER (kind=il), INTENT (in) :: nj !< Array dimension
      CHARACTER (len=*), INTENT (in) :: cd_type !< Grid specification : T, U, V or F
      REAL (kind=rl), DIMENSION (ni * nj), INTENT (inout) :: ptab !< Input array
      REAL (kind=rl), INTENT (in), OPTIONAL :: sgn
      REAL (kind=rl), DIMENSION (ni, nj) :: ptab2d !< Temp array
      INTEGER :: ji, jl, ijt, iju, ijpj
      !
      ptab2d (:,:) = RESHAPE (ptab, (/ ni, nj /) )
      !
      IF ( PRESENT (sgn)) THEN
         CALL lbc_r2 ( ptab2d (:,:), npolj, cd_type, sgn)
      ELSE
         CALL lbc_r2 ( ptab2d (:,:), npolj, cd_type)
      ENDIF
      !
      ptab (:) = RESHAPE ( ptab2d, (/ ni*nj /) )

   END SUBROUTINE lbc_r1
   !
   SUBROUTINE lbc_r3 (ptab3d, npolj, cd_type, sgn)
      !> Periodicity of real fields
      IMPLICIT NONE
      !
      INTEGER (kind=il), INTENT (in) :: npolj !< Type of periodicity
      REAL (kind=rl), DIMENSION (:, :, :), INTENT (inout) :: ptab3d !< Input array
      CHARACTER (len=*), INTENT (in) :: cd_type !< Grid specification : T, U, V or F
      REAL (kind=rl), INTENT (in), OPTIONAL :: sgn
      INTEGER :: jk

      DO jk = 1, SIZE ( ptab3d, DIM=3) 
         IF ( PRESENT (sgn)) THEN
            CALL lbc_r2 ( ptab3d (:,:, jk), npolj, cd_type, sgn)
         ELSE
            CALL lbc_r2 ( ptab3d (:,:, jk), npolj, cd_type)
         ENDIF
      END DO

   END SUBROUTINE lbc_r3

   SUBROUTINE lbc_i2 (pt2d, npolj, cd_type, sgn) 
      !> Periodicity on integer fields
      IMPLICIT NONE
      !!
      INTEGER (kind=il), INTENT (in) :: npolj !< Type of periodicity
      INTEGER (kind=il), DIMENSION (:, :), INTENT (inout) :: pt2d !< Input array
      CHARACTER (len=*), INTENT (in) :: cd_type !< Grid specification : T, U, V or F
      REAL (kind=rl), DIMENSION (SIZE(pt2d,DIM=1),SIZE(pt2d,DIM=2)) :: ztab2d !< Temp array 
      INTEGER (kind=il), INTENT (in), OPTIONAL :: sgn
      !
      ztab2d (:,:) = REAL ( pt2d(:,:), KIND=rl)
      !
      IF ( PRESENT (sgn)) THEN 
         CALL lbc_r2 ( ztab2d (:,:), npolj, cd_type, REAL(sgn,KIND=rl))
      ELSE
         CALL lbc_r2 ( ztab2d (:,:), npolj, cd_type)
      ENDIF
      !
      pt2d (:,:) = NINT ( ztab2d (:,:))
      !

      RETURN
   END SUBROUTINE lbc_i2
   !
   SUBROUTINE lbc_i1 (ptab, npolj, cd_type, ni, nj, sgn)
      IMPLICIT NONE
      !
      INTEGER (kind=il), INTENT (in) :: npolj !< Type of periodicity
      INTEGER (kind=il), INTENT (in) :: ni !< Array dimension
      INTEGER (kind=il), INTENT (in) :: nj !< Array dimension
      CHARACTER (len=*), INTENT (in) :: cd_type !< Grid specification : T, U, V or F
      INTEGER (kind=il), DIMENSION (ni * nj), INTENT (inout) :: ptab !< Input array
      INTEGER (kind=il), INTENT (in), OPTIONAL :: sgn
      REAL (kind=rl), DIMENSION (ni, nj) :: ptab2d !< Temp array
      INTEGER :: ji, jl, ijt, iju, ijpj, ipr2dj

      ptab2d (:,:) = RESHAPE (REAL(ptab,KIND=rl), (/ ni, nj /) )

      IF ( PRESENT (sgn)) THEN
         CALL lbc_r2 ( ptab2d (:,:), npolj, cd_type, REAL(sgn,KIND=rl))
      ELSE
         CALL lbc_r2 ( ptab2d (:,:), npolj, cd_type)
      END IF
      ptab (:) = RESHAPE ( NINT(ptab2d), (/ ni*nj /) )

   END SUBROUTINE lbc_i1

   SUBROUTINE lbc_l1 (ptab, npolj, cd_type, ni, nj)
      IMPLICIT NONE
      !
      INTEGER (kind=il), INTENT (in) :: npolj !< Type of periodicity
      INTEGER (kind=il) :: ni !< Array dimension
      INTEGER (kind=il) :: nj !< Array dimension
      CHARACTER (len=*), INTENT (in) :: cd_type !< Grid specification : T, U, V or F
      LOGICAL, DIMENSION (ni * nj), INTENT (inout) :: ptab !< Input array
      LOGICAL, DIMENSION (ni, nj) :: ptab2d !< Temp array
      INTEGER :: ji, jl, ijt, iju, ijpj, ipr2dj

      ptab2d (:,:) = RESHAPE (ptab, (/ ni, nj /) )

      CALL lbc_l2 ( ptab2d (:,:), npolj, cd_type)

      ptab (:) = RESHAPE ( ptab2d, (/ ni*nj /) )

   END SUBROUTINE lbc_l1

   SUBROUTINE lbc_l2 (pt2d, npolj, cd_type) 
      !> Periodicity on integer fields
      IMPLICIT NONE
      !!
      INTEGER (kind=il), INTENT (in) :: npolj !< Type of periodicity
      LOGICAL, DIMENSION (:, :), INTENT (inout) :: pt2d !< Input array
      CHARACTER (len=*), INTENT (in) :: cd_type !< Grid specification : T, U, V or F
      REAL (kind=rl), DIMENSION (SIZE(pt2d,DIM=1),SIZE(pt2d,DIM=2)) :: ztab2d !< Temp array 
      !
      ztab2d (:,:) = 0.0_rl
      WHERE ( pt2d (:,:)) ztab2d (:,:) = 1.0_rl
!-$$      ztab2d (:,:)  MERGE( 1.0_rl, 0.0_rl, pt2d (:,:) )
      !
      CALL lbc_r2 ( ztab2d (:,:), npolj, cd_type)
      !
      pt2d (:,:) = ztab2d (:,:) .GT. 0.5_rl
      !

      RETURN
   END SUBROUTINE lbc_l2

   SUBROUTINE lbc_r2 (pt2d, npolj, cd_type, sgn)
      !> Periodicity of real fields
      IMPLICIT NONE
      !
      INTEGER (kind=il), INTENT (in) :: npolj !< Type of periodicity
      REAL (kind=rl), DIMENSION (:, :), INTENT (inout) :: pt2d !< Input array
      CHARACTER (len=*), INTENT (in) :: cd_type !< Grid specification : T, U, V or F
      REAL (kind=rl), INTENT (in), OPTIONAL :: sgn
      !
      REAL (kind=rl) :: psgn
      INTEGER (kind=il) :: jpiglo !< Array dimension
      INTEGER (kind=il) :: jpjglo !< Array dimension
      INTEGER :: ji, jl, ijt, iju, ijpj, ijpjm1
      !
      IF ( PRESENT (sgn)) THEN 
         psgn = sgn
      ELSE
         psgn = 1.0_rl
      ENDIF
      !
      !
      jpiglo = SIZE ( pt2d, DIM=1)
      jpjglo = SIZE ( pt2d, DIM=2)
      !
      ijpj   = jpjglo           ! 1 proc only  along the i-direction
      ijpjm1 = ijpj-1

      IF (npolj == 0_il) RETURN
      !
      !> @par East-West boundary conditions
      ! --------------------------------

      !
      SELECT CASE (npolj)
      CASE ( 1, 4, 6 )
         ! ... cyclic
         pt2d ( 1    , :) = pt2d (jpiglo-1, :)
         pt2d (jpiglo, :) = pt2d ( 2      , :)
      END SELECT
      !
      !> @par North-South boundary conditions
      ! ----------------------------------
      !
      SELECT CASE ( npolj )
         !
      CASE ( 3 , 4 )            ! *  North fold  T-point pivot
         !
         SELECT CASE ( cd_type )
         CASE ( 'T' , 'W' )    ! T-, W-point
            DO ji = 2, jpiglo
               ijt = jpiglo-ji+2
               pt2d(ji,ijpj) = psgn * pt2d(ijt,ijpj-2)
            END DO
            pt2d(1,ijpj) = psgn * pt2d(3,ijpj-2)
            DO ji = jpiglo/2+1, jpiglo
               ijt = jpiglo-ji+2
               pt2d(ji,ijpjm1) = psgn * pt2d(ijt,ijpjm1)
            END DO
         CASE ( 'U' )          ! U-point
            DO ji = 1, jpiglo-1
               iju = jpiglo-ji+1
               pt2d(ji,ijpj) = psgn * pt2d(iju,ijpj-2)
            END DO
            pt2d(   1  ,ijpj) = psgn * pt2d(    2   ,ijpj-2)
            pt2d(jpiglo,ijpj) = psgn * pt2d(jpiglo-1,ijpj-2) 
            DO ji = jpiglo/2, jpiglo-1
               iju = jpiglo-ji+1
               pt2d(ji,ijpjm1) = psgn * pt2d(iju,ijpjm1)
            END DO
         CASE ( 'V' )          ! V-point
            DO ji = 2, jpiglo
               ijt = jpiglo-ji+2
               pt2d(ji,ijpj-1) = psgn * pt2d(ijt,ijpj-2)
               pt2d(ji,ijpj  ) = psgn * pt2d(ijt,ijpj-3)
            END DO
            pt2d(1,ijpj) = psgn * pt2d(3,ijpj-3) 
         CASE ( 'F' )          ! F-point
            DO ji = 1, jpiglo-1
               iju = jpiglo-ji+1
               pt2d(ji,ijpj-1) = psgn * pt2d(iju,ijpj-2)
               pt2d(ji,ijpj  ) = psgn * pt2d(iju,ijpj-3)
            END DO
            pt2d(   1  ,ijpj) = psgn * pt2d(    2   ,ijpj-3)
            pt2d(jpiglo,ijpj) = psgn * pt2d(jpiglo-1,ijpj-3) 
         END SELECT
         !
      CASE ( 5 , 6 )            ! *  North fold  F-point pivot
         !
         SELECT CASE ( cd_type )
         CASE ( 'T' , 'W' )    ! T-, W-point
            DO ji = 1, jpiglo
               ijt = jpiglo-ji+1
               pt2d(ji,ijpj) = psgn * pt2d(ijt,ijpj-1)
            END DO
         CASE ( 'U' )          ! U-point
            DO ji = 1, jpiglo-1
               iju = jpiglo-ji
               pt2d(ji,ijpj) = psgn * pt2d(iju,ijpj-1)
            END DO
            pt2d(jpiglo,ijpj) = psgn * pt2d(1,ijpj-1)
         CASE ( 'V' )          ! V-point
            DO ji = 1, jpiglo
               ijt = jpiglo-ji+1
               pt2d(ji,ijpj) = psgn * pt2d(ijt,ijpj-2)
            END DO
            DO ji = jpiglo/2+1, jpiglo
               ijt = jpiglo-ji+1
               pt2d(ji,ijpjm1) = psgn * pt2d(ijt,ijpjm1)
            END DO
         CASE ( 'F' )          ! F-point
            DO ji = 1, jpiglo-1
               iju = jpiglo-ji
               pt2d(ji,ijpj  ) = psgn * pt2d(iju,ijpj-2)
            END DO
            pt2d(jpiglo,ijpj) = psgn * pt2d(1,ijpj-2)
            DO ji = jpiglo/2+1, jpiglo-1
               iju = jpiglo-ji
               pt2d(ji,ijpjm1) = psgn * pt2d(iju,ijpjm1)
            END DO
         END SELECT
         !
      CASE ( -1 )
         !
         !> @par LMDz type boundary conditions
         ! 
         SELECT CASE ( cd_type)
         CASE ( 'T' )
            DO ji = 2, jpiglo
               pt2d (ji,   1   ) = pt2d (1, 1    )
               pt2d (ji, jpjglo) = pt2d (1,jpjglo)
            END DO
         END SELECT
 
         !
      END SELECT                !  npolj
      !
   END SUBROUTINE lbc_r2
   !
END MODULE mod_lbc
