! -*- Mode: f90 -*-
MODULE mod_lbc_1
   !> Periodicite des champs
   !
   !> Multiply 2D vectors
   INTERFACE lbc
      MODULE PROCEDURE lbc_r, lbc_i, lbc_l
   END INTERFACE lbc
CONTAINS
   !
   !> Periodicity of real fields
   SUBROUTINE lbc_r (ptab, kperio, cd_type, ki, kj )
      !> Periodicity of real fields
      USE declare
      IMPLICIT NONE
      !
      INTEGER (kind=il), INTENT (in) :: kperio !< Type of periodicity
      INTEGER (kind=il), INTENT (in) :: ki !< Array dimension
      INTEGER (kind=il), INTENT (in) :: kj !< Array dimension
      CHARACTER (len=1) :: cd_type !< Grid specification : T, U, V or F
      REAL (kind=rl), DIMENSION (ki * kj), INTENT (inout) :: ptab !< Input array
      REAL (kind=rl), DIMENSION (ki, kj) :: ztab !< Temp array
      !
      IF (kperio == 0_il) RETURN
      !
      ztab = RESHAPE (ptab, (/ ki, kj /) )
      !
      !> @par East-West boundary conditions
      ! --------------------------------
      !
      IF (kperio == 1_il .OR. kperio == 4_il ) THEN 
         ! ... cyclic
         ztab (1, :) = ztab (ki-1, :)
         ztab (ki, :) = ztab ( 2  , :)
      ENDIF
      !
      !> @par North-South boundary conditions
      ! ----------------------------------
      !
      IF (kperio == 3 .OR. kperio == 4) THEN 
         ! ... north fold
         !
         SELECT CASE (cd_type) 
         CASE ('T' ) 
            ztab (2     :ki  , kj  ) = ztab (ki    :2:-1, kj-2)
            ztab (ki/2+1:ki  , kj-1) = ztab (ki/2+1:2:-1, kj-1)
         CASE ('U' )
            ztab (1     :ki-1, kj  ) = ztab (ki    :2:-1, kj-2)
            ztab (ki/2  :ki-1, kj-1) = ztab (ki/2+1:2:-1, kj-1)
         CASE ('V' )
            ztab (2     :ki  , kj-1) = ztab (ki    :2:-1, kj-2)
            ztab (2     :ki  , kj  ) = ztab (ki    :2:-1, kj-3)
         CASE ('F' )
            ztab (1     :ki-1, kj  ) = ztab (ki    :2:-1, kj-3)
            ztab (1     :ki-1, kj-1) = ztab (ki    :2:-1, kj-2)
         END SELECT
         !
      ENDIF
      !
      IF (kperio == -4) THEN 
         ! ... north fold
         !
         SELECT CASE (cd_type) 
         CASE ('T' ) 
            ztab (ki/2+1:ki  , kj) = ztab (ki/2+1:2:-1, kj)
         CASE ('U' )
            ztab (ki/2  :ki-1, kj) = ztab (ki/2+1:2:-1, kj)
         CASE ('V' )
            ztab (2     :ki  , kj) = ztab (ki    :2:-1, kj-1)
         CASE ('F' )
            ztab (1     :ki-1, kj) = ztab (ki    :2:-1, kj-1)
         END SELECT
         !
      ENDIF
      !
      !> @par LMDz type boundary conditions
      !
      IF (kperio == -1_il ) THEN 
         ztab (2:ki, 1 ) = ztab (1,1)
         ztab (2:ki, kj) = ztab (1,kj)
      ENDIF
      !
      ptab = RESHAPE (ztab, (/ ki * kj /) )
      !
   END SUBROUTINE lbc_r
   !
   !!---------------------------------------------------------------------
   SUBROUTINE lbc_i (ktab, kperio, cd_type, ki, kj) 
      !> Periodicity on integer fields
      USE declare
      IMPLICIT NONE
      !!
      INTEGER (kind=il), INTENT (in) :: kperio !< Type of periodicity
      INTEGER (kind=il), INTENT (in) :: ki !< Input array dimensions
      INTEGER (kind=il), INTENT (in) :: kj !< Input array dimensions
      CHARACTER (len=1) :: cd_type !< Grid specification : T, U, V or F
      INTEGER (kind=il), DIMENSION (ki * kj), INTENT (inout) :: ktab !< Input array
      INTEGER (kind=il), DIMENSION (ki, kj) :: itab !< Temp array
      !
      IF (kperio == 0_il) RETURN
      !
      itab = RESHAPE (ktab, (/ ki, kj/) )
      !
      ! 1. East-West boundary conditions
      ! --------------------------------
      !
      IF (kperio == 1 .OR. kperio == 4 ) THEN
         ! ... cyclic
         itab (1, :) = itab (ki-1, :)
         itab (ki, :) = itab (  2 , :)
      END IF
      !
      ! 2. North-South boundary conditions
      ! ----------------------------------
      !
      IF (kperio == 3_il .OR. kperio == 4_il) THEN 
         ! ... north fold
         !
         SELECT CASE (cd_type) 
         CASE ('T')
            itab (2     :ki  , kj  ) = itab (ki    :2:-1, kj-2)
            itab (ki/2+1:ki  , kj-1) = itab (ki/2+1:2:-1, kj-1)
         CASE ('U' )
            itab (1     :ki-1, kj  ) = itab (ki    :2:-1, kj-2)
            itab (ki/2  :ki-1, kj-1) = itab (ki/2+1:2:-1, kj-1)
         CASE ('V' )
            itab (2     :ki  , kj-1) = itab (ki    :2:-1, kj-2)
            itab (2     :ki  , kj  ) = itab (ki    :2:-1, kj-3)
         CASE ('F' )
            itab (1     :ki-1, kj-1) = itab (ki    :2:-1, kj-2)
            itab (1     :ki-1, kj  ) = itab (ki    :2:-1, kj-3)
         END SELECT
         !
      ENDIF
      !
      IF (kperio == -4_il) THEN 
         ! ... north fold
         !
         SELECT CASE (cd_type) 
         CASE ('T' ) 
            itab (ki/2+1:ki  , kj) = itab (ki/2+1:2:-1, kj)
         CASE ('U' )
            itab (ki/2  :ki-1, kj) = itab (ki/2+1:2:-1, kj)
         CASE ('V' )
            itab (2     :ki  , kj) = itab (ki    :2:-1, kj-1)
         CASE ('F' )
            itab (1     :ki-1, kj) = itab (ki    :2:-1, kj-1)
         END SELECT
         !
      ENDIF
      !
      ! 3. LMDz type boundary conditions
      !
      IF (kperio == -1_il ) THEN 
         itab (2:ki, 1 ) = itab (1,1)
         itab (2:ki, kj) = itab (1,kj)
      ENDIF
      !
      ktab = RESHAPE (itab, (/ ki * kj /) )
      !
   END SUBROUTINE lbc_i
   !!
   !!---------------------------------------------------------------------
   !> Periodicity of logical fields
   SUBROUTINE lbc_l (ltab, kperio, cd_type, ki, kj ) 
      !> Periodicity of logical fields
      USE declare
      IMPLICIT NONE
      !!
      INTEGER (kind=il), INTENT (in) :: kperio !< Type of periodicity
      INTEGER (kind=il), INTENT (in) :: ki !< Input array dimensions
      INTEGER (kind=il), INTENT (in) :: kj !< Input array dimensions
      CHARACTER (len=1) :: cd_type !< Grid specification : T, U, V or F
      LOGICAL, DIMENSION (ki * kj), INTENT (inout) :: ltab !< Input array
      LOGICAL, DIMENSION (ki, kj) :: itab !< Temp array
      !
      IF (kperio == 0_il) RETURN
      !
      itab = RESHAPE (ltab, (/ ki, kj/) )
      !
      ! 1. East-West boundary conditions
      ! --------------------------------
      !
      IF (kperio == 1_il .OR. kperio == 4_il ) THEN
         ! ... cyclic
         itab (1, :) = itab (ki-1, :)
         itab (ki, :) = itab (  2 , :)
      END IF
      !
      ! 2. North-South boundary conditions
      ! ----------------------------------
      !
      IF (kperio == 3_il .OR. kperio == 4_il ) THEN 
         ! ... north fold
         !
         SELECT CASE (cd_type)
         CASE ('T' )
            itab (2     :ki  , kj  ) = itab (ki    :2:-1, kj-2)
            itab (ki/2+1:ki  , kj-1) = itab (ki/2+1:2:-1, kj-1)
         CASE ('U' )
            itab (1     :ki-1, kj  ) = itab (ki    :2:-1, kj-2)
            itab (ki/2  :ki-1, kj-1) = itab (ki/2+1:2:-1, kj-1)
         CASE ('V' )
            itab (2     :ki  , kj-1) = itab (ki    :2:-1, kj-2)
            itab (2     :ki  , kj  ) = itab (ki    :2:-1, kj-3)
         CASE('F')
            itab (1     :ki-1, kj-1) = itab (ki    :2:-1, kj-2)
            itab (1     :ki-1, kj  ) = itab (ki    :2:-1, kj-3)
         END SELECT
         !
      ENDIF
      !
      IF (kperio == -4_il) THEN 
         ! ... north fold
         !
         SELECT CASE (cd_type) 
         CASE ('T' ) 
            itab (ki/2+1:ki  , kj) = itab (ki/2+1:2:-1, kj)
         CASE ('U' )
            itab (ki/2  :ki-1, kj) = itab (ki/2+1:2:-1, kj)
         CASE ('V' )
            itab (2     :ki  , kj) = itab (ki    :2:-1, kj-1)
         CASE ('F' )
            itab (1     :ki-1, kj) = itab (ki    :2:-1, kj-1)
         END SELECT
         !
      ENDIF
      !
      ! 3. LMDz type boundary conditions
      !
      IF (kperio == -1_il) THEN 
         itab (2:ki, 1 ) = itab (1,1 )
         itab (2:ki, kj) = itab (1,kj)
      ENDIF
      !
      ltab = RESHAPE (itab, (/ ki * kj /) )
      !
   END SUBROUTINE lbc_l
END MODULE mod_lbc_1
