! -*- Mode: f90 -*-
MODULE mod_lecdata
CONTAINS
   SUBROUTINE lecdata (lecweights, leco2amask, cw_num)
      !>
      !! Read model characteristics
      !!
      !! If dummy argument lecweights is true, read also weights
      USE declare
      USE modeles
      USE mod_locio
      USE fliocom
      USE errioipsl
      USE mod_prih
      !!
      IMPLICIT NONE
      LOGICAL, INTENT (in), OPTIONAL :: lecweights, leco2amask
      CHARACTER (len=1), INTENT (in), OPTIONAL :: cw_num
      INTEGER (kind=il) :: kerr = 0
      REAL    (kind=rl), ALLOCATABLE, DIMENSION (:) :: w_t
      INTEGER (kind=il), ALLOCATABLE, DIMENSION (:) :: k_t
      REAL    (kind=rl), ALLOCATABLE, DIMENSION (:, :) :: z2a, z2o, zlon, zlat
      INTEGER (kind=il), ALLOCATABLE, DIMENSION (:, :) :: i2a, i2o
      REAL    (kind=rl), ALLOCATABLE, DIMENSION (:, :, :) :: z3a
      INTEGER (kind=il) :: nb_dim
      INTEGER (kind=il), DIMENSION (:), ALLOCATABLE :: ln_dim
      CHARACTER (len = 80) :: cfname
      REAL (kind=rl) :: ylat1, ylat2
      !! NetCDF
      INTEGER (kind=il) :: il_grid, il_mask, il_area, il_o2a, il_a2o, nco2a
      CHARACTER (len = 25) :: mo_name, o2a_name, a2o_name
      CHARACTER (len =  1) :: c_i, c_r
      CHARACTER (len=1) :: clw_num
      INTEGER (kind=il) :: ierr
      INTEGER (kind=il) :: jn
      !!
      !!
      WRITE (UNIT=nout, FMT=*) 'Lecdata-> Parametres appel '
      WRITE (UNIT=nout, FMT=*) 'lecweights : ', lecweights
      WRITE (UNIT=nout, FMT=*) 'leco2amask : ', leco2amask
      IF ( PRESENT (cw_num)) THEN
         WRITE (UNIT=nout, FMT=*) 'cw_num     : ', cw_num
      ELSE
         WRITE (UNIT=nout, FMT=*) 'cw_num     : non present '
      END IF
      WRITE (UNIT=nout, FMT=*) 'Lecdata-> Dimensions '
      WRITE (UNIT=nout, FMT=*) 'Lecdata-> ', 'jpon, jpoi, jpoj, jpoe, noperio, jpan, jpai, jpaj, jpae, naperio'
      WRITE (UNIT=nout, FMT=*) 'Lecdata-> ', jpon, jpoi, jpoj, jpoe, noperio, jpan, jpai, jpaj, jpae, naperio
      !!
      mo_name  = "mozaic"
      o2a_name = "o2a"
      a2o_name = "a2o"
      !
      IF ( PRESENT(cw_num) ) THEN 
         clw_num = cw_num
      ELSE
         clw_num='2'
      END IF
      !!
      ALLOCATE (z2a (jpai, jpaj), STAT=ierr) ; CALL chk_allo (ierr, 'z2a')
      ALLOCATE (z2o (jpoi, jpoj), STAT=ierr) ; CALL chk_allo (ierr, 'z2o')
      ALLOCATE (i2a (jpai, jpaj), STAT=ierr) ; CALL chk_allo (ierr, 'i2a')
      ALLOCATE (i2o (jpoi, jpoj), STAT=ierr) ; CALL chk_allo (ierr, 'i2o')
      ALLOCATE (z3a (jpai, jpaj,4), STAT=ierr) ; CALL chk_allo (ierr, 'z3a')
      !!  
      !! Read coordinates
      ! CALL ipsldbg (.TRUE.)
!-$$      IF (l_grid_cdf) THEN
!-$$         comod = 'orc'
!-$$      ENDIF
      !!
      IF (locerev .OR. l_grid_cdf) THEN
         comod_t = 't'//TRIM (comod)
         comod_u = 'u'//TRIM (comod)
         comod_v = 'v'//TRIM (comod)
         comod_f = 'f'//TRIM (comod)
      ELSE
         comod_t = TRIM (comod) // 't' 
         comod_u = TRIM (comod) // 'u' 
         comod_v = TRIM (comod) // 'v' 
         comod_f = TRIM (comod) // 'f' 
      ENDIF
      IF ( catyp(1:8) == 'gaussian' ) THEN
         camod_t = 'a' // TRIM (camod)
      ELSE 
         camod_t = 't' // TRIM (camod)
         camod_u = 'u' // TRIM (camod)
         camod_v = 'v' // TRIM (camod)
         camod_f = 'f' // TRIM (camod) 
      END IF
      !!
      IF (l_grid_cdf) THEN
         CALL flioopfd ('grids' // TRIM(c_suffix) // '.nc', il_grid)
         !
         CALL fliogetv (il_grid, TRIM(comod_t)//'.lon', z2o(:,:)) ; xolont(:) = RESHAPE (z2o(:,:), (/jpon/))
         CALL fliogetv (il_grid, TRIM(comod_t)//'.lat', z2o(:,:)) ; xolatt(:) = RESHAPE (z2o(:,:), (/jpon/))
         CALL fliogetv (il_grid, TRIM(camod_t)//'.lon', z2a(:,:)) ; xalont(:) = RESHAPE (z2a(:,:), (/jpan/))
         CALL fliogetv (il_grid, TRIM(camod_t)//'.lat', z2a(:,:)) ; xalatt(:) = RESHAPE (z2a(:,:), (/jpan/))

         IF ( TRIM(comod) == "orc" .OR. TRIM(comod) == "opa" ) THEN 
            CALL fliogetv (il_grid, TRIM(comod_u)//'.lon', z2o(:,:)) ; xolonu(:) = RESHAPE (z2o(:,:), (/jpon/))
            CALL fliogetv (il_grid, TRIM(comod_u)//'.lat', z2o(:,:)) ; xolatu(:) = RESHAPE (z2o(:,:), (/jpon/))
            CALL fliogetv (il_grid, TRIM(comod_v)//'.lon', z2o(:,:)) ; xolonv(:) = RESHAPE (z2o(:,:), (/jpon/))
            CALL fliogetv (il_grid, TRIM(comod_v)//'.lat', z2o(:,:)) ; xolatv(:) = RESHAPE (z2o(:,:), (/jpon/))
            CALL fliogetv (il_grid, TRIM(comod_f)//'.lon', z2o(:,:)) ; xolonf(:) = RESHAPE (z2o(:,:), (/jpon/))
            CALL fliogetv (il_grid, TRIM(comod_f)//'.lat', z2o(:,:)) ; xolatf(:) = RESHAPE (z2o(:,:), (/jpon/))
         END IF
         IF ( TRIM(camod) == "lmd" ) THEN  
!-$$            CALL fliogetv (il_grid, TRIM(camod_u)//'.lon', z2a(:,:)) ; xalonu(:) = RESHAPE (z2a(:,:), (/jpan/))
!-$$            CALL fliogetv (il_grid, TRIM(camod_u)//'.lat', z2a(:,:)) ; xalatu(:) = RESHAPE (z2a(:,:), (/jpan/))
!-$$            CALL fliogetv (il_grid, TRIM(camod_v)//'.lon', z2a(:,:)) ; xalonv(:) = RESHAPE (z2a(:,:), (/jpan/))
!-$$            CALL fliogetv (il_grid, TRIM(camod_v)//'.lat', z2a(:,:)) ; xalatv(:) = RESHAPE (z2a(:,:), (/jpan/))
            CALL fliogetv (il_grid, TRIM(camod_t)//'.clo', z3a(:,:,:))
            xaw (:) = RESHAPE ( MINVAL ( z3a(:,:,:), DIM=3), (/jpan/) )
            xae (:) = RESHAPE ( MAXVAL ( z3a(:,:,:), DIM=3), (/jpan/) )
            CALL fliogetv (il_grid, TRIM(camod_t)//'.cla', z3a(:,:,:))
            yan (:) = RESHAPE ( MAXVAL ( z3a(:,:,:), DIM=3), (/jpan/) )
            yas (:) = RESHAPE ( MINVAL ( z3a(:,:,:), DIM=3), (/jpan/) )

!-$$            WRITE (nout,*) 'LMD : xaw-xalont ', MINVAL (xaw-xalont), MAXVAL(xaw-xalont)
!-$$            CALL prihre(RESHAPE(xaw-xalont,(/jpai,jpaj/)),1.0_rl, nout)
!-$$            WRITE (nout,*) 'LMD : xae-xalont ', MINVAL(xae-xalont), MAXVAL(xae-xalont)
!-$$            CALL prihre(RESHAPE(xae-xalont,(/jpai,jpaj/)),1.0_rl, nout)
!-$$            
!-$$            WRITE (nout,*) 'LMD : yan-xalatt ', MINVAL (yan-xalatt), MAXVAL(yan-xalatt)
!-$$            CALL prihre(RESHAPE(yan-xalatt,(/jpai,jpaj/)),1.0_rl, nout)
!-$$            WRITE (nout,*) 'LMD : yas-xalatt ', MINVAL (yas-xalatt), MAXVAL(yas-xalatt)
!-$$            CALL prihre(RESHAPE(yas-xalatt,(/jpai,jpaj/)),1.0_rl, nout)
         ENDIF
         
         IF ( TRIM(comod) == "grn" .OR. TRIM(comod) == "grs" ) THEN
            DO jn = 1, 4
               CALL fliogetv (il_grid, TRIM(comod_t)//'.clo', z2o (:,:), start=(/1,1,jn/), count=(/jpoi,jpoj,1/) )
               xo_ed (:,jn) = RESHAPE (z2o(:,:), (/jpon/))
               CALL fliogetv (il_grid, TRIM(comod_t)//'.cla', z2o (:,:), start=(/1,1,jn/), count=(/jpoi,jpoj,1/) )
               yo_ed (:,jn) = RESHAPE (z2o(:,:), (/jpon/))
            END DO
         ENDIF

         IF ( TRIM(camod) == "grn" .OR. TRIM(camod) == "grs" ) THEN
            DO jn = 1, 4
               CALL fliogetv (il_grid, TRIM(camod_t)//'.clo', z2a (:,:), start=(/1,1,jn/), count=(/jpai,jpaj,1/) )
               xa_ed (:,jn) = RESHAPE (z2a(:,:), (/jpan/))
               CALL fliogetv (il_grid, TRIM(camod_t)//'.cla', z2a (:,:), start=(/1,1,jn/), count=(/jpai,jpaj,1/) )
               ya_ed (:,jn) = RESHAPE (z2a(:,:), (/jpan/))
            END DO
         ENDIF

         CALL flioclo (il_grid)

      ELSE
         WRITE (unit = cfname, FMT='("grids.r", 1I1)' ) rk_in
         OPEN (unit = ngrd, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read', status = 'old' )
         CALL locread (TRIM(comod_t)//'.lon'    , xolont, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 001
         CALL locread (TRIM(comod_t)//'.lat'    , xolatt, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 002
         CALL locread (TRIM(comod_u)//'.lon'    , xolonu, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 003
         CALL locread (TRIM(comod_u)//'.lat'    , xolatu, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 004
         CALL locread (TRIM(comod_v)//'.lon'    , xolonv, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 001
         CALL locread (TRIM(comod_v)//'.lat'    , xolatv, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 002
         CALL locread (TRIM(comod_f)//'.lon'    , xolonf, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 003
         CALL locread (TRIM(comod_f)//'.lat'    , xolatf, jpon , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 004

         IF ( catyp(1:8) == 'gaussian' ) THEN 
            CALL locread (TRIM(camod_t)//'.lon', xalont, jpan , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 021
            CALL locread (TRIM(camod_t)//'.lat', xalatt, jpan , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 022
         ELSE
            CALL locread (TRIM(camod_t)//'.lon', xalont, jpan , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 031
            CALL locread (TRIM(camod_t)//'.lat', xalatt, jpan , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 032
            CALL locread (TRIM(camod_u)//'.lon', xalonu, jpan , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 033
            CALL locread (TRIM(camod_u)//'.lat', xalatu, jpan , ngrd, kerr, nout) ; IF (kerr /= 0) STOP 034
            CALL locread (TRIM(camod_v)//'.lon', xalonv, jpanv, ngrd, kerr, nout) ; IF (kerr /= 0) STOP 035
            CALL locread (TRIM(camod_v)//'.lat', xalatv, jpanv, ngrd, kerr, nout) ; IF (kerr /= 0) STOP 036
         END IF
         !!
      ENDIF
      !!
      !! Read masks
      !! 
      IF (l_grid_cdf) THEN
         CALL flioopfd ('masks' // TRIM(c_suffix) // '.nc', il_mask)
         CALL fliogetv (il_mask, TRIM(comod_t)//'.msk', i2o(:,:)) ; iomskt(:) = RESHAPE (i2o(:,:), (/jpon/))
         CALL fliogetv (il_mask, TRIM(camod_t)//'.msk', i2a(:,:)) ; iamskt(:) = RESHAPE (i2a(:,:), (/jpan/))
         CALL flioclo (il_mask)
      ELSE
         WRITE (unit = cfname, FMT='("masks.i", 1I1)' ) ik_in
         OPEN (unit = nmsk, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read' )
         CALL locrint (TRIM(comod_t)//'.msk', iomskt, jpon , nmsk, kerr, nout) ; IF (kerr /= 0) STOP 041
         CALL locrint (TRIM(camod_t)//'.msk', iamskt, jpan , nmsk, kerr, nout) ; IF (kerr /= 0) STOP 061
         !!
      ENDIF
      !!
      !! Read grid box areas
      !!
      IF (l_grid_cdf) THEN
         CALL flioopfd ( 'areas' // TRIM(c_suffix) // '.nc', il_area)
         CALL fliogetv (il_area, TRIM(comod_t)//'.srf', z2o(:,:)) ; xosrft(:) = RESHAPE (z2o(:,:), (/ jpon /))
         CALL fliogetv (il_area, TRIM(camod_t)//'.srf', z2a(:,:)) ; xasrft(:) = RESHAPE (z2a(:,:), (/ jpan /))
         CALL flioclo (il_area)
      ELSE
         WRITE (unit = cfname, FMT='("areas.r", 1I1)' ) rk_in
         OPEN (unit = nsrf, file = TRIM(cfname), position = 'rewind', form = 'unformatted', action = 'read' )
         CALL locread (TRIM(comod_t)//'.srf', xosrft, jpon , nsrf, kerr, nout) ;  IF (kerr /= 0) STOP 081
         CALL locread (TRIM(camod_t)//'.srf', xasrft, jpan , nsrf, kerr, nout) ;  IF (kerr /= 0) STOP 101
         !!
      ENDIF
      !!
      LecWeiPresent: IF (PRESENT (lecweights)) THEN 
         LecWei: IF (lecweights) THEN 
            !!
            ko2a = 0 ; ka2o = 0 ; wa2o = 0.0_rl ; wo2a = 0.0_rl
            !!


            SELECT CASE ( TRIM ( c_read_wei) )
            CASE ( 'oasis_3' ) 
               cfname = TRIM(mo_name) // ".wa2o" // TRIM(c_suffix) // ".nc"
               WRITE (nout,*) "Lecdata:  lecture des poids a2o format cdf"
               WRITE (nout,*) "Lecdata, fichier poids : ", TRIM(cfname)
               CALL flioopfd (TRIM(cfname), il_a2o)
               CALL flioinqf (il_a2o, nb_dim=nb_dim )
               ALLOCATE ( ln_dim (nb_dim))
               CALL flioinqf (il_a2o, ln_dim=ln_dim )
               WRITE (nout,*) 'Dimensions  ', nb_dim, ln_dim
               CALL fliogetv (il_a2o, 'WEIGHTS2', wa2o(1:jma2o, 1:jpon) )
               CALL fliogetv (il_a2o, 'ADRESSE2', ka2o(1:jma2o, 1:jpon) )
               WRITE (nout,*) MINVAL (wa2o), MAXVAL(wa2o)
               WRITE (nout,*) MINVAL (ka2o), MAXVAL(ka2o)
               CALL flioclo (il_a2o)
               DEALLOCATE (ln_dim)

               cfname = TRIM(mo_name) // ".wo2a" // TRIM(c_suffix) // ".nc"
               WRITE (nout,*) "Lecdata:  lecture des poids o2a format cdf"
               WRITE (nout,*) "Lecdata, fichier poids : ", TRIM(cfname)
               CALL flioopfd (TRIM(cfname), il_o2a)
               CALL flioinqf (il_o2a, nb_dim=nb_dim )
               ALLOCATE ( ln_dim (nb_dim))
               CALL flioinqf (il_o2a, ln_dim=ln_dim )
               WRITE (nout,*) 'Dimensions  ', nb_dim, ln_dim
               
               CALL fliogetv (il_o2a, 'WEIGHTS1', wo2a(1:jmo2a, 1:jpan) )
               CALL fliogetv (il_o2a, 'ADRESSE1', ko2a(1:jmo2a, 1:jpan) )
               WRITE (nout,*) MINVAL (wo2a), MAXVAL(wo2a)
               WRITE (nout,*) MINVAL (ko2a), MAXVAL(ko2a)
               CALL flioclo (il_o2a)
               DEALLOCATE (ln_dim)
               
               !!
            CASE ( 'oasis_2' )
               WRITE (nout,*) "Lecdata:  lecture des poids format i4.r8"
               WRITE(UNIT=nout,FMT=*) 'Lecdata-> ',         'kerr, nout, nwei8o2a'
               WRITE(UNIT=nout,FMT='(" Lecdata-> ", 3I8)' )  kerr, nout, nwei8o2a
               WRITE(UNIT=nout,FMT=*) 'Lecdata-> ',         'jpon, jpa2o, jma2o, jpon*jpa2o, jpon*jma2o, SIZE(ka2o), SIZE(wa2o)'
               WRITE(UNIT=nout,FMT='(" Lecdata-> ", 7I9)' )  jpon, jpa2o, jma2o, jpon*jpa2o, jpon*jma2o, SIZE(ka2o), SIZE(wa2o)
               WRITE(UNIT=nout,FMT=*) 'Lecdata-> ',         'jpan, jpo2a, jmo2a, jpan*jpo2a, jpan*jmo2a, SIZE(ko2a), SIZE(wo2a)'
               WRITE(UNIT=nout,FMT='(" Lecdata-> ", 7I9)' )  jpan, jpo2a, jmo2a, jpan*jpo2a, jpan*jmo2a, SIZE(ko2a), SIZE(wo2a)
               
               clweight =  "WEIGHTS1" ; cladress = "ADRESSE1"
               WRITE (UNIT=nout, FMT=*) 'Lecdata -> cladress : ', cladress
               IF (ALLOCATED (k_t) ) DEALLOCATE (k_t)
               ALLOCATE (k_t(jmo2a*jpan), STAT=ierr) ; CALL chk_allo (ierr, 'k_t(jmo2a*jpan)', lreset=.TRUE., crout='lecdata')
               CALL locrint (cladress, k_t(1:jmo2a*jpan), jmo2a*jpan, nwei8o2a, kerr, nout) ;  IF (kerr /= 0) STOP 055
               ko2a (1:jmo2a, 1:jpan) = RESHAPE (k_t (1: jpan * jmo2a), (/ jmo2a, jpan /))
               DEALLOCATE (k_t)
               WRITE (UNIT=nout, FMT=*) 'Lecdata -> clweight : ', clweight
               IF (ALLOCATED (w_t) ) DEALLOCATE (w_t)
               ALLOCATE (w_t (jmo2a*jpan), STAT=ierr) ; CALL chk_allo (ierr, 'w_t(jmo2a*jpan)')
               CALL locread (clweight, w_t(1:jmo2a*jpan), jmo2a*jpan, nwei8o2a, kerr, nout) ;  IF (kerr /= 0) STOP 056
               wo2a (1:jmo2a, 1:jpan) = RESHAPE (w_t (1: jpan * jmo2a), (/ jmo2a, jpan /))
               DEALLOCATE (w_t)
               !!  
               clweight =  "WEIGHTS"//clw_num ; cladress = "ADRESSE"//clw_num
               IF (ALLOCATED (k_t) ) DEALLOCATE (k_t)
               ALLOCATE (k_t (jma2o*jpon), STAT=ierr) ; CALL chk_allo (ierr, 'k_t(jma2o*jpon)')
               WRITE (UNIT=nout, FMT=*) 'Lecdata-> ', cladress
               CALL locrint (cladress, k_t(1:jma2o*jpon), jma2o*jpon, nwei8a2o, kerr, nout) ;  IF (kerr /= 0) STOP 201
               ka2o (1:jma2o, 1:jpon) = RESHAPE (k_t (1:jpon * jma2o) , (/ jma2o, jpon /))
               DEALLOCATE (k_t)  
               WRITE (UNIT=nout, FMT=*) clweight
               IF (ALLOCATED (w_t) ) DEALLOCATE (w_t)
               ALLOCATE (w_t (jma2o*jpon), STAT=ierr) ; CALL chk_allo (ierr, 'w_t(jma2o*jpon)')
               CALL locread (clweight, w_t(1:jma2o*jpon), jma2o*jpon, nwei8a2o, kerr, nout) ;  IF (kerr /= 0) STOP 202
               wa2o (1:jma2o, 1:jpon) = RESHAPE (w_t (1:jpon * jma2o) , (/ jma2o, jpon /))
               DEALLOCATE (w_t)
               CLOSE (unit = nwei8a2o)
            CASE Default
               WRITE (nout, *) 'Mauvais cas de lecture des poids '
               STOP
            END SELECT
            !!
         ENDIF LecWei
      END IF LecWeiPresent
      !!
      IF (PRESENT (leco2amask) ) THEN 
         IF (leco2amask) THEN 
            !
            ALLOCATE (zlon(jpai,jpaj) , zlat(jpai,jpaj) )
            CALL flioopfd ('o2a.diag' // TRIM(c_suffix) // '.nc', nco2a)
            CALL fliogetv (nco2a , 'OceMask', z2a)
            CALL fliogetv (nco2a , 'nav_lon', zlon)
            CALL fliogetv (nco2a , 'nav_lat', zlat)
            ! Faut-il retourner o2amask ?
            ylat1 = zlat(1,   1) ;  ylat2 = zlat(1,jpaj)
            WRITE (nout,*) 'zlon, zlat : ', ylat1, ylat2
            IF ( ABS (ylat1) .LT. 80. .OR. ABS (ylat2) .LT. 80. ) THEN
               WRITE (nout, *) 'Erreur latitude dans o2amask) ' , ylat1, ylat2
               STOP
            END IF
            IF (     ylat1 .LT. ylat2 .AND. .NOT. la_nortop &
               .OR.  ylat1 .GT. ylat2 .AND.       la_nortop ) THEN
               WRITE (nout,*) 'Retournement o2amask '
               WRITE (nout,*) 'Retournement'
               z2a (1:jpai, 1:jpaj) = z2a (1:jpai, jpaj:1:-1)
               WRITE (nout,*) 'Reshape'
               o2amask = RESHAPE (z2a (1:jpai, 1:jpaj), (/ jpan/) )
               WRITE (nout,*) 'Fin Reshape'
            ELSE
               WRITE (nout,*) 'Pas de retournement o2amask '
               o2amask = RESHAPE (z2a (1:jpai, 1:jpaj), (/ jpan/) )
               WRITE (nout,*) 'Fin Reshape'
            END IF
       
            o2amask_i_int = 1
            o2amask_i_ext = 1
            WHERE ( o2amask > 0.0_rl        ) o2amask_i_int = 0
            WHERE ( o2amask > 1.0_rl-epsfrac) o2amask_i_ext = 0
            CALL flioclo  (nco2a)

            DEALLOCATE (zlon ,zlat)
         END IF
      END IF
      !!
      DEALLOCATE (z2o)
      DEALLOCATE (z2a)
      DEALLOCATE (i2o)
      DEALLOCATE (i2a)
      !!
      WRITE (UNIT=nout, FMT=*) 'Fin Lecdata'
      RETURN
      !!
   END SUBROUTINE lecdata
END MODULE mod_lecdata
