! -*- Mode: f90 -*-
MODULE mod_locio
 !> Read operations on fortran binary files, oasis format
CONTAINS
   SUBROUTINE locread (cdfldn, pfield, kdimax, knulre, kflgre, kout)
      !> Read real in fortran binary files, oasis format
      !
      !     Purpose:
      !     -------
      !     Find string cdfldn on unit knulre and read array pfield
      !
      !     Reference:
      !     ---------
      !     See OASIS manual (1995) 
      !
      !     History:
      !     -------
      !       Version   Programmer     Date      Description
      !       -------   ----------     ----      -----------  
      !       2.0       L. Terray      95/09/01  created
      !
      ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      !
      USE declare 
      !!
      IMPLICIT NONE
      !
      CHARACTER (len = 8), INTENT ( in) :: cdfldn ! Name of field to search for
      INTEGER (kind=il), INTENT (  in) :: kdimax, knulre    ! Field dimension, unit to read
      INTEGER (kind=il), INTENT (  in) :: kout              ! Standard output
      INTEGER (kind=il), INTENT ( out) :: kflgre            ! Error status code
      REAL (kind=rl)  , INTENT ( out), DIMENSION ( :) :: pfield ! Field
      REAL (kind=rk_in), DIMENSION ( :), ALLOCATABLE :: ztemp
      INTEGER (kind=il) :: ki
      !
      CHARACTER (len = 8) clecfl
      INTEGER (kind=il) :: kerr
      !
      !*    1. Initialization
      !        --------------
      !
      WRITE (UNIT = kout, FMT = '( " Locrint :  Read field ", A8, " in unit = ", 1I3, " Dim : ", 1I8)' ) cdfldn, knulre, kdimax
      kerr = 0 ; ki = -1
      !
      !     2. Find field in file
      !        ------------------
      IF ( ALLOCATED (ztemp)) DEALLOCATE ( ztemp)
      ALLOCATE (ztemp ( 1: kdimax), STAT = kerr )
      IF ( kerr /= 0 ) THEN 
         WRITE ( kout, *) 'Error allocating ztemp in locread : ', kerr
         kflgre = kerr
         RETURN
      END IF
      !
      REWIND ( UNIT = knulre)
200   CONTINUE
      !* Find string
      READ (UNIT = knulre, ERR = 200, END = 210) clecfl
      IF ( clecfl /= cdfldn ) GO TO  200
      !* Read associated field
      WRITE ( unit = kout, fmt = *) 'Field found : ', clecfl
      !$$$    READ (UNIT = knulre, IOSTAT = kerr, ERR = 210, END = 210 ) ztemp (1: kdimax)
      !$$$    READ (UNIT = knulre, ERR = 210, END = 210 ) ztemp (1: kdimax)
      ki = -2
      READ (UNIT = knulre, ERR = 210, END = 210 ) (ztemp (ki), ki = 1, kdimax)
      IF ( kerr == 0 ) THEN 
         !* Reading done and ok
         pfield (1: kdimax) = REAL ( ztemp (1: kdimax), KIND = rl )
         kflgre = 0
      ELSE 
         WRITE ( unit = kout, fmt = *) 'Problem reading field ', cdfldn, kdimax, ki
         kflgre = 1
      END IF
      GO TO 220
      !* Problem in reading
210   kflgre = 1
      WRITE ( unit = kout, fmt = *) 'Problem in reading ', cdfldn, kdimax, ki
220   CONTINUE
      !
      DEALLOCATE ( ztemp)
      !
      !*    3. End of routine
      !        --------------
      !
      RETURN
   END SUBROUTINE locread
  !
   SUBROUTINE locrint ( cdfldn, kfield, kdimax, knulre, kflgre, kout)
      !> Read integer field in fortran binary files, oasis format
      !
      !     Purpose:
      !     -------
      !     Find string cdfldn on unit knulre and read integer array pfield
      !
      !     Reference:
      !     ---------
      !     See OASIS manual (1995) 
      !
      !     History:
      !     -------
      !       Version   Programmer     Date      Description
      !       -------   ----------     ----      -----------  
      !       2.0       L. Terray      95/09/01  created
      !
      ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      !
      USE declare
      !!
      IMPLICIT none
      !
      CHARACTER (LEN = 8), INTENT (in) :: cdfldn ! Name of field to search for
      INTEGER (kind=il), INTENT ( in) :: kdimax, knulre    ! Field dimension, unit to read
      INTEGER (kind=il), INTENT ( in) :: kout              ! Standard output
      INTEGER (kind=il), INTENT (out) :: kflgre            ! Error status code
      INTEGER (kind=il), INTENT (inout), DIMENSION (:) :: kfield ! Field
      INTEGER (kind=ik_in), DIMENSION ( :), ALLOCATABLE :: itemp
      INTEGER (kind=il) :: ji
      !
      CHARACTER (len = 8) :: clecfl
      INTEGER (kind=il) :: kerr
      !
      !*    1. Initialization
      !        --------------
      !
      WRITE (UNIT = kout, FMT = '( " Locrint :  Read field ", A8, " in unit = ", 1I3, " Dim : ", 1I8)' ) cdfldn, knulre, kdimax
      !
      !     2. Find field in file
      !        ------------------
      IF ( ALLOCATED ( itemp)) DEALLOCATE ( itemp)
      ALLOCATE ( itemp ( 1: kdimax), STAT = kerr )
      IF ( kerr /= 0 ) THEN 
         WRITE ( kout, *) 'Error allocating itemp in locrint : ', kerr
         kflgre = kerr
         RETURN
      END IF
      !
      REWIND ( UNIT = knulre )
200   CONTINUE
      !
      !* Find string
      READ (UNIT = knulre, ERR = 200, END = 210) clecfl
      IF ( clecfl /= cdfldn ) GO TO  200
      !* Read associated field
      WRITE (UNIT = kout, FMT= *)  clecfl, ' found'
      !$$$    READ (UNIT = knulre, END = 210, IOSTAT = kerr) (itemp (ji), ji = 1, kdimax)
      READ (UNIT = knulre, END = 210) (itemp (ji), ji = 1, kdimax)
      IF ( kerr == 0 ) THEN 
         !* Reading done and ok
         kfield (1: kdimax) = INT ( itemp (1: kdimax), KIND = il)
         kflgre = 0
      ELSE 
         WRITE ( unit = kout, fmt = *) 'Problem reading field ', cdfldn, kdimax, ji
         kflgre = 1
      END IF
      GO TO 220
      !* Problem in reading
210   kflgre = 1
      WRITE ( unit = kout, fmt = *) 'Problem in reading ', cdfldn, kdimax, ji
      !
220   CONTINUE
      !
      DEALLOCATE ( itemp)
      !
      !*    3. End of routine
      !        --------------
      !
      RETURN
   END SUBROUTINE locrint
  !!
END MODULE mod_locio
