! -*- Mode: f90 -*-
MODULE mod_neuf
CONTAINS
   SUBROUTINE neuf
      !>
      !! One ocean grid box sea an atmospheric grid boxes if it is above one
      !! point T, u, v or f :
      !!    sketch of an ocean grid box
      !!
      !!       f(ji-1,jj)--------v(ji,jj)-----------f(ji,jj)
      !!                |                           |
      !!       u(ji-1,jj)        T(ji,jj)           u(ji,jj)
      !!                |                           |
      !!     f(ji-1,jj-1)-------v(ji,jj-1)----------f(ji,jj-1)
      !!
      !!
      !! Original     : Olivier MARTI, juin 96
      !!
      USE declare
      USE modeles
      USE formula
      !!
      IMPLICIT NONE
      !!
      INTEGER (kind=il) :: ja, jo, jai, jaj, joi, joj, jn ! Dummy indexes
      !!
      !! Generation of weights
      !!
      ! Initialisation
      wo2a = zero ; ko2a = 0 ; wa2o = zero ; ka2o = 0 ; nva = 0 ; nvo = 0 ; nborda = 0 ; nbordo = 0
      WRITE ( unit = nout, fmt = *)
      WRITE ( unit = nout, fmt = *) 'Number of points to compute : ', jpan
      Seek_atmos: DO ja = 1, jpan ! Loop over atmospheric grid points
         IF ( iamskt ( ja) == 1 .AND. .NOT. lmaska ) THEN ! This point is not considered
            nva  ( ja)   = -1
            ko2a (ja, 1) =  0
            CYCLE  Seek_atmos
         ENDIF
         jai = MOD ( ja - 1, jpai) + 1 ; jaj = ( ja - 1) / jpai + 1   ! 2D index of atm box
         Seek: DO jo = 1, jpon
            ! 
            joi = MOD ( jo - 1, jpoi) + 1 ; joj = ( jo - 1) / jpoi + 1 ! Indices 2D
            IF ( joi == 1 .OR. joi == jpoi .OR. joj == 1 .OR. joj == jpoj ) CYCLE Seek ! 
            IF (  iomskt (jo) .EQ. 1 .AND. .NOT. lmasko ) THEN  ! This point is not considered
               nborda ( ja) = 1
               CYCLE Seek
            END IF
            nsum = 0
            IF( dedans(xolont(jo       ),xaw(ja),xae(ja)) .AND. dedans(xolatt(jo       ),yas(ja),yan(ja))) nsum=nsum+4 ! Point T
            IF( dedans(xolonu(jo       ),xaw(ja),xae(ja)) .AND. dedans(xolatu(jo       ),yas(ja),yan(ja))) nsum=nsum+2 ! Point u Right
            IF( dedans(xolonu(jo-1     ),xaw(ja),xae(ja)) .AND. dedans(xolatu(jo-1     ),yas(ja),yan(ja))) nsum=nsum+2 ! Point u Left
            IF( dedans(xolonv(jo       ),xaw(ja),xae(ja)) .AND. dedans(xolatv(jo       ),yas(ja),yan(ja))) nsum=nsum+2 ! Point v Upper
            IF( dedans(xolonv(jo-jpoi  ),xaw(ja),xae(ja)) .AND. dedans(xolatv(jo-jpoi  ),yas(ja),yan(ja))) nsum=nsum+2 ! Point v Lower
            IF( dedans(xolonf(jo       ),xaw(ja),xae(ja)) .AND. dedans(xolatf(jo       ),yas(ja),yan(ja))) nsum=nsum+1 ! Point f UR
            IF( dedans(xolonf(jo-1     ),xaw(ja),xae(ja)) .AND. dedans(xolatf(jo-1     ),yas(ja),yan(ja))) nsum=nsum+1 ! Point f UL
            IF( dedans(xolonf(jo-jpoi  ),xaw(ja),xae(ja)) .AND. dedans(xolatf(jo-jpoi  ),yas(ja),yan(ja))) nsum=nsum+1 ! Point f LR
            IF( dedans(xolonf(jo-jpoi-1),xaw(ja),xae(ja)) .AND. dedans(xolatf(jo-jpoi-1),yas(ja),yan(ja))) nsum=nsum+1 ! Point f LL 
            !! 
            Trouve: IF ( nsum /= 0 ) THEN ! This ocean point is under the atm point
               IF ( nva ( ja) >= jpo2a ) THEN
                  WRITE ( unit = nout, fmt = *) 'Not enough ocean neighbors in ', jai, jaj, xalont(ja),xalatt(ja)
                  EXIT Seek
               END IF
               nva ( ja) = nva ( ja) + 1
               jmo2a = MAX ( jmo2a, nva ( ja))
               ko2a ( ja, nva ( ja)) = jo 
               ! Poids = surface ponderee par le nombre de points
               wo2a ( ja, nva ( ja)) = REAL ( nsum, kind = rl) / 16.0 * xosrft ( jo)
               IF ( nvo ( jo) >= jpa2o ) THEN
                  WRITE ( unit = nout, fmt = *) 'Not enough atm neighbors in ', joi, joj, xolont ( jo), xolatt ( jo)
                  EXIT Seek
               ENDIF
               nvo ( jo) = nvo ( jo) + 1
               jma2o = MAX ( jma2o, nvo ( jo))
               ka2o ( jo, nvo ( jo)) = ja
               wa2o ( jo, nvo ( jo)) = REAL ( nsum, kind = rl) / 16.0 * xosrft ( jo)
            END IF Trouve
         END DO Seek
      END DO Seek_atmos
      !!
      !! Correction a la main (mers fermes )
      !! grep maille pts_pbs_mozaic | awk '{print "  CALL correct(",$3,",",$4,",",$7,",",$8,")"}'
      !!
      !! Check points isoles
      !!
      WRITE ( nout, *) 'Atmos point with no oce point below : i, j, lon, lat, west, est, south, north '
      DO ja = 1, jpan
         IF ( nva ( ja) == 0 .AND. iamskt ( ja) == 0 ) THEN 
            jai = m2ai  ( ja) ; jaj = m2aj ( ja) ! Indices 2D de la maille atm
            WRITE ( unit = nout, fmt = '(2I4,6F8.1)' ) jai, jaj, xalont ( ja), xalatt ( ja), xaw ( ja), &
               & xae ( ja), yas ( ja), yan ( ja)
         END IF
      END DO
      WRITE ( nout, *) 'Oce point with no atm point above : i, j, lon, lat '
      DO jo = 1, jpon
         IF  ( nvo ( jo) == 0 .AND. iomskt ( jo) == 0 ) THEN
            joi = MOD ( jo - 1, jpoi) + 1 ; joj = ( jo - 1) / jpoi + 1 ! Indices 2D
            IF ( joi == 1 .OR. joi == jpoi ) EXIT
            WRITE ( unit = nout, fmt =  '(2I4,6F8.1)') joi, joj, xolont ( jo), xolatt ( jo)
         ENDIF
      END DO
      ! check for some points
      DO jai = 40, 40
         jaj = 24
         ja = m1a ( jai, jaj)
         WRITE ( unit = nout, fmt = *) ' '
         WRITE ( unit = nout, fmt = *) ' Check of atm point : ', jai,jaj, xalont(ja),xalatt(ja),nva(ja),iamskt(ja)
         DO jn = 1, nva ( ja)
            jo = ko2a ( ja, jn)
            WRITE ( unit = nout, fmt = *) '  point oce ',jn,jo,xolont ( jo), xolatt ( jo),iomskt(jo),wo2a(ja,jn)
         END DO
         WRITE ( unit = nout, fmt = *) ' sum ', SUM ( wo2a ( ja,:))
         WRITE ( unit = nout, fmt = *) ' '
      END DO
      !!
      !! Sortie visuelle du nb de points sous les mailles atmos
      WRITE( unit = ndeb, fmt = * ) ' Nomber of point under atmos box - nva'
      WRITE( unit = ndeb, fmt = '(96I2)' ) nva
      !!
      WRITE ( unit = nout, fmt = *) ' '
      WRITE ( unit = nout, fmt = *) ' jma2o : ', jma2o
      WRITE ( unit = nout, fmt = *) ' jmo2a : ', jmo2a
      WRITE ( unit = nout, fmt = *) ' maxi of nvo : ', MAXVAL ( nvo)
      WRITE ( unit = nout, fmt = *) ' maxi of nva : ', MAXVAL ( nva)
      !!
   END SUBROUTINE neuf
  !!
END MODULE mod_neuf
