! -*- Mode: f90 -*-
MODULE mod_norma
CONTAINS
   SUBROUTINE norma (knorma2o, knormo2a ) 
      !>
      !! Normalize weights
      !!
      !! Ocean to atmosphere : intensive variables (i.e. temperature). Constant field should stay constant
      !!
      !! Atmosphere to ocean : extensive variables (fluxes) . Should conserve integral of fields
      !!
      USE declare
      USE modeles
      !!
      IMPLICIT NONE
      !!
      INTEGER (kind=il), INTENT (in) :: knorma2o, knormo2a
      REAL    (kind=rl) :: zzz, zrplus, zrminus
      REAL    (kind=rd) :: zzd
      INTEGER (kind=il) :: ja, jo, jn
      REAL    (kind=il), DIMENSION (jpan) :: zsuma
      REAL    (kind=il), DIMENSION (jpon) :: zsumo
      !!
      WRITE (unit = nout, fmt = *) 'Normalisation atm : ', knormo2a
      WRITE (unit = nout, fmt = *) 'Normalisation oce : ', knorma2o
      zzz = 0.0_rl ; zsuma = 0.0_rl ; zsumo = 0.0_rl ; zzd = 0.0_rd
      zrplus  = 1.0_rl +  2.0_rl*EPSILON(1.0_rl)
      zrminus = 1.0_rl -  2.0_rl*EPSILON(1.0_rl)         
      WRITE(nout,fmt='("facteur correction ", 3(1PE23.15))' ) zrplus, zrplus-1.0_rd, zrminus, zrminus-1.0_rd
      !!
      Normalize_A2O: IF (lwra2o ) THEN
         IF      (knorma2o == 0) THEN 
            WRITE (unit = nout, fmt = *) 'Normalisation atmosphere -> ocean : none'
            ! --------------
         ELSE IF (knorma2o == 1) THEN 
            WRITE (unit = nout, fmt = *) 'Normalisation atmosphere -> ocean : intensive'
            nsum = 0
            DO jo = 1, jpon
               zzd = SUM (REAL (wa2o(:,jo), rd) )
               IF (zzd /= 0.0_rd) THEN
                  wa2o (:,jo) = REAL (REAL (wa2o(:,jo),rd) / zzd, rl)
                  zzd = SUM (REAL(wa2o(:,jo), rd))
                  IF (  zzd > 1.0_rd ) THEN 
                     !$$$                     WRITE (unit = nout, fmt = '("Phase 1 Pb 1 with normalisation a2o : ", 1I8, 2(1PE15.5))' ) &
                     !$$$                        jo, zzd, zzd-1.0_rd
                     wa2o(:,jo) = wa2o(:,jo) * zrminus
                  END IF
                  IF (zzd < 1.0-rd-epsfrac  ) THEN 
                     !$$$                     WRITE (unit = nout, fmt = '("Phase 1 Pb 1 with normalisation a2o : ", 1I8, 2(1PE15.5))' ) &
                     !$$$                        jo, zzd, zzd-1.0_rd
                     wa2o(:,jo) = wa2o(:,jo) * zrplus
                  END IF
                  !$$$               ELSE 
                  !$$$                  IF (o2amask(ja) > 0.0_rl) THEN 
                  !$$$                     WRITE (unit = nout, fmt = '("Phase 1, Pb 2 with normalisation a2o  : ", 1I8, 2(1PE25.6))' ) &
                  !$$$                        & ja, zzd, o2amask(ja)
                  !$$$                  ENDIF
               END IF
            END DO
            DO jo = 1, jpon
               zzd = SUM (REAL (wa2o(:,jo), rd) )
               IF ( zzd /= 0.0_rd .AND. (zzd < 1.0_rd-epsfrac .OR.  zzd > 1.0_rd+epsd) ) THEN 
                  WRITE (unit = nout, fmt = '("Phase 2, Pb with normalisation : ", 1I8, 2(1PE15.5))' ) &
                     jo, zzd, zzd-1.0_rd
               END IF
            END DO
            ! ------------
         ELSE IF (knorma2o == 2) THEN 
            WRITE (unit = nout, fmt = *) 'Normalisation atmosphere -> ocean : extensive'
            WRITE (unit = nout, fmt = *) 'Cumulating surfaces'
            zsuma = 0.0_rl
            DO jo = 1, jpon
               DO jn = 1, jma2o
                  ja = ka2o(jn,jo)
                  IF (ja > 0 ) zsuma(ja) = zsuma(ja) + xosrft(jo) * wa2o(jn,jo) * REAL(1-iomskt(jo),kind=rl) &
                     &  * REAL(1-iomskp(jo),kind=rl)
               END DO
            END DO
            !!
            WRITE (unit = nout, fmt = *) 'Computes surface fraction'
            DO ja = 1, jpan
               zzz = o2amask(ja)
               IF (zzz > epsfrac .AND. xasrft(ja) .GE. epsfrac) THEN
                  zzz = zzz * xasrft(ja)
                  zsuma(ja) = zsuma(ja) / zzz
               ELSE
                  zsuma(ja)  = 0.0_rl
               ENDIF
               IF (ja == jpan-jpai+1 ) WRITE(nout,*) 'c ', ja, o2amask(ja), zzz, zsuma(ja), xasrft(ja)
            END DO
            !!
            WRITE (unit = nout, fmt = *) 'Normalize weights'
            DO jo = 1, jpon
               DO jn = 1, jma2o
                  ja = ka2o(jn,jo)
                  IF (ja > 0 ) THEN 
                     zzz = zsuma(ja)
                     IF (zzz /= 0.0_rl) THEN 
                        wa2o(jn,jo) = wa2o(jn,jo) / zzz
                     ELSE
                        wa2o(jn,jo) = 0.0_rl
                     END IF
                  END IF
               END DO
            END DO
         ELSE
            WRITE (unit = nout, fmt = *) 'Normalisation atmosphere -> ocean '
            STOP 002
         END IF
      ELSE
         WRITE (unit = nout, fmt = *) 'No normalisation atmosphere -> ocean'
      END IF Normalize_A2O
      !! ----
      Normalize_O2A: IF (lwro2a ) THEN 
         IF      (knormo2a == 0 ) THEN 
            WRITE (unit = nout, fmt = *) 'Normalisation ocean -> atmosphere : none'
         ELSE IF (knormo2a == 1 ) THEN
            WRITE (unit = nout, fmt = *) 'Normalisation ocean -> atmosphere : intensive'
            nsum = 0
            DO ja = 1, jpan
               zzd = SUM (REAL (wo2a(:,ja), rd) )
               IF (zzd /= 0.0_rd) THEN
                  wo2a (:,ja) = REAL (REAL (wo2a(:,ja),rd) / zzd, rl)
                  zzd = SUM (REAL(wo2a(:,ja), rd))
                  IF (zzd < 1.0_rd-epsfrac  ) THEN 
                     !$$$                     WRITE (unit = nout, fmt = '("Phase 1 Pb 1 with normalisation : ", 1I8, 2(1PE15.5))' ) &
                     !$$$                        ja, zzd, zzd-1.0_rd
                     wo2a(:,ja) = wo2a(:,ja) * zrplus
                  END IF
                  IF (  zzd > 1.0_rd ) THEN 
                     !$$$                     WRITE (unit = nout, fmt = '("Phase 1 Pb 1 with normalisation : ", 1I8, 2(1PE15.5))' ) &
                     !$$$                        ja, zzd, zzd-1.0_rd
                     wo2a(:,ja) = wo2a(:,ja) * zrminus
                  END IF
               ELSE 
                  IF (o2amask(ja) > 0.0_rl) THEN 
                     WRITE (unit = nout, fmt = '("Phase 1, Pb 2 with normalisation : ", 1I8, 2(1PE25.6))' ) &
                        & ja, zzd, o2amask(ja)
                  ENDIF
               END IF
            END DO
            DO ja = 1, jpan
               zzd = SUM (REAL (wo2a(:,ja), rd) )
               IF ( zzd /= 0.0_rd .AND. (zzd < 1.0_rd-epsfrac .OR.  zzd > 1.0_rd+epsd) ) THEN 
                  WRITE (unit = nout, fmt = '("Phase 2, Pb with normalisation : ", 1I8, 2(1PE15.5))' ) &
                     ja, zzd, zzd-1.0_rd
               END IF
            END DO

         ELSE IF (knormo2a == 2 ) THEN 
            WRITE (unit = nout, fmt = *) 'Normalisation ocean -> atmosphere : extensive'
            WRITE (unit = nout, fmt = *) 'Cumulating surfaces'
            zsumo = 0.0_rl
            DO ja = 1, jpan
               DO jn = 1, jmo2a
                  jo = ko2a(jn,ja)
                  !$$$                IF (jo == jpon-jpoi+1 ) WRITE(nout,*) 'o ', ja, jn, jo, xasrft(ja), wo2a(jn,ja), iamskt(ja), iamskp(ja), a2omask(jo)  
                  IF (jo > 0 ) zsumo(jo) = zsumo(jo) + xasrft(ja) * o2amask(ja) * wo2a(jn,ja) * REAL(1-iamskt(ja),kind=rl) &
                     &  * REAL(1-iamskp(ja),kind=rl)
               END DO
            END DO
            !!
            WRITE (unit = nout, fmt = *) 'Computes surface fraction'
            DO jo = 1, jpon
               IF (iomskt(jo) == 0 .AND. xosrft(jo) .GE. epsfrac ) THEN
                  zsumo(jo) = zsumo(jo) / xosrft(jo)
               ELSE
                  zsumo(jo)  = 0.0_rl
               ENDIF
               IF (jo == jpon-jpoi+1 ) WRITE(nout,*) 'c ', jo, zsumo(jo), xosrft(jo)
            END DO
            !!
            WRITE (unit = nout, fmt = *) 'Normalize weights'
            DO ja = 1, jpan
               DO jn = 1, jmo2a
                  jo = ko2a(jn,ja)
                  !$$$                IF (jo == jpon-jpoi+1 ) WRITE(nout,*) 'd ', jo, ja, jn, a2omask(jo), zsumo(jo), xosrft(jo), wo2a(jn,ja)
                  IF (jo > 0 ) THEN 
                     zzz = zsumo(jo)
                     IF (zzz  /= 0.0_rl ) THEN 
                        wo2a(jn,ja) = wo2a(jn,ja) / zzz 
                     ELSE
                        wo2a(jn,ja) = 0.0_rl
                     END IF
                  END IF
               END DO
            END DO
         ELSE
            WRITE (unit = nout, fmt = *) 'Normalisation ocean -> atmosphere '
            STOP 004
         END IF
      ELSE
         WRITE (unit = nout, fmt = *) 'No normalisation ocean -> atmosphere'
      END IF Normalize_O2A
      !!
      RETURN
   END SUBROUTINE norma
END MODULE mod_norma

