! -*- Mode: f90 -*-
SUBROUTINE norma
!!!
!!! Normalize weights
!!!
!!! Ocean to atmosphere : intensive variables (i.e. temperature). Constant field should stay constant
!!!
!!! Atmosphere to ocean : extensive variables (fluxes) . Should conserve integral of fields
!!!
  USE declare
  USE modeles
  !!
  IMPLICIT NONE
  !!
  REAL (kind=rl) :: zzz, zsum
  INTEGER (kind=il) :: ja, jo, jn, ko, kom
  INTEGER (kind=il), DIMENSION ( jpo2a, 2) :: jxo
  !!
  IF ( lwra2o ) THEN
      WRITE ( unit = nout, fmt = *) 'Normalisation atmosphere -> ocean'
      !! Normalisation is OK for extensive variables (Heat flux, ....)
      CALL flush(nout)
      DO ja = 2, jpan
        zzz = zero
        kom = 0
        IF ( MOD ( ja,  20) == 0 ) WRITE ( UNIT = nout, FMT = '( 1I6)', ADVANCE = 'NO' ) ja
        IF ( MOD ( ja, 100) == 0 ) WRITE ( UNIT = nout, FMT = *)
        DO jo = 2, jpon
          DO jn = 1, jpa2o
            IF (  ka2o ( jo, jn) == ja .AND. iomskp ( jo) < 0.5 .AND. ( lmasko .OR. iomskt ( jo) == 0 ) ) THEN 
                zzz = zzz + xosrft ( jo) * wa2o ( jo, jn)
                kom = kom + 1
                IF ( kom > jpo2a ) THEN
                    WRITE ( unit = nout, fmt = '("Too many ocean points : ", 6I6) ') ja, jo, jn, kom, ka2o ( jo, jn)
                END IF
                jxo ( kom, :) = (/ jo, jn /)
                IF ( ndebug >= 2 ) THEN
                    WRITE ( unit = nout, fmt = '(6I6,16F18.6) ') ja, jo, jn, kom, jxo ( kom, :), &
                       wa2o ( jo, jn), wa2o ( jxo ( kom, 1), jxo ( kom, 2)), zzz, &
                       xosrft ( jo), xasrft ( ja)
                END IF
            END IF
          END DO
        ENDDO
        !!
        IF ( (1.0_rl - o2amask ( ja)) >= epsfrac ) THEN
            zzz = zzz / ( xasrft ( ja) * (1.0_rl - o2amask ( ja)) )
        ELSE
            zzz = 0.0_rl
        ENDIF
        !!
        IF ( kom /= 0 ) THEN 
            DO ko = 1, kom
              IF ( zzz == 0.0_rl ) THEN 
                  wa2o ( jxo ( ko, 1), jxo ( ko, 2)) = 0.0_rl
              ELSE
                  wa2o ( jxo ( ko, 1), jxo ( ko, 2)) = wa2o ( jxo ( ko, 1), jxo ( ko, 2)) / zzz 
              END IF
              IF ( ndebug >= 2 ) THEN 
                  WRITE ( unit = nout, fmt = '(4I6,16(1PE15.3)) ') ja, ko, jxo ( ko, :), &
                     wa2o ( jxo ( kom, 1), jxo ( ko, 2)), zzz, xosrft ( jxo ( ko, 1)), xasrft ( ja)
              END IF
            END DO
        END IF
      ENDDO
  ELSE
      WRITE ( unit = nout, fmt = *) 'No normalisation atmosphere -> ocean'
  END IF
  IF ( lwro2a ) THEN 
      WRITE ( unit = nout, fmt = *) 'Normalisation ocean -> atmosphere'
      !! Normalisation is OK for intensives variables (Temperature, ...)
      CALL flush(nout)
      nsum = 0
      DO ja = 1, jpan
        zsum = SUM ( wo2a ( ja, :) )
        IF ( zsum /= 0 ) THEN
            wo2a ( ja, :) = wo2a ( ja, :) / zsum
        END IF
        zsum = SUM ( wo2a ( ja, :))
        IF ( ABS ( zsum) > zeps .AND. ABS ( zsum - 1.0) > zeps ) THEN 
            WRITE ( unit = nout, fmt = '("Pb with normalisation : ", 1I6, 1(1PE15.3))' ) jo, zsum
        END IF
      END DO
  ELSE
      WRITE ( unit = nout, fmt = *) 'No normalisation ocean -> atmosphere'
  END IF
  !!
  RETURN
END SUBROUTINE norma
