! -*- mode: f90 -*-
MODULE mod_polom
CONTAINS
  !
  SUBROUTINE orien (pw)
    !!
    !! Orientation d'un triangle
    !!
    USE declare
    USE u_polyg
    !!
    IMPLICIT NONE
    TYPE (polyg), INTENT(inout) :: pw
    REAL (kind=rp) :: zprod
    !!
    IF ( pw%n /= 3 ) STOP 123
    !!
    zprod =  &
       &   (pw%pt(2)%x-pw%pt(1)%x) &
       & * (pw%pt(3)%y-pw%pt(2)%y) &
       & - (pw%pt(3)%x-pw%pt(2)%x) & 
       & * (pw%pt(2)%y-pw%pt(1)%y)
    !!
    IF ( zprod < 0.0_rp ) THEN 
        WRITE(nout,*) 'avant ', pw%pt(1:pw%n)
        pw%pt(1:pw%n:1) = pw%pt(pw%n:1:-1)
        WRITE(nout,*) 'apres ',  pw%pt(1:pw%n)
    ENDIF
    !!
  END SUBROUTINE orien
  !
  SUBROUTINE polom (x1_in, y1_in, x2_in, y2_in, zsurf_1, zsurf_2, zsurf)
    !---------------------------------------------------------------------
    !- surfaces d'intersection de polygones
    !---------------------------------------------------------------------
    USE u_polyg
    USE declare
    USE mod_build
    !-
    IMPLICIT NONE
    !-
    REAL (kind=rd), DIMENSION (:), INTENT (in) :: x1_in, y1_in
    REAL (kind=rd), DIMENSION (:), INTENT (in) :: x2_in, y2_in
    REAL (kind=rd), INTENT ( out) :: zsurf_1, zsurf_2, zsurf
    !-
    INTEGER (kind=il) :: npd1, npd2
    !-
    REAL (kind=rd), DIMENSION (SIZE(x1_in)) :: x1, y1
    REAL (kind=rd), DIMENSION (SIZE(x2_in)) :: x2, y2
    TYPE (polyg) :: pp1, pp2
    TYPE (polyg), DIMENSION (SIZE(x1_in)-1) :: pd1
    TYPE (polyg), DIMENSION (SIZE(x2_in)-1) :: pd2
    TYPE (polyg) :: pw
    INTEGER (kind=il) :: k1, k2, k, jn
    REAL (kind=rd) :: spp1, spp2, spp, f_ech
    !-
    INTEGER (kind=il), PARAMETER :: i_eor = -2, i_eof = -1
    !-
    LOGICAL, PARAMETER :: ld = .FALSE.
    !-
    x1 = x1_in ; y1 = y1_in ; x2 = x2_in ; y2 = y2_in
    !---------------------------------------------------------------------
    !- Build polygones
    npd1 = SIZE (x1_in) - 1
    npd2 = SIZE (x2_in) - 1
    CALL build_poly ( x1(npd1:1:-1), y1(npd1:1:-1), pp1 )
    CALL build_poly ( x2(npd2:1:-1), y2(npd2:1:-1), pp2 )
    !-
    !- facteur de normalisation ------------------------------------------
    spp1    = p_srf(pp1)
    zsurf_1 = spp1
    DO jn = 1, 3
       f_ech = SQRT ( p_srf(pp1))
       IF (ld) WRITE (unit=*,fmt='(" f_echelle : ", 1pe13.5)') f_ech
       x1 = x1 / f_ech ; y1 = y1 / f_ech
       x2 = x2 / f_ech ; y2 = y2 / f_ech
!-$$       IF ( jn == 3 .AND. ABS ( f_ech - 1.0_rd) .GE. SQRT ( EPSILON ( 1.0_r_4) ) ) THEN
!-$$       END IF
       CALL build_poly ( x1(npd1:1:-1), y1(npd1:1:-1), pp1 )
    END DO
    CALL build_poly ( x2(npd2:1:-1), y2(npd2:1:-1), pp2 )
    f_ech = SQRT ( p_srf(pp1))
    IF ( ABS ( f_ech - 1.0_rd) .GE. SQRT (EPSILON (1.0_r_4) ) ) THEN
        WRITE (unit=nout,fmt=*) 'Erreur normalisation poly : ', f_ech
    END IF
    !
    IF (ld) WRITE (UNIT=*,FMT='("PP1 X ", 8(1PE10.2))') pp1%pt(1:npd1)%x
    IF (ld) WRITE (UNIT=*,FMT='("PP1 Y ", 8(1PE10.2))') pp1%pt(1:npd1)%y
    IF (ld) WRITE (UNIT=*,FMT='("PP2 X ", 8(1PE10.2))') pp2%pt(1:npd2)%x
    IF (ld) WRITE (UNIT=*,FMT='("PP2 Y ", 8(1PE10.2))') pp2%pt(1:npd2)%y
    !
    ! - Calcul de la surface
    zsurf = 0.0_rd
    !- Traitement des intersections --------------------------------------
    spp1 = p_srf (pp1) ; spp2 = p_srf (pp2)
    zsurf_2 = spp2 * zsurf_1

    IF (ld) WRITE ( UNIT=*, FMT='(" surface 1 : ", 1PE10.3)') spp1
    IF (ld) WRITE ( UNIT=*, FMT='(" surface 2 : ", 1PE10.3)') spp2
    !--- Decoupage du polygone 1 en triangles
    DO k1 = 1, npd1
      k = k1+1 ; IF (k > npd1) k = 1
      CALL build_poly ( (/ x1(k1), x1(npd1+1), x1(k) /), (/ y1(k1), y1(npd1+1), y1(k) /), pd1(k1) )
!$$$      CALL p_orp (pd1(k1))
!!$      CALL orien (pd1(k1))
    END DO
    !--- Decoupage du polygone 2 en triangles
    DO k2 = 1, npd2
      k = k2+1 ; IF (k > npd2) k = 1
      CALL build_poly ( (/ x2(k2), x2(npd2+1), x2(k) /), (/ y2(k2), y2(npd2+1), y2(k) /), pd2(k2) )
!$$$      CALL p_orp (pd2(k2))
!!$      CALL orien (pd2(k2))
    END DO
    !
    IF (ld)  WRITE (*,'(" --> ",i5," polygones")') npd1
    Decoup1: DO k1 = 1, npd1
      spp = p_srf (pd1 (k1))
      IF ( spp < zeps ) CYCLE
      IF (ld) WRITE ( UNIT=*, FMT='(" surface 1 ", I3," : ", 1PE10.3)') k1, spp
      IF (ld) WRITE (*,'(" --> ",i5," polygones")') npd2
      Decoup2: DO k2 = 1, npd2
        spp = p_srf ( pd2(k2))
        IF ( spp < zeps ) CYCLE
        IF (ld) WRITE (unit=*,fmt='(" surface 2 ",i3," : ", 1PE10.3)') k2, spp
        !--------- intersections 1 - 2
        IF (ld) WRITE (UNIT=*, FMT='(" intersection 1 ", 1(I4)," - 2 ", 1(I4))') k1, k2
        CALL p_pcp ( pd1(k1), pd2(k2), pw)
        IF (pw%n > 0) THEN
            IF (ld) WRITE (*,'(" intersection :")')
            spp = p_srf (pw)
            zsurf = zsurf + spp
            IF (ld) WRITE (unit=*,fmt='(" surface : ", 1PE10.3)') spp
        ELSE
            IF (ld) WRITE (*,'(" pas d''intersection")')
        ENDIF
      ENDDO Decoup2
    ENDDO Decoup1
    !
!$$$    WRITE ( UNIT = *, FMT = * ) 'zsurf : ', zsurf
    !----------------
    RETURN
    !
  END SUBROUTINE polom
END MODULE mod_polom 
