! -*- Mode: f90 -*-
MODULE mod_polyw_o
CONTAINS
   SUBROUTINE polyw_o (lo2a, la2o)
      !>
      !! Compute weights by exact intersection of polygones
      !!
      USE declare
      USE modeles
      USE math
      USE formula
      USE mod_proj
      USE u_polyg
      USE mod_polom
      USE mod_lbc
      !!
      IMPLICIT NONE
      !!
      REAL (kind=rd), DIMENSION (:), ALLOCATABLE :: xa, ya
      REAL (kind=rd), DIMENSION (:), ALLOCATABLE :: xo, yo !> Polygons coordinates (last is center of polygon)
      REAL (kind=rd) :: xp, yp !> Pole of projection
      REAL (kind=rp) :: zra, zro, zr !> Radius of projected boxes 
      REAL (kind=rp) :: zga, zgo, zg !> Radius of boxes 
      REAL (kind=rd) :: zsa, zso, zs !> Surface of boxes (in cartesian coordinates)
      INTEGER (kind=il) :: kproj = 2 !> Type of projection
      INTEGER (kind=il) :: kaerr(jpae), koerr(jpoe) !> Error code of projection
      INTEGER (kind=il) :: ja, jo, jo2, jai, jaj, joi, joj, jn, nta, na !> Dummy indexes
      LOGICAL (kind=il) :: lwrite  !> For test outputs
      INTEGER (kind=il) :: ld = 1 , ld_t !> Debug flag
      REAL (kind = rl) :: xoloninf, xolonsup, yolatinf, yolatsup
      REAL (kind = rl) :: xaloninf, xalonsup, yalatinf, yalatsup
      REAL (kind=rl) :: dx, dy
      LOGICAL, INTENT (in) :: lo2a, la2o
      REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: xat_n, yat_n, xat_s, yat_s
      REAL (kind=rp), DIMENSION (:), ALLOCATABLE :: xat_p, yat_p
      INTEGER (kind=il) :: kerr_p(jpai+1) !> Error code of projection
      !!
      ALLOCATE (xa (jpae))
      ALLOCATE (ya (jpae))
      ALLOCATE (xo (jpoe))
      ALLOCATE (yo (jpoe))
      !!
      ALLOCATE (xat_n(jpai+1))
      ALLOCATE (yat_n(jpai+1))
      ALLOCATE (xat_s(jpai+1))
      ALLOCATE (yat_s(jpai+1))
      ALLOCATE (xat_p(jpai+1))
      ALLOCATE (yat_p(jpai+1))
      !!
      DO jai = 1, jpai
         xat_s(jai) = -180.0_rl + 360.0_rl * REAL(jai-1,KIND=rl) / REAL(jpai,KIND=rl) 
         xat_n(jai) = +180.0_rl - 360.0_rl * REAL(jai-1,KIND=rl) / REAL(jpai,KIND=rl) 
      END DO
      xat_s(jpai+1) = 0.0_rl ; xat_n(jpai+1) = 0.0_rl
      yat_s(1:jpai) = MAXVAL(ya_ed(1  ,:))   ; yat_s(jpai+1) = -90.0_rl
      yat_n(1:jpai) = MINVAL(ya_ed(jpan,:))  ; yat_n(jpai+1) =  90.0_rl
      !!
      !! Generation of weights
      !!
      ! Initialisation
      nta = 0
      IF (lo2a) THEN 
         jmo2a = 0 ; wo2a = zero ; ko2a = 0 ; nva = 0
      END IF
      IF ( la2o ) THEN 
         jma2o = 0 ; wa2o = zero ; ka2o = 0 ; nvo = 0
      ENDIF
      nborda = 0 ; nbordo = 0
      zra = zero ; zro = zero ; zsa = zero ; zso = zero ; zs = zero
      WRITE (unit = nout, fmt = *)
      WRITE (unit = nout, fmt = *) 'Number of points to compute : ', jpon
      !
      dx = 4.0_rl * 360.0_rl / REAL ( MIN ( jpai, jpaj, jpoi, jpoj), kind=rl)
      dy = 4.0_rl * 180.0_rl / REAL ( MIN ( jpai, jpaj, jpoi, jpoj), kind=rl)

      !!
      !! Atmosphere
      OPEN ( unit = 36, file = 'poly.atm', form = 'formatted', status ='unknown', action = 'write', position = 'rewind' )
      !! Ocean
      OPEN (unit = 37, file = 'poly.oce', form = 'formatted', status ='unknown', action = 'write', position = 'rewind' )
      !!
      Ocean: DO jo = 1, jpon ! Loop over ocean grid points
         ld_t = ld
         lwrite = .FALSE. 
         na = 0
         joi = m2oi(jo) ; joj = m2oj(jo) ! 2D indexes of oce box
         xolonsup = MAXVAL (xo_ed(jo:jo,1:jpoe))
         xoloninf = MINVAL (xo_ed(jo:jo,1:jpoe)) 
         yolatsup = MAXVAL (yo_ed(jo:jo,1:jpoe))
         yolatinf = MINVAL (yo_ed(jo:jo,1:jpoe))
         IF ( yolatsup > 85.0_rl ) yolatsup = 90.0_rl
         IF ( joj == jpoj-1 .AND. yolatsup > 87.0_rl .AND. yalatsup > 85.0_rl) ld_t = 3
         
         IF ( xoloninf > xa_sup+dx .OR. xolonsup < xa_inf-dx ) THEN ! This point is not considered
            IF (ld_t >= 2)  WRITE ( nout, fmt ='("rejected (Oce point outside atm latitude limits)",1I6,2I4,2F8.1)' ) &
               & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe)
            CYCLE  Ocean
         END IF
         IF ( yolatinf > ya_sup+dy .OR. yolatsup < ya_inf-dy ) THEN ! This point is not considered
            IF (ld_t >= 2)  WRITE ( nout, fmt ='("rejected (Oce point outside atm longitude limits)",1I6,2I4,2F8.1)' ) &
               & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe)
            CYCLE  Ocean
         END IF

         IF (iomskt (jo) .EQ. 1 .AND. .NOT. lmasko) THEN  ! This point is not considered
            IF (ld_t >= 4) WRITE (nout, fmt ='("rejected (Ocea masked)",1I6,2I4)') jo, joi, joj
            CYCLE Ocean
         END IF
         IF (iomskp (jo) .EQ. 1_il) THEN  ! Periodicity point rejected
            IF (ld_t >= 4) WRITE (nout, fmt ='("rejected (Oce perio)",1I6,2I4)') jo, joi, joj
            CYCLE Ocean 
         END IF
         !!
         zgo = 0.0_rd
         DO jn = 1, jpoe-1
            zgo = MAX (zgo,  REAL(cartdist (xo_ed(jo,jn), yo_ed(jo,jn), xo_ed(jo,jpoe), yo_ed(jo,jpoe)), rl))
         END DO
         !! Project ocean edges
         xp = xo_ed (jo,jpoe) ; yp = yo_ed (jo,jpoe) ! Projection pole : center of ocean box
         IF (ld_t >= 2) WRITE (nout, fmt ='("working on oce : ", 1I5,2I4,11(2F8.1))') &
            & jo, joi, joj, xo_ed(jo,:), yo_ed(jo,:), yolatinf, yolatsup
         DO jn = 1, jpoe
            CALL proj (xo_ed(jo,jn), yo_ed(jo,jn), xp, yp, xo(jn), yo(jn), kproj, koerr(jn)) ! Project ocean box
         ENDDO
         !! Radius of ocean box in cartesian coordinates
         zro = 0.0_rd
         DO jn = 1, jpoe-1
            zro = MAX (zro, ddist (xo(jn), yo(jn), xo(jpoe), yo(jpoe)))
         END DO
         !! 
         Atmos: DO ja = 1, jpan
            !!
            ld_t = ld
            jai = m2ai(ja) ; jaj = m2aj(ja)   ! 2D atm indexes
            xalonsup = MAXVAL(xa_ed(ja,:))
            xaloninf = MINVAL(xa_ed(ja,:)) 
            yalatsup = MAXVAL(ya_ed(ja,:))
            yalatinf = MINVAL(ya_ed(ja,:)) 

            ! Latitude limits criterion
            IF (dehors (yolatinf-dy, yolatsup+dy, yalatinf-dy, yalatsup+dy)) THEN ! Boxes do not overlap
               IF (ld_t >= 3) WRITE (nout, fmt ='("rejected (lat) : ",2(1I6,2I4,4F8.2))') &
                  & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe), yolatinf, yolatsup, &
                  & ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), yalatinf, yalatsup
               CYCLE Atmos ! On peut passer toute la ligne atmosphere à ce stade
            END IF

            ! Masked point
            IF (iamskt (ja) == 1 .AND. .NOT. lmaska) THEN
               IF (ld_t >= 4) WRITE (nout, fmt ='("rejected (Atm masked)",2(1I6,2I4))') jo, joi, joj, ja, jai, jaj
               IF (ld_t >= 4) WRITE (nout, fmt ='("rejected (mask) : ",2(1I5,2I4,2F8.1,1F8.4))') & 
                  & jo, joi, joj, xp, yp, zro, ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), zra
               CYCLE  Atmos
            END IF

            ! Periodicity
            IF (iamskp (ja) .EQ. 1_il) THEN
               IF (ld_t >= 4) WRITE (nout, fmt ='("rejected (Atm perio)",2(1I6,2I4,2F8.1,1F8.4))') &
                  jo, joi, joj, xp, yp, zra, ja, jai, jaj, xa_ed(jo,jpoe), ya_ed(ja,jpae), zra
               CYCLE Atmos
            END IF

            !!
            zga = 0.0_rd
            DO jn = 1, jpae-1
               zga = MAX (zga, cartdist (xa_ed(ja,jn), ya_ed(ja,jn), xa_ed(ja,jpae), ya_ed(ja,jpae)))
            END DO
            IF ( naperio == -1 ) THEN
               !! Special handling at poles
               IF ( jaj == 1 .OR. jaj == jpaj ) zga = 2.0_rl * zga 
            END IF

            !! Geographical distance criterion
            zg = REAL(cartdist (xo_ed(jo,jpoe), yo_ed(jo,jpoe), xa_ed(ja,jpae), ya_ed(ja,jpae)),KIND=rd)
            IF (zg .GE. 1.2_rd * (zga + zgo)) THEN ! Boxes do not overlap
               IF (ld_t >= 3) WRITE (nout, fmt ='("rejected (cartdist) : ",2(1I5,2I4,2F8.1,1E13.4),1E13.4)') & 
                  & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe), zgo, &
                  & ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), zga, zg
               CYCLE Atmos
            END IF
            !! Project atm edges
            DO jn  = 1, jpae
               CALL proj (xa_ed(ja,jn), ya_ed(ja,jn), xp, yp, xa(jn), ya(jn), kproj, kaerr(jn))
            END DO
            IF (SUM(kaerr(:)) /= 0) THEN
               IF (ld_t >= 2) WRITE (nout, fmt ='("rejected (kerr) : ",2(1I5,2I4,2F8.1,1F8.4))') &
                  & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe), zro, ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), zra
               CYCLE Atmos
            END IF
            IF (ld_t >= 3) THEN 
               WRITE (nout, fmt ='("working on atm : ", 1I5,2I4,11(2F8.1))') ja, jai, jaj, xa_ed(ja,:), ya_ed(ja,:), &
                  & yalatinf, yalatsup
               !$$$            WRITE (nout, fmt = '(9E10.2)') xa(:)
               !$$$            WRITE (nout, fmt = '(9E10.2)') ya(:)
            END IF
            !! Distance criterion
            zra = 0.0_rd
            DO jn = 1, jpae - 1
               zra = MAX (zra, ddist (xa(jn), ya(jn), xa(jpae), ya(jpae)))
            END DO
            zr = ddist (xo(jpoe), yo(jpoe), xa(jpae), ya(jpae)) ! Distance between center of atm and oce grid boxes
            IF (zr .GE. 5.0 * (zra + zro)) THEN ! Boxes do not overlap
               IF (ld_t >= 3) WRITE (nout, fmt ='("rejected (dist) : ",2(1I5,2I4,2F8.1,1F8.4))') & 
                  & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe), zro, ja, jai, jaj, xp, yp, zra
               CYCLE Atmos
            END IF
            IF (ld_t >=3) WRITE (nout, fmt ='("accepted         : ",2(1I5,2I4,2F8.1,1E13.4),1E13.4)') &
               & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe), zro, ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), zra, zr 

            !! Find surface of intersection. Surface is given as the ratio between intersection and oce box
            IF ( naperio == -1 ) THEN
               !! Special handling at poles
               IF      (jai == 1 .AND. jaj == 1   ) THEN 
                  WRITE (nout,*) 'LMDZ South pole'
                  DO jn = 1, jpai+1
                     CALL proj (xat_s(jn), yat_s(jn), xp, yp, xat_p(jn), yat_p(jn), kproj, kerr_p(jn))
                  ENDDO
                  CALL polom (xo, yo, xat_p, yat_p, zso, zsa, zs)
               ELSE IF (jai == 1 .AND. jaj == jpaj) THEN
                  WRITE (nout,*) 'LMDZ North pole'
                  DO jn = 1, jpai+1
                     CALL proj (xat_n(jn), yat_n(jn), xp, yp, xat_p(jn), yat_p(jn), kproj, kerr_p(jn))
                  END DO
                  CALL polom (xo, yo, xat_p, yat_p, zso, zsa, zs)
               ELSE 
                  CALL polom (xo, yo, xa, ya, zso, zsa, zs)
               ENDIF
            ELSE
               CALL polom (xo, yo, xa, ya, zso, zsa, zs) 
            ENDIF
            !!
            IF (zs < zeps) THEN ! Boxes do not overlap
               IF (ld_t >= 3) WRITE (nout, fmt ='("rejected (poly)  : ",2(1I6,2I4,2F8.1),1E13.4)') &
                  & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe), ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), zs
               CYCLE Atmos
            END IF
            IF (ld_t >= 3) WRITE (nout, fmt ='("accepted (final) : ",2(1I5,2I4,2F8.1,1E13.4),3(1PE13.4))') &
               & jo, joi, joj, xo_ed(jo,jpoe), yo_ed(jo,jpoe), zro, &
               & ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), zra, &
               & zsa, zso, zs
            !!
            IF (ld_t >= 2) THEN 
               WRITE (nout, fmt = '(9E10.2)') xa(:)
               WRITE (nout, fmt = '(9E10.2)') ya(:)
               WRITE (nout, fmt = '(9E10.2)') xo(:)
               WRITE (nout, fmt = '(9E10.2)') yo(:)
            END IF
            IF (lo2a) THEN 
               IF ( nva (ja) >= jpo2a ) THEN
                  WRITE ( unit = nout, fmt = '("Not enough oce neighbors in : ", 3I4, 2F8.1)' ) &
                     & jai, jaj, nva(ja), xalont(ja), xalatt(ja)
                  WRITE ( unit = nout, fmt = *)  'xa_ed : ', xa_ed(ja,:)
                  WRITE ( unit = nout, fmt = *)  'ya_ed : ', ya_ed(ja,:)
                  WRITE ( unit = nout, fmt = *) xp, yp, xa, ya
                  DO jn = 1, nva (ja) 
                     jo2 = ko2a (jn,ja)
                     WRITE (nout, fmt = *) jn,jo2,m2oi(jo2),m2oj(jo2),wo2a(jn,ja), &
                        & xolont(jo2),xolatt(jo2),xo_ed(jo2,:),yo_ed(jo2,:)
                  END DO
                  STOP
                  EXIT Atmos
               END IF
               !
               nva(ja) = nva(ja) + 1
               jmo2a = MAX (jmo2a, nva(ja))
               ko2a (nva(ja), ja) = jo 
               ! Weights : surface of intersection
               wo2a (nva(ja), ja) = REAL (zs, KIND = rl)
               IF ( wo2a (nva(ja), ja) > 1.0_rl+epsfrac ) THEN 
                  WRITE ( nout, fmt='("o2a >1 ",2(1I6,2I3),1I3,1E13.4)' )  ja,jai,jaj,jo,joi,joj,jn,wo2a(nva(ja),ja)
                  WRITE (nout, fmt = '(9E10.2)') xa(:)
                  WRITE (nout, fmt = '(9E10.2)') ya(:)
                  WRITE (nout, fmt = '(9E10.2)') xo(:)
                  WRITE (nout, fmt = '(9E10.2)') yo(:)
               END IF
            END IF
            !
            IF (la2o) THEN 
               IF (nvo (jo) >= jpa2o ) THEN
                  WRITE (unit = nout, fmt = '("Not enough atm neighbors in : ", 3I4, 2F8.1)' ) &
                     &  joi, joj, nvo(jo), xolont(jo), xolatt(jo)
                  WRITE (unit = nout, fmt = *) ja, jai, jaj, xalont(ja), xalatt(ja)
                  WRITE (unit = nout, fmt = *) (ka2o(jn,jo), jn=1,jpa2o)
                  WRITE (unit = nout, fmt = *) xo_ed(jo,:), yo_ed(jo,:)
                  WRITE (unit = nout, fmt = *) zs
                  STOP
                  EXIT Atmos
               ENDIF
               nvo (jo) = nvo (jo) + 1
               jma2o = MAX (jma2o, nvo (jo))
               ka2o (nvo(jo), jo) = ja
               !! Choix entre 3 formules de calcul
               !wa2o = zs * zsa / zso
               wa2o (nvo(jo), jo) = zs * xasrft(ja) / xosrft(jo)
               !wa2o (nvo(jo),jo) = REAL (zs * xasrft_pol(ja) / xosrft_pol(jo), KIND=rl)
               IF (wa2o (nvo (jo), jo) > 1.0_rl+epsfrac ) THEN 
                  WRITE (nout, fmt='("a2o >1 ",2(1I6,2I3),1I3,1E13.4)' ) &
                     & ja,jai,jaj,jo,joi,joj,jn,wa2o(nvo(jo),jo)
                  WRITE (nout, fmt = '(9E10.2)') xa(:)
                  WRITE (nout, fmt = '(9E10.2)') ya(:)
                  WRITE (nout, fmt = '(9E10.2)') xo(:)
                  WRITE (nout, fmt = '(9E10.2)') yo(:)
               END IF
               !!
            ENDIF
            na = na + 1
            !
         END DO Atmos
         !!
         IF (MOD(jo,jpon/100) .EQ. 0) WRITE (nout, fmt = '("O overlaping : ", 1I6, 2I4, 2F8.1, 2E10.2, 1I3, 1F12.5)') &
            &   jo, joi, joj, xp, yp, zro, zso, na, SUM(wa2o(:,jo))
         ! IF (MOD(jo,10) .EQ. 0) WRITE (UNIT=nout, FMT='(1I8)') jo
         CALL flush (nout)
         nta = na + nta
         !!
      END DO Ocean
      !!
      WRITE (UNIT=nout,FMT=*) 
      !!
      WRITE (nout, fmt = '("Total points : ", 1I9 , ", Average : ", 1F9.3, " atm points per oce point")') &
         & nta, REAL(nta,KIND=rl) / REAL(jpon,KIND=rl)
      !!
      !$$$    IF ( lo2a ) THEN 
      !$$$        DO ja = 1, jpan
      !$$$          IF ( ABS ( SUM(wo2a(ja,:)) - 1.0_rl ) > SQRT(zeps) ) THEN 
      !$$$              jai = m2ai ( ja) ; jaj = m2aj ( ja) 
      !$$$              WRITE ( nout, fmt = '("pb sum : ", 3I5, " : ", 2(F9.1), " : ", 1PE15.6)' ) ja, jai, jaj, xa_ed(ja,jpae), ya_ed(ja,jpae), SUM(wo2a(ja,:))
      !$$$          END IF
      !$$$        END DO
      !$$$    ENDIF
      !!
      IF (la2o) THEN 
         jma2o = MAXVAL (nvo)
         jma2o = MIN (jma2o, jpa2o)
      ENDIF
      IF (lo2a) THEN 
         jmo2a = MAXVAL (nva)
         jmo2a = MIN (jmo2a, jpo2a)
      END IF
      !!
      WRITE (nout, fmt = *) jpa2o, jma2o, jpo2a, jmo2a
      !!
      CLOSE (unit = 36)
      CLOSE (unit = 37)
      !!
   END SUBROUTINE polyw_o
END MODULE mod_polyw_o
