! -*- Mode: f90 -*-
MODULE mod_prih
   !> Print 2D fields, with ad hoc formats
   USE declare
   IMPLICIT NONE
CONTAINS
   SUBROUTINE  prihre (ptab, pscale, kout, l_norsud)
      !> Print 2D real fields, with ad hoc format
      !!c
      IMPLICIT NONE
      !!
      REAL   (kind=rl), DIMENSION (:, :), INTENT (in) :: ptab
      LOGICAL, INTENT(in), OPTIONAL :: l_norsud 
      INTEGER, INTENT (in), OPTIONAL :: kout
      REAL    (kind=rl), INTENT (in), OPTIONAL :: pscale
      INTEGER (kind=il) :: ki, kj, il1, il2, ji, jj, jn, isca, iout
      REAL    (kind=rl) :: zscale
      INTEGER (kind=il) :: j_deb, j_fin, j_inc

      !
      zscale = 1.0_rl 
      iout   = 6
      IF ( PRESENT (pscale)) zscale = pscale
      IF ( PRESENT (kout))   iout   = kout
      !
      ki = SIZE (ptab, DIM=1)
      kj = SIZE (ptab, DIM=2)
      isca = 20
      IF (zscale .NE. 0.) isca = 30
      !
      il1 = 1
      !
      j_deb=kj ; j_fin=1 ; j_inc=-1
      IF ( PRESENT (l_norsud)) THEN 
         IF ( l_norsud ) THEN
            j_deb=1 ; j_fin=kj ; j_inc=1
         END IF
      END IF
      DO jn = 1, (ki-1+1)/isca + 1
         !
         IF (il1 .GT. ki) RETURN 
         !
         WRITE (UNIT = iout, FMT = '(/)')
         !
         il2 = MIN(ki,il1+1*(isca-1))
         IF (zscale .EQ. 0.) THEN 
            WRITE (UNIT=iout, FMT = '(3X, 20I12, 1X, /)') (ji, ji = il1, il2, 1)
            DO jj = kj, 1, -1
               WRITE (UNIT=iout, FMT = '(1X, I3, 1X, 20(1PE12.4))') jj, ptab (il1:il2,jj)
            END DO
         ELSE 
            WRITE (UNIT=iout, FMT ='(3X, 40I7, 1X, /)') (ji, ji = il1, il2, 1)
            DO jj = j_deb, j_fin, j_inc
               WRITE (UNIT=iout, FMT = '(1X, I3, 1X, 40(F7.2))   ') jj, zscale*ptab (il1:il2,jj)
            END DO
         END IF
         il1 = il1 + 1 * isca
         !
      END DO
      !
      ! ---------------------------------------------------------------------
      !
   END SUBROUTINE prihre

   SUBROUTINE  prihbo (ltab, kout, l_norsud)
      !> Print 2D logical fields
      !!c
      IMPLICIT NONE
      !!
      LOGICAL, DIMENSION (:, :), INTENT (in) :: ltab
      LOGICAL, INTENT(in), OPTIONAL :: l_norsud 
      INTEGER, INTENT (in), OPTIONAL :: kout
      INTEGER :: ki, kj
      INTEGER :: il1, il2, ji, jj, jn, isca, iout
      CHARACTER, DIMENSION (SIZE(ltab,1)) :: cltab
      INTEGER :: j_deb, j_fin, j_inc

      !
      ki = SIZE (ltab, DIM=1) ; kj = SIZE (ltab, DIM=2)
      isca = 200
      iout = 6
      IF ( PRESENT (kout)) iout = kout
      !
      il1 = 1
      ! 
      j_deb=kj ; j_fin=1 ; j_inc=-1
      IF ( PRESENT (l_norsud)) THEN 
         IF ( l_norsud ) THEN
            j_deb=1 ; j_fin=kj ; j_inc=1
         END IF
      END IF
      DO jn = 1, (ki-1+1)/(1*isca) + 1
         !
         IF (il1 .GT. ki) RETURN 
         !
         WRITE (UNIT = iout, FMT = '(/)')
         !
         il2 = MIN (ki, il1+1*(isca-1))
         WRITE (UNIT = iout, FMT ='(1X,200I5, 1X, /)') (ji, ji = il1, il2, 1*5)
         DO jj =  j_deb, j_fin, j_inc
            cltab (:) = "."
            WHERE (ltab(:,jj)) cltab = "*"
            WRITE (UNIT = iout, FMT = '(1X, I3, 1X, 200(1A))   ') jj, cltab (il1:il2)
         END DO
         il1 = il1 + 1 * isca
         !
      END DO
      !
      ! ---------------------------------------------------------------------
      !
   END SUBROUTINE prihbo
   !!
   SUBROUTINE prihin (ktab, kscale, kout, l_norsud)
      !> Print 2D integer fields, with ad hoc format
      !!   Method :
      !!   -------
      !!      The format of print is selected with the dummy argument kscale
      !!
      !!   Input :
      !!   -----
      !!      Argument :
      !!               ktab            : integer array for print
      !!               kscale          : if kscale=0
      !!                                 or if > 5   print ktab with format i8
      !!                                 if kscale=1 print ktab with format i1
      !!                                 if kscale=2 print ktab with format i2
      !!                                 if kscale=3 print ktab with format i3
      !!                                 if kscale=4 print ktab with format i4
      !!                                 if kscale=5 print ktab with format i5
      !!
!!! -------------------------------------------------------------------
!!!  OPA7, Lodyc (5/11/91)
!!! -------------------------------------------------------------------
      !
      IMPLICIT NONE
      !
      INTEGER, INTENT (in), OPTIONAL :: kscale, kout
      LOGICAL, INTENT(in), OPTIONAL :: l_norsud 
      INTEGER, INTENT (in), DIMENSION (:,:) :: ktab
      INTEGER :: ki, kj, il1, il2, ji, jj, jn, isca, iskip, iscale, iout
      CHARACTER (LEN=30) :: clform1, clform2, clform3
      CHARACTER (len=6), DIMENSION (:), ALLOCATABLE :: c_data
      INTEGER :: j_deb, j_fin, j_inc
      !
      iscale = 3
      iout   = 6
      IF ( PRESENT (kscale)) iscale = kscale
      IF ( PRESENT (kout))   iout   = kout
      !
      ki = SIZE (ktab, DIM=1) ; kj = SIZE (ktab, DIM=2)
      !
      ALLOCATE (c_data (ki))
      !
      SELECT CASE (iscale)
      CASE (0) ; isca =  20_il ; iskip = 1_il
      CASE (1) ; isca = 200_il ; iskip = 5_il
      CASE (2) ; isca = 120_il ; iskip = 4_il
      CASE (3) ; isca =  80_il ; iskip = 2_il
      CASE (4) ; isca =  60_il ; iskip = 2_il
      CASE (5) ; isca =  40_il ; iskip = 1_il
      CASE (6) ; isca =  40_il ; iskip = 1_il
      CASE Default 
         isca =  20_il ; iskip = 1_il
      END SELECT
      !
      il1 = 1
      !
      WRITE (UNIT=clform1, FMT='("(5X, "         , 1I3.3, "I", 1I3.3, ")" )' ) isca/iskip, iscale*iskip
      WRITE (UNIT=clform2, FMT='("(1X, 1I3 ,1X, ", 1I3.3, "A", 1I2.2, ")" )' ) isca, iscale
      WRITE (UNIT=clform3, FMT='("(I", 1I2.2,")")') iscale
      !
      ! ---------------------------------------------------------------------
      !
      j_deb=kj ; j_fin=1 ; j_inc=-1
      IF ( PRESENT (l_norsud)) THEN 
         IF ( l_norsud ) THEN
            j_deb=1 ; j_fin=kj ; j_inc=1
         END IF
      END IF
      DO jn = 1_il, (ki-1+1_il)/(1* isca) + 1_il
         !
         IF (il1 .GT. ki) RETURN 
         WRITE (UNIT = iout, FMT = *) ' '
         il2 = MIN ( ki, il1+1*(isca-1_il)) 
         !
         WRITE (UNIT = iout, FMT = clform1) (ji, ji = il1+iskip*1-1, il2, iskip)
         WRITE (UNIT = iout, FMT = *) ' '
         DO  jj = j_deb, j_fin, j_inc
            DO ji = il1, il2
               IF ( ktab (ji, jj) == 0 ) THEN 
                  c_data (ji) = '......'
               ELSE
                  WRITE (UNIT=c_data(ji), FMT=clform3) ktab(ji,jj)
               END IF
            END DO
            WRITE (UNIT = iout, FMT = clform2) jj, c_data (il1:il2)
         END DO
         !
         il1 = il1 + 1* isca
      END DO
      !
      ! ---------------------------------------------------------------------
      !
   END SUBROUTINE prihin
   !!
END MODULE mod_prih

