! -*- Mode: f90 -*-
MODULE mod_proj
CONTAINS
   SUBROUTINE proj (plon, plat, plon0, plat0, px, py, ktype, kerr )
!!!
!!!
!!! Projections between geographic to cartesian
!!! type of projection : 
!!!  1: from geographic to orthographic 
!!!  2: from geographic to Lambert Equal Area 
!!!
    USE declare
    USE u_polyg
    IMPLICIT NONE
    !!
    !!
    REAL (kind=rl), INTENT (in) :: plon , plat  ! Geographical coordinates of point to project
    REAL (kind=rd), INTENT (in) :: plon0, plat0 ! Pole of projection
    INTEGER (kind=il), INTENT (in) :: ktype     ! Type of projection
    INTEGER (kind=il), INTENT (out) :: kerr     ! Error code
    REAL (kind=rd), INTENT (out) :: px, py      ! Cartesian coordinates of projected point
    REAL (kind=rd) :: zr, zlon, zlat, zlat0
    INTEGER (kind=il) :: jn
    !!
    zlon = plon - plon0
    DO jn = 1, 2
      IF ( zlon >  180.0_rd) zlon = zlon - 360.0_rd
      IF ( zlon < -180.0_rd ) zlon = zlon + 360.0_rd
    END DO
    zlon  = rad * zlon
    zlat0 = rad * plat0
    zlat  = rad * plat
    kerr  = 0
    !!
    IF      ( ktype == 1 ) THEN ! Orthographic
        px = COS(zlat ) * SIN(zlon)                   
        py = COS(zlat0) * SIN(zlat) - SIN(zlat0) * COS(zlat) * COS(zlon)
    ELSE IF ( ktype == 2 ) THEN ! Lambert Equal Area
        zr = SIN (zlat0) * SIN(zlat) + COS(zlat0) * COS(zlat) * COS(zlon)
        IF ( zr <= -1.0 ) THEN 
            kerr = 1
            RETURN
        END IF
        zr = SQRT ( 2.0_rd / ( 1.0_rd + zr ) )
        px = zr * COS (zlat) * SIN(zlon)
        py = zr * ( COS(zlat0) * SIN(zlat) - SIN(zlat0) * COS(zlat) * COS(zlon) )
    END IF
    !!
    RETURN
    !!
  END SUBROUTINE proj
END MODULE mod_proj

