! -*- Mode: f90 -*-
MODULE mod_rectifie
CONTAINS
   SUBROUTINE rectifie
      !> Correction de quelques elements
      !!
      USE declare
      USE modeles
      USE mod_lbc
      USE formula
      USE mod_prih
      !!
      IMPLICIT NONE
      INTEGER (kind=il)  :: joi, joj, jai, jaj
      REAL (kind=rl) :: zxc, zyc, ztc, zrc, zlonc, zlatc, zarea, zs_n, zs_s
      REAL (kind=rl), DIMENSION (:), ALLOCATABLE :: zx, zy, zt, zr       !> Dummies for polar projection
      INTEGER (kind=il) :: ierr
      !!
      ALLOCATE (zx (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zs in rectifie')
      ALLOCATE (zy (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zy in rectifie')
      ALLOCATE (zt (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zt in rectifie')
      ALLOCATE (zr (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zr in rectifie')
      !!
      WRITE (nout,*) 'Debut rectifie'
      !!
      !! Cheat for surface (only for LMD5) 
      SELECT CASE (TRIM(camod))
      CASE ('lmd5')
         xasrft = 4.0_rl * rpi * ra * ra / REAL (jpan, kind = rl)
      CASE ('lmdz')
         !!
         IF ( la_pole ) THEN
            !! One point cover all around the poles
            IF (xasrft(m1a(1_il,1_il)) < xasrft(m1a(1_il,2_il)) ) THEN 
               zs_n = 0.0_rl ; zs_n = 0.0_rl
               DO jai = 1, jpai
                  zs_s = zs_s + xasrft(m1a(jai,1_il))
                  zs_n = zs_n + xasrft(m1a(jai,jpaj))
               ENDDO
               DO jai = 1, jpai
                  xasrft(m1a(jai,1_il)) = zs_s
                  xasrft(m1a(jai,jpaj)) = zs_n
               ENDDO
            ENDIF
         END IF
      END SELECT
      !!

      !! Find size of earth 
      ra = DOT_PRODUCT (xasrft, REAL (1-iamskp, kind=rl ))
      WRITE (UNIT=nout, FMT="('surface from xasrft', 1E12.3)" ) ra
      ra = SQRT (ra / (4.0_rl * rpi))
      WRITE (nout, *) "Earth radius (km): ", ra / 1000.0_rl
      WRITE (nout, fmt = '("Raw surfaces ocean, atm : ", 2(1PE15.3)) ' ) SUM (xosrft), SUM (xasrft)
      !!
      !! Ocean southernmost grid box (masked anyway) is extended up to South Pole
      ! Cheat for surface of these boxes
      zarea = ra * ra * rpi * (1.0_rl - SIN(rad * ABS(xolatu (jpoi+1))))
      xosrft (1: jpoi) = zarea / REAL(jpoi-2, KIND = rl)
      WRITE (nout, fmt = '("surface ocean 1 : ", 1PE15.3)' )  DOT_PRODUCT (xosrft, REAL(1-iomskp,kind=rl))
      !!
      !! Ocean : last grid box (in 2nd dimension) is extended up to the pole of the grid.
      !!
      ! Position of grid 'pole' 
      zt    = xolonv * rad
      zr    = 2.0 * TAN (rad * (45.0 - xolatv / 2.0 ) )
      zx    = zr * COS (zt) ; zy  = zr * SIN (zt)
      zxc   = SUM (zx (jpon - jpoi + 2: jpon - 1)) / REAL (jpoi - 2, kind = rl)
      zyc   = SUM (zy (jpon - jpoi + 2: jpon - 1)) / REAL (jpoi - 2, kind = rl)
      zrc   = SQRT (zxc**2 + zyc**2 )
      ztc   = ATAN2 (zyc, zxc)
      zlonc = ztc / rad
      zlatc = 90.0 - ATAN (zrc) / rad
      !
      WRITE (unit = nout, fmt = *) 'Center for ocean : ', zlonc, zlatc
      !
!-$$    IF ( cotyp(1:4) == 'orca' ) THEN 
!-$$       xolonv (jpon - jpoi + 1: jpon) = zlonc
!-$$       xolonf (jpon - jpoi + 1: jpon) = zlonc
!-$$       xolatv (jpon - jpoi + 1: jpon) = zlatc
!-$$       xolatf (jpon - jpoi + 1: jpon) = zlatc
!-$$       WRITE (unit = nout, fmt = *) 'Set grid pole'
!-$$       ! Compute surface of modified boxes
!-$$       zarea = 4.0 * rpi * ra * ra - SUM (xosrft (1: jpon - jpoi - 1) * REAL (iomskp (1: jpon - jpoi - 1), kind=rl)  )
!-$$       xosrft (jpon - jpoi + 1: jpon) = zarea / REAL (jpoi - 2, kind = rl)
!-$$       WRITE (nout, fmt = '("surface ocean 2 : ", 1PE15.3)' )  SUM (xosrft)
!-$$    END IF
      !!
      !! Normalize surface (to the surface of a sphere with radius 1)
      !!
      zarea = 4.0_rl * ra * ra * rpi
      !$$$  xosrft = xosrft / zarea
      !$$$  xasrft = xasrft / zarea
      !!
      xsurfa = SUM (xasrft)
      xsurfo = DOT_PRODUCT (xosrft, REAL(1-iomskp, kind=rl) )
      WRITE (unit = nout, fmt = '("Surface atmos         : ", 1PE20.6)' ) xsurfa
      WRITE (unit = nout, fmt = '("Surface ocean         : ", 1PE20.6)' ) xsurfo
      xsurfa = DOT_PRODUCT (xasrft, REAL ((1-iamskt)*(1-iamskp), kind=rl) )
      xsurfo = DOT_PRODUCT (xosrft, REAL ((1-iomskt)*(1-iomskp), kind=rl) ) 
      WRITE (unit = nout, fmt = '("Surface atmos (masked): ", 1PE20.6)' ) xsurfa
      WRITE (unit = nout, fmt = '("Surface ocean (masked): ", 1PE20.6)' ) xsurfo
      !!
      RETURN
   END SUBROUTINE rectifie
END MODULE mod_rectifie
