! -*- Mode: f90 -*-
!!! 
!!! lecture d'un fichier NetCDF et interpolation
!!! en temps 
!!! Modifiez le module 'modele' selon votre cas de figure
!!!
MODULE declare_2
   !!
   USE defprec
   IMPLICIT NONE
   !! Data types
   INTEGER, PARAMETER :: r4 = r_4, r8 = r_8, i4 = i_4, i8 = i_8
   INTEGER, PARAMETER :: rl = r_std, il = i_std
   INTEGER (kind=il), PARAMETER :: nout = 6 ! Fichier de sortie des messages
   REAL (kind=r8) :: rpi = 3.141592653589_rl
   !!
END MODULE declare_2
!!
MODULE modele_2
  USE declare_2
  IMPLICIT NONE
  CHARACTER (len=80) :: cfile_in = 'AMIP_7d_SSTICE_grid_T_nomask.nc' ! Fichier d'entree
  CHARACTER (len=80) :: cfile_ou = 'AMIP_1d_SSTICE_grid_T_nomask.nc' ! Fichier sortie
  INTEGER (kind=il), PARAMETER :: jpt1 =  52 ! Dimension temps en entre
  INTEGER (kind=il), PARAMETER :: jpt2 = 360 ! Dimension temps en sortie
  INTEGER (kind=il), PARAMETER :: jpi = 92, jpj = 76, jpk = 1 ! Dimension modele
  INTEGER (kind=il), PARAMETER :: jpdim =  4 ! Nombre maxi de dimensions
  INTEGER (kind=il), PARAMETER :: jpvar = 30 ! Nombre maxi de variables
  REAL (kind=r8) :: zfreeze = -1.77_r8 ! Point de conglation
  !
END MODULE modele_2
!!
PROGRAM time
   USE declare_2
   USE modele_2
   USE fliocom
   USE errioipsl
   IMPLICIT NONE
   !! 
   REAL (kind=r8), DIMENSION (jpi*jpj*jpk*jpt1) :: zvar_in8
   REAL (kind=r8), DIMENSION (jpi*jpj*jpk*jpt2) :: zvar_ou8
   REAL (kind=r8), DIMENSION (jpt1) :: time1
   REAL (kind=r8), DIMENSION (jpt2) :: time2
   INTEGER (kind=il), DIMENSION ( jpdim) :: dim_len1, dim_len2
   CHARACTER (len=20), DIMENSION ( jpdim) :: cldim
   INTEGER (kind=il) :: jdim, jvar
   INTEGER (kind=il), DIMENSION ( jpvar) :: xtype, ndims, natts
   CHARACTER (len=20), DIMENSION ( jpvar) :: clvar
   CHARACTER (len=20) :: cname
   CHARACTER (len=20), DIMENSION (1) :: catt
   INTEGER (kind=il), DIMENSION ( jpvar, jpdim) :: dimsid
   INTEGER (kind=il) :: ji, jt, jd, jv, ja, jnt, tid, xid, yid, zid
   INTEGER (kind=il) :: ncid_in, ncid_ou, len1, len2
   CHARACTER (len=8) :: c8
   INTEGER (kind=il), DIMENSION ( jpdim) :: istart1, istart2, icount1, icount2
   LOGICAL :: l_ex
   !!
   !! Ouverture fichier entree
   CALL flioopfd (TRIM(cfile_in), ncid_in, nb_dim=jdim, nb_var=jvar, nb_gat=natts(1))
   !! Dimensions
   CALL flioinqf (ncid_in, ln_dim=dim_len1(1:jdim))
   CALL flioinqn (ncid_in, cn_dim=cldim(1:jdim))
   SeekDim: DO jd = 1, jdim
      WRITE ( unit = c8, fmt = '(1I8)' ) jd
      SELECT CASE ( dim_len1(jd))
      CASE ( jpi)
         dim_len2 (jd) = jpi
      CASE ( jpj )
         dim_len2 (jd) = jpj
      CASE ( jpk)
         dim_len2 (jd) = jpk
      CASE default
         dim_len2 (jd) = dim_len1 (jd)
      END SELECT
      ! Determination du nombre de pas de temps
      jnt = 0
      WRITE(unit=c8,fmt='(2I4)') jd
      IF ( INDEX ( cldim(jd), 'time') /= 0 .OR. INDEX ( cldim(jd), 'TIME') /= 0 .OR. INDEX ( cldim(jd), 'Time') /= 0) THEN 
         jnt = dim_len1(jd) ; tid = jd ; dim_len2 (jd) = -1 ! Unlimited
      END IF
      IF ( INDEX ( cldim(jd), 'X') /= 0 .OR. INDEX ( cldim(jd), 'x') /= 0 ) xid = jd 
      IF ( INDEX ( cldim(jd), 'Y') /= 0 .OR. INDEX ( cldim(jd), 'y') /= 0 ) yid = jd 
      IF ( INDEX ( cldim(jd), 'deptht') /= 0 )  zid = jd 
   END DO SeekDim
   WRITE ( nout, *) 'Nombre de dimensions : ', jdim
   WRITE ( nout, *) 'Longueur temps       : ', jnt
   IF ( jnt /= jpt1) THEN 
      WRITE(nout,*) 'Erreur sur nombre de pas de temps en entree '
      STOP
   END IF
   !! Ouverture fichier sortie
   CALL fliocrfd ( TRIM(cfile_ou), cldim(1:jdim), dim_len2(1:jdim), ncid_ou)
   !!
   !! Attributs globaux
   DO ja = 1, natts (1) 
      WRITE ( unit = c8, fmt = '(2I4)' ) ja
      CALL flioinqn (ncid_in, ia_start=ja, ia_count=1, cn_gat=catt ) ; cname=catt(1)
      CALL fliocpya (ncid_in, '?', cname, ncid_ou, '?' )
   END DO

   !
   !! Definition variables
   SeekVar: DO jv = 1, jvar
      CALL flioinqv (ncid_in, TRIM(clvar(jv)), l_ex, nb_atts=natts(jv), v_t=xtype(jv), nb_dims=ndims(jv))
      CALL flioinqv (ncid_in, TRIM(clvar(jv)), l_ex, id_dims=dimsid(jv,1:ndims(jv)))
      WRITE ( nout, FMT='(1A20,8I5)' ) TRIM(clvar(jv)), xtype(jv), dimsid(jv,1:ndims(jv)), natts(jv), jv
      IF ( INDEX ( clvar(jv), 'time') /= 0 .OR. INDEX ( clvar(jv), 'TIME') /= 0 .OR. &
         & INDEX ( clvar(jv), 'Time') /= 0) tid = jv
      CALL fliodefv (ncid_ou, TRIM(clvar(jv)), dimsid(jv,1:ndims(jv)), v_t=xtype(jv))
      !! Traitements des attributs
      DO ja = 1, natts(jv)
         WRITE ( unit = c8, fmt = '(2I4)' ) jv, ja
         CALL flioinqv (ncid_in, TRIM(clvar(jv)), l_ex, ia_start=ja, ia_count=1, cn_atts=catt) ; cname=catt(1)
         CALL fliocpya (ncid_in, TRIM(clvar(jv)), cname, ncid_ou, TRIM(clvar(jv)) )
      END DO
   END DO SeekVar
   !!
   Var: DO jv = 1, jvar
      WRITE(unit=c8,fmt='(2I4)') jv
      istart1 = 1 ; icount1 = 1 ; istart2 = 1 ; icount2 = 1
      SELECT CASE ( ndims(jv) )
      CASE (1)
         IF      ( jv == tid ) THEN 
            istart1 (1)= 1 ; icount1 (1) = jpt1 ; istart2 (1) = 1 ; icount2 (1) = jpt2
         ELSE IF ( jv == xid ) THEN
            istart1 (1) = 1 ; icount1 (1) = jpi ; istart2 (1) = 1 ; icount2 (1) = jpi
            IF ( jt > 1) CYCLE var
         ELSE IF ( jv == yid ) THEN 
            istart1 (1) = 1 ; icount1 (1) = jpj ; istart2 (1) = 1 ; icount2 (1) = jpj
            IF ( jt > 1) CYCLE var
         ELSE IF ( jv == zid ) THEN 
            istart1 (1) = 1 ; icount1 (1) = jpk ; istart2 (1) = 1 ; icount2 (1) = jpk
            IF ( jt > 1) CYCLE var
         ELSE
            IF ( jt > 1) CYCLE var
         END IF
      CASE ( 2) 
         IF ( jt > 1 ) CYCLE var
         istart1 (1:2) = (/  1, 1 /) ; icount1 (1:2) = (/jpi, jpj /)
         istart2 (1:2) = (/  1, 1 /) ; icount2 (1:2) = (/jpi, jpj /)
      CASE ( 3) 
         istart1 (1:3) = (/  1, 1, 1 /) ; icount1 (1:3) = (/jpi, jpj, jpt1 /)
         istart2 (1:3) = (/  1, 1, 1 /) ; icount2 (1:3) = (/jpi, jpj, jpt2 /)
      CASE ( 4) 
         istart1 (1:4) = (/  1, 1, 1, 1 /) ; icount1 (1:4)  = (/jpi, jpj, jpk, jpt1 /)
         istart2 (1:4) = (/  1, 1, 1, 1 /) ; icount2 (1:4)  = (/jpi, jpj, jpk, jpt2 /)
      END SELECT
      !!
      len1 = product ( icount1 ) ; len2 = product ( icount2 )
      WRITE ( nout, *) TRIM(clvar(jv)), " ", jv, ndims(jv), len1, len2, xtype(jv)
      !
      ! Lecture
      CALL fliogetv (  ncid_in, TRIM(clvar(jv)), zvar_in8(1:len1), istart1, icount1)
      !
      WRITE (nout,*) 'extrema entree : ', MINVAL ( zvar_in8(1:len1)) , MAXVAL ( zvar_in8(1:len1))
      ! Interpolation en temps
      IF ( jv == xid .OR. jv == yid .OR. jv == zid) THEN 
         zvar_ou8(1:len1) = zvar_in8(1:len1)
      ELSE IF ( jv == tid ) THEN
         time1 = zvar_in8(1:len1)
         DO jt = 1, jpt2
            time2 ( jt) = REAL ( jt, kind = r8) * 86400 * 360_r8 / REAL ( jpt2,r8)
         END DO
         zvar_ou8(1:len2) = time2
!$$$          WRITE (nout,*) 'time1 : ', time1
!$$$          WRITE (nout,*) 'time2 : ', time2
      ELSE
         CALL inter ( zvar_in8(1:len1), zvar_ou8(1:len2))
         IF ( INDEX ( clvar(jv), 'ice') /= 0 .OR. INDEX ( clvar(jv), 'ICE') /= 0  )  THEN 
            zvar_ou8(1:len2) = MIN ( 1.0_r8, MAX ( 0.0_r8, zvar_ou8(1:len2)))
         END IF
      END IF
      WRITE (nout,*) 'extrema sortie : ', MINVAL ( zvar_ou8(1:len2)) , MAXVAL ( zvar_ou8(1:len2))
      ! Ecriture
      IF ( ndims(jv) == 3 ) THEN 
         DO jt = 1, jpt2
            istart2 (1:3) = (/  1, 1, jt /) ; icount2 (1:3) = (/jpi, jpj, 1 /)
            CALL flioputv (ncid_ou, TRIM(clvar(jv)), zvar_ou8(jpi*jpj*(jt-1)+1:jpi*jpj*jt), &
               & istart2(1:ndims(jv)), icount2(1:ndims(jv)) )
         END DO
      ELSE IF ( ndims(jv) == 4 ) THEN 
         DO jt = 1, jpt2
            istart2 (1:4) = (/  1, 1, 1, jt /) ; icount2 (1:4) = (/jpi, jpj, jpk, 1 /)
            CALL flioputv (ncid_ou, TRIM(clvar(jv)), zvar_ou8(jpi*jpj*jpk*(jt-1)+1:jpi*jpj*jpk*jt), &
               & istart2(1:ndims(jv)), icount2(1:ndims(jv)) )
         END DO
      ELSE
         CALL flioputv( ncid_ou, TRIM(clvar(jv)), zvar_ou8(1:len2),  &
            & istart2(1:ndims(jv)), icount2(1:ndims(jv)) )
      END IF
   END DO Var
   !!
   CALL flioclo( ncid_in)
   CALL flioclo( ncid_ou)
   !!
   STOP
CONTAINS
   SUBROUTINE inter ( pin, pou )
      USE declare_2
      USE modele_2
      IMPLICIT NONE
      REAL (kind=r8), INTENT (  in), DIMENSION ( :) :: pin
      REAL (kind=r8), INTENT ( out), DIMENSION ( :) :: pou
      REAL (kind=r8), DIMENSION (-1:jpt1+2) :: zz1, zzt, zz
      INTEGER (kind=il) :: idim, jn1, jn2, jn, jt1, jt2
      REAL (kind=r8) :: dt
      !!
      !! Interpolation en temps
      ! Dimension spatiale
      idim = SIZE ( pin) / jpt1
      IF ( idim /= ( SIZE ( pou) / jpt2) ) STOP 001
      ! WRITE (nout,*) 'idim : ', idim
      IF ( jpt1 == jpt2 ) THEN 
         pou = pin
         !!
      ELSE IF ( jpt1 == 2 ) THEN 
         ! On suppose que l'on a les valeurs de fevrier et d'aout
         ! On fait passer un sinus entre ces deux valeurs, en conservant la moyenne pour ces deux mois
         DO jt2 = 1, jpt2  ! nouvel indice 
            DO ji = 1, idim
               jn1 = ji + idim * ( 1 - 1)
               jn2 = ji + idim * ( 2 - 1)
               jn  = ji + idim * ( jt2 - 1)
               pou (jn)= (  ( pin(jn1) + pin(jn2) ) * 360_r8 / REAL(jpt2,r8) ) / 60_r8   & 
                  & +  ( pin(jn1) - pin(jn2) ) * ( COS ( ( 360_r8*REAL(jt,r8)/REAL(jpt2,r8) - 135_r8 ) * rpi /180_r8 ) &
                  & - COS ( ( 360_r8*REAL(jt-1,r8)/REAL(jpt2,r8) - 135_r8 ) * rpi /180_r8 ) ) / 2_r8  &
                  & * ( COS ( 75_r8 * rpi / 180_r8 ) - COS ( 105_r8 * rpi / 180_r8 ) )
            END DO
         END DO
      ELSE  IF ( jpt2 > jpt1 ) THEN ! Spline cubique
         dt = time1(2)-time1(1)
         DO ji = 1, idim
            zzt(1:jpt1) = time1 ; zzt(0) = zzt(1)-dt ; zzt(-1) = zzt(0)-dt
            zzt(jpt1+1) = zzt(jpt1)+dt ; zzt(jpt1+2) = zzt(jpt1+1)+dt
            DO jt1 = 1, jpt1
               zz1 (jt1) = pin (ji+idim*(jt1-1)) 
            END DO
            zz1(0) = zz1 (jpt1) ; zz1(-1) = zz1(jpt1-1) ; zz1 (jpt1+1) = zz1(1) ; zz1(jpt1+2) = zz1(2)
            CALL spline ( zzt, zz1, jpt1+2, zz)
            DO jt2 = 1, jpt2
               CALL splint ( zzt, zz1, zz, jpt1+2, time2(jt2), pou(ji+idim*(jt2-1)) )
            END DO
         END DO
!$$$    ELSE IF ( jpt2 > jpt1 ) THEN ! Lineaire
!$$$        dt = time1(2)-time1(1)
!$$$        DO jt2 = 1, jpt2
!$$$          IF      ( time2(jt2) .LE. time1(1) ) THEN 
!$$$              jt1 = 1 ; jt1p = jpt1
!$$$              z1 = -( time1(1) - time2(jt2)) / dt
!$$$          ELSE IF ( time2(jt2) .GE. time1(jpt1) ) THEN 
!$$$              jt1 = jpt1 ; jt1p = 1
!$$$              z1 = -( time1(jt1) - time2(jt2)) / dt
!$$$          ELSE
!$$$              DO jt1 = 1, jpt1-1
!$$$                IF ( time1(jt1) .LE. time2(jt2) .AND. time1(jt1+1) .GT. time2(jt2) ) EXIT
!$$$              END DO
!$$$              jt1p = jt1+1
!$$$              z1 = ( time2(jt2) - time1(jt1)) / dt
!$$$          END IF
!$$$          z2 = 1.0_r8 - z1
!$$$          DO ji = 1, idim
!$$$            jn1 = ji + idim * ( jt1  - 1) ; jn2 = ji + idim * ( jt1p - 1) ; jn  = ji + idim * ( jt2  - 1)
!$$$            pou(jn) = z1 * pin(jn1) + z2 * pin(jn2)
!$$$          END DO
!$$$        END DO
      END IF
   END SUBROUTINE inter
   SUBROUTINE spline ( x, y, n, y2, yp1, ypn)
      ! Routine to set up the interpolating function for a cubic spline
      ! interpolation (see "Numerical Recipes" for details).
      USE declare_2
      IMPLICIT NONE
      INTEGER (kind=il) :: n
      REAL (kind=r8), DIMENSION (n), INTENT (in) :: x, y
      REAL (kind=r8), INTENT (in), OPTIONAL :: yp1, ypn ! First derivative
      REAL (kind=r8), DIMENSION (n), INTENT (out) :: y2
      REAL (kind=r8), DIMENSION (n) :: u
      REAL (kind=r8) :: un, qn, sig, p
      INTEGER (kind=il) :: i, k
      !
      IF ( PRESENT ( yp1) ) THEN 
         ! have a specified first derivative 
         y2(1)=-0.5_r8
         u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      ELSE 
         ! the lower boundary condition is set to be "natural"
         y2(1)=0.0_r8 ; u(1)=0.0_r8
      END IF
      !
      DO i=2,n-1
         ! decomposition loop of the tridiagonal algorithm. Y2 and U are used
         ! for temporary storage of the decomposed factors
         sig   = (x(i)-x(i-1))/(x(i+1)-x(i-1))
         p     = sig*y2(i-1)+2.0_r8
         y2(i) = (sig-1.)/p
         u(i)  = (6.0_r8*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
      END DO
      !
      IF ( PRESENT ( ypn) ) THEN 
         ! Have a specified first derivative
         qn = 0.50_r8
         un = (3.0_r8/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      ELSE 
         ! the upper boundary condition is set to be "natural"
         qn = 0.0_r8 ; un = 0.0_r8
      END IF
      !
      y2(n) = (un-qn*u(n-1))/(qn*y2(n-1)+1.0_r8)
      !
      DO k = n-1,1,-1
         ! this is the backsubstitution loop of the tridiagonal algorithm
         y2(k) = y2(k)*y2(k+1)+u(k)
      END DO
      RETURN
   END SUBROUTINE spline
   SUBROUTINE splint ( xa, ya, y2a, n, x, y)
      ! Routine to compute a cubic-spline interpolated value Y given the
      ! value of X, the arrays XA, YA and the 2nd derivative array Y2A
      ! computed by SUBROUTINE SPLINE. See "Numerical Recipes" for details
      USE declare_2
      IMPLICIT NONE
      INTEGER (kind=il) :: n
      REAL (kind=r8), INTENT (  in) :: x
      REAL (kind=r8), INTENT ( out) :: y
      REAL (kind=r8), INTENT ( in), DIMENSION(n) :: xa, ya, y2a
      INTEGER (kind=il) :: kl0, khi, k
      REAL (kind=r8) :: h, a, b
      !
      kl0=1; khi=n
      ! means of bisection
      DO WHILE (khi-kl0.GT.1)
         k = (khi+kl0)/2
         IF(xa(k) .GT. x) THEN
            khi=k
         ELSE
            kl0=k
         END IF
      END DO
      ! KL0 and KHI now bracket the X
      h=xa(khi)-xa(kl0)
      IF(h .eq. 0.0_r8) STOP
      a=(xa(khi)-x)/h
      ! evaluation of cubic spline polynomial
      b=(x-xa(kl0))/h
      y=a*ya(kl0)+b*ya(khi)+((a**3-a)*y2a(kl0)+(b**3-b)*y2a(khi))*(h**2)/6.
      !
      RETURN
   END SUBROUTINE splint
END PROGRAM time
