! -*- Mode: f90 -*-
PROGRAM tst_field
!!! -------------------------------------------------------------------------------
!!! Tests des poids pour OASIS 2, Mozaic
!!! Adapte a LMD5 et OPA
!!! -------------------------------------------------------------------------------
   !!
   !! 1) Lit les fichiers de masques, grilles et surfaces des deux modeles
   !! 2) Lit fichier de poids
   !! 3) Lance une srie de test
   !!
   !! --------------------------------------------------------------------------------
   !! LMCE, CEA-Saclay
   !! --------------------------------------------------------------------------------
   !!
   USE declare
   USE modeles
   USE mod_locio
   USE fliocom
   USE errioipsl
   USE formula
   USE mod_lbc
   USE mod_inter
   USE mod_lecdata
   USE mod_inipar
   !!
   IMPLICIT NONE
   !!
   REAL (kind=rl)   , DIMENSION (:), ALLOCATABLE :: zoce, zo        ! Champ ocean
   REAL (kind=rl)   , DIMENSION (:), ALLOCATABLE :: zatm, za        ! Champ atmosphere
   REAL (kind=rl)   , DIMENSION (:), ALLOCATABLE :: zaoce           ! Fraction d'ocean dans la maille atmosphere
   INTEGER (kind=il), DIMENSION (:), ALLOCATABLE :: iaoce
   LOGICAL :: l_run = .FALSE.
   LOGICAL :: l_land = .FALSE.
   LOGICAL :: l_bord = .FALSE.
   LOGICAL, DIMENSION (:), ALLOCATABLE :: lacot, laland
   REAL (kind=rl) :: bila, bilo
   REAL (kind=rd) :: zzd
   CHARACTER (len=1) :: cw_wei = '2'
   CHARACTER (LEN=3) :: lsens
   CHARACTER (LEN=20) :: clarg
   REAL (kind=rl) :: y_lat_min, y_lat_max
   CHARACTER (LEN=1) :: c_lat_min, c_lat_max
   INTEGER :: narg
   !INTEGER, EXTERNAL :: iargc
   !!
   CHARACTER (len = 15) :: clo = 'oce.nc', cla = 'atm.nc'
   INTEGER (kind = i_4) :: nco, nca
   CHARACTER (len = 14) :: ctitle
   !!
   INTEGER (kind=il) :: ja, jo, jn, joi, joj, jai, jaj, ierr
   INTEGER, PARAMETER :: jpn = 34
   !! Initialisation
   !CALL ipsldbg (.TRUE. )
   ngrd = 11 ; nsrf = 12 ; nmsk = 13; nchk = 16 ; ndeb = 9 ; nbug1 = 10
   nwei4o2a = 81 ; nwei8o2a = 82 ; nwei4a2o = 83 ; nwei8a2o = 84
   !!
   CALL inipar
   CALL alloc_modeles
   !!
   ALLOCATE (zoce   (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zoce', lreset=.TRUE., crout='tstmosaic')
   ALLOCATE (zo     (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zo')
   ALLOCATE (zatm   (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'zatm')
   ALLOCATE (za     (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'za')
   ALLOCATE (zaoce  (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'zaoce')
   ALLOCATE (iaoce  (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'iaoce')
   ALLOCATE (lacot  (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'lacot')
   ALLOCATE (laland (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'laland')
   !!
   lwra2o = .FALSE. ; lwro2a = .TRUE.
   !!
   narg = iargc ()
   DO ja = 1, narg
      CALL getarg (ja, clarg)
      SELECT CASE (TRIM(clarg))
      CASE ('-r')
         WRITE (UNIT=nout, FMT=*) 'Runoff case'
         l_run = .TRUE.
         cw_wei = '3'
         jma2o = jma2or
      CASE ('-R')
         l_run = .FALSE.
      CASE ('-2') ; 
         cw_wei = '2'
      CASE ('-3') ; 
         cw_wei = '3'
         jma2o = jma2or
      CASE ('-4') ;
         cw_wei = '4'
         jma2o = jma2or
      CASE ('-5') ; 
         cw_wei = '5'
         jma2o = jma2oi
      CASE ('-l') ;
         l_land = .TRUE.
      CASE ('-L') ;
         l_land = .FALSE.
      CASE ('-b')
         l_bord = .TRUE.
      CASE ('-B')
         l_bord = .FALSE.
      END SELECT
   END DO
   !!
   !! Diag
   !!
   !! Fichiers de diagnostics
   !!
   OPEN (unit = nchk, file = 'tests.out', position = 'REWIND', action = 'write', status = 'replace' )
   !!
   !! Lecture des coordonnees
   !!
   CALL lecdata (lecweights = .TRUE., leco2amask = .TRUE., cw_num=cw_wei)
   !!
   nva = COUNT (mask = (ko2a /= 0), DIM = 1)
   nvo = COUNT (mask = (ka2o /= 0), DIM = 1)
   WRITE (nout, *) 'MAX nva nvo ', MAXVAL(nva), MAXVAL (nvo)
   WRITE (nout, *) 'MIN et MAX ko2a ', MINVAL (ko2a), MAXVAL (ko2a)
   WRITE (nout, *) 'MIn et MAX ka2o ', MINVAL (ka2o), MAXVAL (ka2o)
   !!
   IF (MAXVAL (ko2a) .GT. jpon) WRITE (nout,*) 'Erreur :  MAXVAL (ko2a) .GT. jpon '
   IF (MINVAL (ko2a) .LT. 0   ) WRITE (nout,*) 'Erreur :  MINVAL (ko2a) .LT. 0   '
   IF (MAXVAL (ka2o) .GT. jpan) WRITE (nout,*) 'Erreur :  MAXVAL (ka2o) .GT. jpan '
   IF (MINVAL (ka2o) .LT. 0   ) WRITE (nout,*) 'Erreur :  MINVAL (ka2o) .LT. 0   '
   !!
   !! Periodicity masks
   iomskp = 0_il
   IF (noperio == 1_il .OR. noperio == 4_il ) THEN 
      DO joj = 1, jpoj ! Periodical edge set to zero to suppress double points for ocean
         iomskp (m1o (1_il, joj)) = 1_il
         iomskp (m1o (jpoi, joj)) = 1_il
      END DO
   END IF
   IF (noperio == 1_il .OR. noperio == 4_il) THEN 
      ! Supress duplicate points due to the North fold
      DO joi = 1, jpoi
         iomskp (m1o (joi, jpoj  ) ) = 1_il
      END DO
      DO joi = jpoi/2+2, jpoi
         iomskp (m1o (joi, jpoj-1) ) = 1_il
      END DO
   ENDIF
   !
   iamskp = 0_il
   IF (naperio == 1_il .OR. naperio == 4_il ) THEN 
      DO jaj = 1, jpaj ! Periodical edge set to zero to suppress double points for atm
         iamskp (m1a (1_il, jaj)) = 1_il
         iamskp (m1a (jpai, jaj)) = 1_il
      END DO
   END IF
   IF (naperio == 1 .OR. naperio == 4) THEN 
      ! Supress duplicate points due to the North fold
      DO jai = 1, jpai
         iamskp (m1a (jai, jpaj  ) ) = 1_il
      END DO
      DO jai = jpai/2+2, jpai
         iamskp (m1a (joi, jpaj-1) ) = 1_il
      END DO
   ENDIF
   !!
   !! Ecriture des champs dans un fichier NetCDF
   !! Open file for ocean
   CALL fliocrfd (TRIM(clo), (/ 'x', 'y'/), (/jpoi, jpoj/), nco, mode='REPLACE')
   ! Quelques attributs globaux
   CALL flioputa (nco, '?', 'Conventions', 'COARDS')
   !
   CALL fliodefv (nco, 'omask'  , (/1_il, 2_il/), v_t=flio_i1)
   !
   CALL flioputv (nco, 'omask'  , RESHAPE(iomskt ,(/jpoi, jpoj/)) )
   !!
   !! Open file for atmosphere
   CALL fliocrfd (TRIM(cla), (/ 'x', 'y'/), (/jpai, jpaj/), nca, mode='REPLACE')
   ! Quelques attributs globaux
   CALL flioputa (nca, '?', 'Conventions', 'COARDS')
   !
   CALL fliodefv (nca, 'o2amask', (/1_il, 2_il/), v_t=flio_r4)
   CALL fliodefv (nca, 'amask'  , (/1_il, 2_il/), v_t=flio_i1)
   !
   CALL flioputv (nca, 'amask'  , RESHAPE(iamskt ,(/jpai, jpaj/)) )
   CALL flioputv (nca, 'o2amask', RESHAPE(o2amask,(/jpai, jpaj/)) )
   !!
   CALL inter_o2a (REAL (1-iomskt, kind = rl), zaoce) 
   iaoce = 1
   WHERE (zaoce .GT. 0) iaoce = 0
   !!
   !! Point cotier Atm
   lacot = .FALSE.
   WHERE (o2amask .GT. epsfrac .AND. o2amask .LT. (1.0_rl-epsfrac) ) lacot = .TRUE.
   laland = .FALSE.
   WHERE (o2amask .LT. (1.0_rl-epsfrac) ) laland = .TRUE.
   !!
   !! Verif normalisation
   !$$$  DO ja = 1, jpan
   !$$$     zzd = SUM (REAL (wo2a(:,ja), rd) )
   !$$$     IF (zzd /= 0.0_rd .AND. zzd < 1.0_rd-epsfrac .OR. zzd > 1.0_rd ) THEN 
   !$$$        WRITE (unit=nout, fmt = '("Pb with normalisation o2a: ", 1I8, 2(1PE15.5))' ) &
   !$$$           ja, zzd, zzd-1.0_rd
   !$$$     END IF
   !$$$  END DO
   !$$$  !!
   !$$$  DO jo = 1, jpon
   !$$$     zzd = SUM (REAL (wa2o(:,jo), rd) )
   !$$$     IF (zzd /= 0.0_rd .AND. zzd < 1.0_rd-epsfrac .OR.  zzd > 1.0_rd ) THEN 
   !$$$        WRITE (unit=nout, fmt = '("Pb with normalisation a2o: ", 1I8, 2(1PE15.5))' ) &
   !$$$           jo, zzd, zzd-1.0_rd
   !$$$     END IF
   !$$$  END DO
   !!
   !! Verifs ocean vers atmosphere
   WRITE (UNIT=nout, FMT=*)   '    Nom               Int. Atm      Int. oce   Erreur'

   ctitle = '              '
   LoopCase: DO jn = 1, jpn-1
      !WRITE (UNIT=nout, FMT=*) jn
      SELECT CASE (jn)
      CASE (1_il) ! Oceanic field = constant
         zoce = 1.0_rl ; zatm = 0.0_rl
         CALL lbc (zoce, noperio, 'T', jpoi, jpoj)
         CALL inter_o2a (zoce, zatm)
         ctitle = 'Oce_constant' ; lsens='o2a'
      CASE (2_il) ! Atm field = constant
         zatm = 1.0_rl ; zoce = 0.0_rl
         IF (l_land) WHERE (.NOT. laland) zatm = 0.0_rl
         IF (l_bord) WHERE (.NOT. lacot ) zatm = 0.0_rl
         CALL lbc (zatm, naperio, 'T', jpai, jpaj)
         CALL inter_a2o (zatm, zoce)
         ctitle = 'Atm_constant' ; lsens = 'a2o'
      CASE (3_il) ! Oceanic field = masked constant
         zoce = REAL (1-iomskt, kind = rl)
         zatm = 0.0_rl
         CALL inter_o2a (zoce, zatm)
         ctitle = 'Oce_masked' ; lsens='o2a'
      CASE (4_il) ! Atm field = masked constant
         zatm = REAL (1-iaoce, kind = rl)
         IF (l_land) WHERE (.NOT. laland) zatm = 0.0_rl
         IF (l_bord) WHERE (.NOT. lacot ) zatm = 0.0_rl
         CALL lbc (zatm, naperio, 'T', jpai, jpaj)
         zoce = 0.0_rl
         CALL inter_a2o (zatm, zoce)
         ctitle = 'Atm_masked' ; lsens = 'a2o'
      CASE (5_il) ! Oceanic field = latitude
         zoce = ABS(xolatt) * REAL (1-iomskt, kind = rl)
         CALL lbc (zoce, noperio, 'T', jpoi, jpoj)
         zatm = 0.0_rl
         CALL inter_o2a (zoce, zatm)
         ctitle = 'Oce_lat' ; lsens='o2a'
      CASE (6_il) ! Atm field = latitude
         zatm = ABS(xalatt) * REAL (1-iaoce, kind = rl)
         IF (l_land) WHERE (.NOT. laland) zatm = 0.0_rl
         IF (l_bord) WHERE (.NOT. lacot ) zatm = 0.0_rl
         CALL lbc (zatm, naperio, 'T', jpai, jpaj)
         zoce = 0.0_rl
         CALL inter_a2o (zatm, zoce)
         ctitle = 'Atm_lat' ; lsens = 'a2o'
      CASE (7_il) ! Oceanic field = longitude
         zoce = COS (rad*xolont)**2 * REAL (1-iomskt, kind = rl)
         CALL lbc (zoce, noperio, 'T', jpoi, jpoj)
         zatm = 0.0_rl
         CALL inter_o2a (zoce, zatm)
         ctitle = 'Oce_lon' ; lsens='o2a'
      CASE (8_il) ! Atm field = longitude
         zatm = COS (rad*xalont)**2 * REAL (1-iaoce, kind = rl)
         IF (l_land) WHERE (.NOT. laland) zatm = 0.0_rl
         IF (l_bord) WHERE (.NOT. lacot ) zatm = 0.0_rl
         CALL lbc (zatm, naperio, 'T', jpai, jpaj)
         zoce = 0.0_rl
         CALL inter_a2o (zatm, zoce)
         ctitle = 'Atm_lon' ; lsens = 'a2o'
      CASE (9_il) ! Oceanic field = COS(longitude) * latitude
         zoce = ABS(xolatt) * COS (rad*xolont)**2 * REAL (1-iomskt, kind = rl)
         CALL lbc (zoce, noperio, 'T', jpoi, jpoj)
         zatm = 0.0_rl
         CALL inter_o2a (zoce, zatm)
         ctitle = 'Oce_lonxlat' ; lsens='o2a'
      CASE (10_il) ! Atm field = longitude * latitude
         zatm = COS (rad*xalont)**2 * ABS(xalatt) * REAL (1-iaoce, kind = rl)
         IF (l_land) WHERE (.NOT. laland) zatm = 0.0_rl
         IF (l_bord) WHERE (.NOT. lacot ) zatm = 0.0_rl
         CALL lbc (zatm, naperio, 'T', jpai, jpaj)
         zoce = 0.0_rl
         CALL inter_a2o (zatm, zoce)
         ctitle = 'Atm_lonxlat' ; lsens = 'a2o'
      CASE (11_il) ! Atm field = random
         CALL RANDOM_NUMBER (zatm)
         zatm = zatm * REAL (1-iaoce, kind = rl)
         IF (l_land) WHERE (.NOT. laland) zatm = 0.0_rl
         IF (l_bord) WHERE (.NOT. lacot ) zatm = 0.0_rl
         CALL lbc (zatm, naperio, 'T', jpai, jpaj)
         zoce = 0.0_rl
         CALL inter_a2o (zatm, zoce)
         ctitle = 'Atm_random' ; lsens = 'a2o'
      CASE (12_il) ! Oce field = random
         CALL RANDOM_NUMBER (zoce)
         zoce = zoce * REAL (1-iomskt, kind = rl)
         zatm = 0.0_rl
         CALL inter_o2a (zoce, zatm)
         ctitle = 'Oce_random' ; lsens='o2a' 
      CASE Default ! Oce field = bande de latitude
         IF ( MOD (jn,2) == 0 ) THEN 
            y_lat_min = REAL( (jn-13)/2,rl)*10.0 - 70.0_rl
            y_lat_max = y_lat_min + 10.0_rl
            c_lat_min='S' ; c_lat_max='S'
            IF ( y_lat_min >= 0.0_rl ) c_lat_min = 'N'
            IF ( y_lat_max >= 0.0_rl ) c_lat_max = 'N'
            zoce = 0.0_rl
            WHERE ( xolatt .GE. y_lat_min .AND. xolatt .LE. y_lat_max ) zoce = 1.0_rl
            zoce = zoce * REAL (1-iomskt, kind = rl)
            zatm = 0.0_rl
            CALL inter_o2a (zoce, zatm)
            !WRITE (UNIT=nout  , FMT=*) INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
            !WRITE (UNIT=nout  , FMT='("Oce_", 1I2.2, 1A, "_", 1I2.2, 1A)') INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
            WRITE (UNIT=ctitle, FMT='("Oce_", 1I2.2, 1A, "_", 1I2.2, 1A)') INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
            lsens='o2a'
         ELSE
            y_lat_min = REAL( (jn-13)/2,rl)*10.0 - 70.0_rl
            y_lat_max = y_lat_min + 10.0_rl
            c_lat_min='S' ; c_lat_max='S'
            IF ( y_lat_min >= 0.0_rl ) c_lat_min = 'N'
            IF ( y_lat_max >= 0.0_rl ) c_lat_max = 'N'
            zatm = 0.0_rl
            WHERE ( xalatt .GT. y_lat_min .AND. xalatt .LT. y_lat_max ) zatm = 1.0_rl
            zatm = zatm * REAL (1-iaoce, kind = rl)
            zoce = 0.0_rl
            CALL inter_a2o (zatm, zoce)
            !WRITE (UNIT=nout  , FMT=*) INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
            !WRITE (UNIT=nout  , FMT='("Oce_", 1I2.2, 1A, "_", 1I2.2, 1A)') INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
            WRITE (UNIT=ctitle, FMT='("Atm_", 1I2.2, 1A, "_", 1I2.2, 1A)') INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
            lsens='a2o'
         ENDIF
      CASE (jpn) ! 
         WRITE (nout, *) 'Bilan a -> o'
         zatm = 0.0_rl
         DO ja = 1, jpan
            IF ( iaoce (ja) == 0 ) THEN 
               za = 0.0_rl ; zo = 0.0_rl
               za (ja) = 1.0_rl
               CALL inter_a2o (za, zo)
               zatm (ja) = za (ja) * o2amask (ja) - DOT_PRODUCT (zo, xosrft) / xasrft (ja) 
            END IF
         END DO
         !
         WRITE (nout, *) 'Bilan o -> a'
         zoce = 0.0_rl
         DO jo = 1, jpon 
            IF ( iomskt (jo) == 0 ) THEN 
               zo = 0.0_rl ; za = 0.0_rl
               zo (jo) = 1.0_rl
               CALL inter_o2a (zo, za)
               zoce (jo) = zo (jo) - DOT_PRODUCT (za, xasrft * o2amask) / xosrft (jo)  
            END IF
         END DO
         ctitle = 'Bilan'
      END SELECT
      !!
      SELECT CASE (lsens) 
      CASE ('a2o')
         IF (l_run) THEN
            bila = DOT_PRODUCT (zatm     , REAL((1-iamskp)*(1-iamskt),kind=rl))
         ELSE
            bila = DOT_PRODUCT (zatm*xasrft*o2amask, REAL((1-iamskp)*(1-iamskt),kind=rl))
         END IF
      CASE ('o2a')
         bila = DOT_PRODUCT (zatm*xasrft*o2amask, REAL((1-iamskp)*(1-iamskt),kind=rl))
      END SELECT
      bilo = DOT_PRODUCT (zoce*xosrft, REAL((1-iomskp)*(1-iomskt),kind=rl))
      !
      WRITE (nout, FMT= '(1I3, 1A14, "  ", 2(1PE12.2, "  "), "  ", 1PE8.2)' ) jn, TRIM (ctitle), bila, bilo, ABS((bila-bilo)/(bila+bilo))*2.0_rl
      CALL fliodefv (nco, TRIM (ctitle), (/1, 2/), v_t=flio_r4)
      !CALL flioputa (nco, TRIM (ctitle), 'missing_value', 0.0_r4)
      !
      CALL fliodefv (nca, TRIM(ctitle), (/1, 2/), v_t=flio_r4)
      !CALL flioputa (nca, TRIM(ctitle), 'missing_value', 0.0_r4)
      !
      CALL flioputv (nco, TRIM (ctitle), RESHAPE (zoce, (/jpoi, jpoj/)) ) 
      CALL flioputv (nca, TRIM (ctitle), RESHAPE (zatm, (/jpai, jpaj/)) ) 
      !!
   END DO LoopCase
   !!
   CALL flioclo (nco) 
   CALL flioclo (nca)
   !!
   STOP
   !!
END PROGRAM tst_field
!!
