! -*- Mode: f90 -*-
PROGRAM tstmosaic
!!! -------------------------------------------------------------------------------
!!! Tests des poids pour OASIS 2, Mozaic
!!! Adapte a LMD5 et OPA
!!! -------------------------------------------------------------------------------
   !!
   !! 1) Lit les fichiers de masques, grilles et surfaces des deux modeles
   !! 2) Lit fichier de poids
   !! 3) Lance une srie de test
   !!
   !! --------------------------------------------------------------------------------
   !! LMCE, CEA-Saclay
   !! --------------------------------------------------------------------------------
   !!
   USE declare
   USE modeles
   USE mod_locio
   USE fliocom
   USE errioipsl
   USE formula
   USE mod_lbc
   USE mod_inter
   USE mod_lecdata
   USE bords
   USE mod_inipar
   !!
   IMPLICIT NONE
   !!
   INTEGER (kind=il), PARAMETER :: jpn = 16 ! Nombre de tests d'interpolation
   INTEGER (kind=il) :: jpfirst = 8 ! Premier test avec bandes latitudes
   !!
   REAL (kind=rl)   , DIMENSION (:), ALLOCATABLE :: zoce, zo        ! Champ ocean
   REAL (kind=rl)   , DIMENSION (:), ALLOCATABLE :: zatm, za        ! Champ atmosphere
   REAL (kind=rl)   , DIMENSION (:), ALLOCATABLE :: zaoce           ! Fraction d'ocean dans la maille atmosphere
   INTEGER (kind=il), DIMENSION (:), ALLOCATABLE :: iaoce
   LOGICAL :: l_run = .FALSE.
   LOGICAL :: l_land = .FALSE.
   LOGICAL :: l_bord = .FALSE.
   REAL (kind=rl) :: bila, bilo
   REAL (kind=rd) :: zzd
   CHARACTER (len=1) :: cw_wei = '2'
   CHARACTER (LEN=3) :: lsens
   CHARACTER (LEN=20) :: clarg
   REAL (kind=rl) :: y_lat_min, y_lat_max
   CHARACTER (LEN=1) :: c_lat_min, c_lat_max
   INTEGER :: narg
   !INTEGER, EXTERNAL :: iargc
   !!
   CHARACTER (len = 64) :: clout
   INTEGER (kind = i_4) :: ncout
   CHARACTER (len = 14) :: ctitle
   !!
   INTEGER (kind=il) :: ja, jo, jn, joi, joj, jai, jaj, ierr, jlat, jplat
   REAL (kind=rl) :: dy, y_inf, y_sup
   !! Initialisation
   !CALL ipsldbg (.TRUE. )
   ngrd = 11 ; nsrf = 12 ; nmsk = 13; nchk = 16 ; ndeb = 9 ; nbug1 = 10
   nwei4o2a = 81 ; nwei8o2a = 82 ; nwei4a2o = 83 ; nwei8a2o = 84
   !!
   CALL inipar
   CALL alloc_modeles
   !!
   ALLOCATE (zoce   (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zoce', lreset=.TRUE., crout='tstmosaic')
   ALLOCATE (zo     (jpon), STAT=ierr) ; CALL chk_allo (ierr, 'zo')
   ALLOCATE (zatm   (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'zatm')
   ALLOCATE (za     (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'za')
   ALLOCATE (zaoce  (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'zaoce')
   ALLOCATE (iaoce  (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'iaoce')
   ALLOCATE (lacot  (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'lacot')
   ALLOCATE (laland (jpan), STAT=ierr) ; CALL chk_allo (ierr, 'laland')
   !!
   lwra2o = .FALSE. ; lwro2a = .TRUE.
   !!
   narg = iargc ()
   DO ja = 1, narg
      CALL getarg (ja, clarg)
      SELECT CASE (TRIM(clarg))
      CASE ('-r')
         WRITE (UNIT=nout, FMT=*) 'Runoff case'
         l_run = .TRUE.
         cw_wei = '3'
         jma2o = jma2or
         lsens = 'a2o'
      CASE ('-R')
         l_run = .FALSE.
      CASE ('-2') ; 
         cw_wei = '2'
      CASE ('-3') ; 
         cw_wei = '3'
         jma2o = jma2or
      CASE ('-4') ;
         cw_wei = '4'
         jma2o = jma2or
      CASE ('-5') ; 
         cw_wei = '5'
         jma2o = jma2oi
      CASE ('-l') ;
         l_land = .TRUE.
      CASE ('-L') ;
         l_land = .FALSE.
      CASE ('-b')
         l_bord = .TRUE.
      CASE ('-B')
         l_bord = .FALSE.
      CASE ('-o')
         lsens = 'o2a'
      CASE ('-a')
         lsens = 'a2o'
      END SELECT
   END DO
   !!
   !! Diag
   !!
   !! Fichiers de diagnostics
   !!
   OPEN (unit = nchk, file = 'tests.out', position = 'REWIND', action = 'write', status = 'replace' )
   !!
   !! Lecture des coordonnees
   !!
   CALL lecdata (lecweights = .TRUE., leco2amask = .TRUE., cw_num=cw_wei)
   CALL bord_a
   CALL bord_o
   CALL bord_oa
   !!
   nva = COUNT (mask = (ko2a /= 0), DIM = 1)
   nvo = COUNT (mask = (ka2o /= 0), DIM = 1)
   WRITE (nout, *) 'MAX nva nvo ', MAXVAL(nva), MAXVAL (nvo)
   WRITE (nout, *) 'MIN et MAX ko2a ', MINVAL (ko2a), MAXVAL (ko2a)
   WRITE (nout, *) 'MIN et MAX ka2o ', MINVAL (ka2o), MAXVAL (ka2o)
   !!
   IF (MAXVAL (ko2a) .GT. jpon) WRITE (nout,*) 'Erreur :  MAXVAL (ko2a) .GT. jpon '
   IF (MINVAL (ko2a) .LT. 0   ) WRITE (nout,*) 'Erreur :  MINVAL (ko2a) .LT. 0   '
   IF (MAXVAL (ka2o) .GT. jpan) WRITE (nout,*) 'Erreur :  MAXVAL (ka2o) .GT. jpan '
   IF (MINVAL (ka2o) .LT. 0   ) WRITE (nout,*) 'Erreur :  MINVAL (ka2o) .LT. 0   '
   !!
   !! Ecriture des champs dans un fichier NetCDF
   clout = 'tstmosaic_' // TRIM (lsens)
   CALL fliocrfd (TRIM(clout), (/ 'xa', 'ya', 'xo', 'yo'/), (/jpai, jpaj, jpoi, jpoj/), ncout, mode='REPLACE')
   ! Quelques attributs globaux
   CALL flioputa (ncout, '?', 'Conventions', 'COARDS')
   !
   CALL fliodefv (ncout, 'o2amask' , (/1_il, 2_il/), v_t=flio_r4)
   CALL fliodefv (ncout, 'amask'   , (/1_il, 2_il/), v_t=flio_i1)
   CALL fliodefv (ncout, 'tlmd_lon', (/1_il, 2_il/), v_t=flio_r4)
   CALL fliodefv (ncout, 'tlmd_lat', (/1_il, 2_il/), v_t=flio_r4)
   !
   CALL fliodefv (ncout, 'omask'   , (/3_il, 4_il/), v_t=flio_i1)
   CALL fliodefv (ncout, 'torc_lon', (/3_il, 4_il/), v_t=flio_r4)
   CALL fliodefv (ncout, 'torc_lat', (/3_il, 4_il/), v_t=flio_r4)
   !
   CALL flioputv (ncout, 'omask'   , RESHAPE(iomskt ,(/jpoi, jpoj/)) )
   CALL flioputv (ncout, 'torc_lon', RESHAPE(xolont ,(/jpoi, jpoj/)) )
   CALL flioputv (ncout, 'torc_lat', RESHAPE(xolatt ,(/jpoi, jpoj/)) )
   !
   CALL flioputv (ncout, 'amask'   , RESHAPE(iamskt ,(/jpai, jpaj/)) )
   CALL flioputv (ncout, 'o2amask' , RESHAPE(o2amask,(/jpai, jpaj/)) )
   CALL flioputv (ncout, 'tlmd_lon', RESHAPE(xalont ,(/jpai, jpaj/)) )
   CALL flioputv (ncout, 'tlmd_lat', RESHAPE(xalatt ,(/jpai, jpaj/)) )
   !!
   CALL inter_o2a (REAL (1-iomskt, kind = rl), zaoce) 
   iaoce = 1
   WHERE (zaoce .GT. 0) iaoce = 0
   !!
   !! Point cotier Atm
   lacot = .FALSE.
   WHERE (o2amask .GT. epsfrac .AND. o2amask .LT. (1.0_rl-epsfrac) ) lacot = .TRUE.
   laland = .FALSE.
   WHERE (o2amask .LT. (1.0_rl-epsfrac) ) laland = .TRUE.
   !!
   jplat = jpn-jpfirst
   y_inf = MAX (-90.1_rl, MIN( yo_inf, ya_inf ) - 2.0_rl)
   y_sup = MIN ( 90.1_rl, MAX (yo_sup, ya_sup ) + 2.0_rl)
   dy = (y_sup-y_inf)/REAL(jplat+1,rl)
   WRITE (nout,*) 'y_inf, y_suf, dy ; ', y_inf, y_sup, dy

   !! Verifs 
   WRITE (UNIT=nout, FMT=*)   '    Nom               Int. Atm      Int. oce   Erreur'
   ctitle = ' '
   LoopCase: DO jn = 1, jpn
      !WRITE (UNIT=nout, FMT=*) jn
      IF ( lsens == 'o2a' ) THEN
         SELECT CASE (jn)
         CASE (1_il) ! Oceanic field = constant
            zoce = 1.0_rl ; zatm = 0.0_rl
            ctitle = 'Oce_constant'
         CASE (2_il) ! Oceanic field = masked constant
            zoce = REAL (1-iomskt, kind = rl)
            ctitle = 'Oce_masked'
         CASE (3_il) ! Oceanic field = latitude
            zoce = ABS(xolatt) * REAL (1-iomskt, kind = rl)
            ctitle = 'Oce_lat'
         CASE (4_il) ! Oceanic field = longitude
            zoce = 2.0_rl + COS (rad*xolont)**2 * REAL (1-iomskt, kind = rl)
            ctitle = 'Oce_lon'
         CASE (5_il) ! Oceanic field = COS(longitude) * latitude
            zoce = 2.0_rl + ABS(xolatt) * COS (rad*xolont)**2 * REAL (1-iomskt, kind = rl)
            ctitle = 'Oce_lonxlat'
         CASE (6_il) ! Oce field = random
            CALL RANDOM_NUMBER (zoce)
            zoce = 1.0_rl + zoce * REAL (1-iomskt, kind = rl)
            ctitle = 'Oce_random'
         CASE (7_il) ! Oce field = 1 point
            zoce = 0.0_rl 
            zoce (jpon/2)  = 1.0_rl
            ctitle = 'Oce_point'
         CASE Default ! Oce field = bande de latitude
            jlat  = jn-jpfirst
            y_lat_min = REAL( jlat,rl)*dy + y_inf
            y_lat_max = y_lat_min + dy
            !WRITE (nout,*) jn, jlat, y_lat_min, y_lat_max
            c_lat_min='S' ; c_lat_max='S'
            IF ( y_lat_min >= 0.0_rl ) c_lat_min = 'N'
            IF ( y_lat_max >= 0.0_rl ) c_lat_max = 'N'
            zoce = 0.0_rl
            WHERE ( xolatt .GE. y_lat_min .AND. xolatt .LE. y_lat_max ) zoce = 1.0_rl
            zoce = zoce * REAL (1-iomskt, kind = rl)
            WRITE (UNIT=ctitle, FMT='("Oce_", 1I2.2, 1A, "_", 1I2.2, 1A)') &
               & INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
         END SELECT

         CALL lbc (zoce, noperio, 'T', jpoi, jpoj)
         zatm (:) = 0.0_rl
         CALL inter_o2a (zoce, zatm)

      END IF
      IF ( lsens == 'a2o' ) THEN 
         SELECT CASE (jn)
         CASE (1_il) ! Atm field = constant
            zatm = 1.0_rl
            ctitle = 'Atm_constant'
         CASE (2_il) ! Atm field = masked constant
            zatm = REAL (1-iaoce, kind = rl)
            ctitle = 'Atm_masked'
         CASE (3_il) ! Atm field = latitude
            zatm = ABS(xalatt) * REAL (1-iaoce, kind = rl)
            ctitle = 'Atm_lat'
         CASE (4_il) ! Atm field = longitude * latitude
            zatm= 2.0_rl + COS (rad*xalont)**2 * ABS(xalatt) * REAL (1-iaoce, kind = rl)
            ctitle = 'Atm_lonxlat'
         CASE (5_il) ! Atm field = longitude
            zatm = 2.0_rl + COS (rad*xalont)**2 * REAL (1-iaoce, kind = rl)
            ctitle = 'Atm_lon'
         CASE (6_il) ! Atm field = random
            CALL RANDOM_NUMBER (zatm)
            zatm = 1.0_rl + zatm * REAL (1-iaoce, kind = rl)
            ctitle = 'Atm_random'
         CASE (7_il) ! Atm field = 1 point
            zatm = 0.0_rl
            zatm (1:jpan/2) = 1.0_rl
            ctitle = 'Atm_north'
         CASE Default ! Oce field = bande de latitude
            jlat  = jn-jpfirst
            y_lat_min = REAL( jlat,rl)*dy + y_inf
            y_lat_max = y_lat_min + dy
            c_lat_min='S' ; c_lat_max='S'
            IF ( y_lat_min >= 0.0_rl ) c_lat_min = 'N'
            IF ( y_lat_max >= 0.0_rl ) c_lat_max = 'N'
            zatm = 0.0_rl
            WHERE ( xalatt .GT. y_lat_min .AND. xalatt .LT. y_lat_max ) zatm = 1.0_rl
            zatm = zatm * REAL (1-iaoce, kind = rl)
            zoce = 0.0_rl
            WRITE (UNIT=ctitle, FMT='("Atm_", 1I2.2, 1A, "_", 1I2.2, 1A)') &
               & INT(ABS(y_lat_min)), c_lat_min, INT(ABS(y_lat_max)), c_lat_max
         END SELECT

         IF (l_land) WHERE (.NOT. laland) zatm = 0.0_rl
         IF (l_bord) WHERE (.NOT. lacot ) zatm = 0.0_rl
         CALL lbc (zatm, naperio, 'T', jpai, jpaj)
         CALL inter_a2o (zatm, zoce)

      END IF
      !!
      SELECT CASE (lsens) 
      CASE ('a2o')
         IF (l_run) THEN
            bila = DOT_PRODUCT (zatm     , REAL((1-iamskp)*(1-iamskt),kind=rl))
         ELSE
            bila = DOT_PRODUCT (zatm*xasrft*o2amask, REAL((1-iamskp)*(1-iamskt),kind=rl))
         END IF
      CASE ('o2a')
         bila = DOT_PRODUCT (zatm*xasrft*o2amask, REAL((1-iamskp)*(1-iamskt),kind=rl))
      END SELECT
      bilo = DOT_PRODUCT (zoce*xosrft, REAL((1-iomskp)*(1-iomskt),kind=rl))
      !
      IF ( bila+bilo .EQ. 0.0_rl ) THEN 
         WRITE (nout, FMT= '(1I3, 1A14, "  ", 2(1PE12.2, "  "), "  ", 1PE8.2)' ) &
            & jn, TRIM (ctitle), bila, bilo
      ELSE
         WRITE (nout, FMT= '(1I3, 1A14, "  ", 2(1PE12.2, "  "), "  ", 1PE8.2)' ) &
            & jn, TRIM (ctitle), bila, bilo, ABS((bila-bilo)/(bila+bilo))*2.0_rl
      END IF
      CALL fliodefv (ncout, TRIM (ctitle)//'_OCE', (/3, 4/), v_t=flio_r4)
      !
      CALL fliodefv (ncout, TRIM(ctitle)//'_ATM', (/1, 2/), v_t=flio_r4)
      !
      CALL flioputv (ncout, TRIM (ctitle)//'_OCE', RESHAPE (zoce, (/jpoi, jpoj/)) ) 
      CALL flioputv (ncout, TRIM (ctitle)//'_ATM', RESHAPE (zatm, (/jpai, jpaj/)) ) 

      !!
   END DO LoopCase
   !!
   CALL flioclo (ncout) 
   !!
   STOP
   !!
END PROGRAM tstmosaic
!!
