! -*- Mode: f90 -*-
MODULE mod_wri_wei
   !> Set of routine to write weights and adresses
   USE modeles
   USE formula
   USE fliocom
   USE errioipsl

   LOGICAL :: l_nor2sou
   
   INTERFACE atm_reshape
      MODULE PROCEDURE atm_reshape_2d_r, atm_reshape_3d_r, atm_reshape_2d_i
   END INTERFACE atm_reshape

CONTAINS
   
   FUNCTION atm_reshape_2d_r ( ptab )
      IMPLICIT NONE
      REAL (kind=rl), DIMENSION (jpai, jpaj) :: atm_reshape_2d_r
      REAL (kind=rl), INTENT (in), DIMENSION (jpan) :: ptab
      INTEGER :: jai, jaj, jan, jaj_rev
      
      DO jaj = 1, jpaj
         IF (l_nor2sou) THEN
            jaj_rev = jpaj - jaj + 1
         ELSE
            jaj_rev = jaj
         END IF
         DO jai = 1, jpai
            jan = jai + jpai * ( jaj_rev - 1)
            atm_reshape_2d_r (jai, jaj) = ptab (jan)
         END DO
      END DO
   END FUNCTION atm_reshape_2d_r
   
   FUNCTION atm_reshape_3d_r ( ptab )
      IMPLICIT NONE
      REAL (kind=rl), DIMENSION (jpai, jpaj, jpae) :: atm_reshape_3d_r
      REAL (kind=rl), INTENT (in), DIMENSION (jpan, jpae) :: ptab
      INTEGER :: jai, jaj, jan, jaj_rev
      
      DO jaj = 1, jpaj
         IF (l_nor2sou) THEN
            jaj_rev = jpaj - jaj + 1
         ELSE
            jaj_rev = jaj
         END IF
         DO jai = 1, jpai
            jan = jai + jpai * ( jaj_rev - 1)
            atm_reshape_3d_r (jai, jaj, :) = ptab (jan, :)
         END DO
      END DO
   END FUNCTION atm_reshape_3d_r
   
   FUNCTION atm_reshape_2d_i ( ktab )
      IMPLICIT NONE
      INTEGER, DIMENSION (jpai, jpaj) :: atm_reshape_2d_i
      INTEGER, INTENT (in), DIMENSION (jpan) :: ktab
      INTEGER :: jai, jaj, jan, jaj_rev
      
      DO jaj = 1, jpaj
         IF (l_nor2sou) THEN
            jaj_rev = jpaj - jaj + 1
         ELSE
            jaj_rev = jaj
         END IF
         DO jai = 1, jpai
            jan = jai + jpai * ( jaj_rev - 1)
            atm_reshape_2d_i (jai, jaj) = ktab (jan)
         END DO
      END DO
   END FUNCTION atm_reshape_2d_i
   
   SUBROUTINE wri_weights_o2a (cldiag_o2a, clw_o2a, clw_o2a_mct, clnum, l_fulldiag, lo_src_grid_frac, lo_dst_grid_frac)
      !> Write ocean -> atmosphere weights and adresses
      IMPLICIT NONE
      !!
      CHARACTER (LEN=1), INTENT (in) :: clnum
      CHARACTER (LEN=*), INTENT (in) :: cldiag_o2a, clw_o2a, clw_o2a_mct
      LOGICAL, INTENT (in), OPTIONAL :: l_fulldiag, lo_src_grid_frac, lo_dst_grid_frac
      INTEGER (KIND=il) :: nco2a, ierr, num_links, num_wgts 
      INTEGER (KIND=il) :: il_ncid
      INTEGER (KIND=il) :: jai, jaj, ja, jo, j_link
      INTEGER (KIND=il) :: jn, jw, i_var, i_stat, isize_max, isize, ideb, ifin, ja_deb, ja_fin, ja_inc
      LOGICAL :: l_fd, l_src_grid_frac, l_dst_grid_frac
      CHARACTER (len=180) :: c_date, c_time, c_zone, c_tmp
      !!
      REAL    (kind=rl), DIMENSION (:,:,:), ALLOCATABLE :: w_3d
      INTEGER (kind=il), DIMENSION (:,:,:), ALLOCATABLE :: k_3d
      !!
      REAL    (kind=rl), DIMENSION (:,:), ALLOCATABLE :: w_mct
      INTEGER (kind=il), DIMENSION (:)  , ALLOCATABLE :: k_src, k_dst
      !
      l_fd = .TRUE.
      IF (PRESENT (l_fulldiag)) THEN 
         l_fd = l_fulldiag
      ELSE
         !! Estimation de la taille des variables, limitation si > 2GB
         i_var = jpoi * jpoj * jmo2a
         IF ( i_var >= 2E9_il ) l_fd = .TRUE.
      END IF
      
      IF ( PRESENT (lo_src_grid_frac) ) THEN
         l_src_grid_frac = lo_src_grid_frac
      ELSE
          l_src_grid_frac = .TRUE.
       END IF

      IF ( PRESENT (lo_dst_grid_frac) ) THEN
         l_dst_grid_frac = lo_dst_grid_frac
      ELSE
         l_dst_grid_frac = .TRUE.
      END IF
      
      !!
      !! Dataset ocean --> atmosphere
      !!
      clweight =  "WEIGHTS" // clnum ; cladress = "ADRESSE" // clnum
      WRITE (unit = nout, fmt = *) " oce -> atm ", cladress, " ", clweight, " Neighbors : ", jmo2a
      IF (c_oasis == '2.2' ) THEN
         IF (l_wei_i4 .OR. l_wei_i8) WRITE (unit = nout, fmt = *) 'Ecriture o2a, fichiers binaires IEEE'
         IF (l_wei_i4) THEN 
            WRITE (UNIT = nwei4o2a) cladress
            WRITE (UNIT = nwei4o2a) INT  (ko2a (1_il:jmo2a, 1_il:jpan), KIND=i_4 )
            WRITE (UNIT = nwei4o2a) clweight
            WRITE (UNIT = nwei4o2a) REAL (wo2a (1_il:jmo2a, 1_il:jpan), KIND=rk_out )
         ENDIF
         IF (l_wei_i8) THEN
            WRITE (UNIT = nwei8o2a) cladress
            WRITE (UNIT = nwei8o2a) INT  (ko2a (1_il:jmo2a, 1_il:jpan), KIND=i_8 )
            WRITE (UNIT = nwei8o2a) clweight
            WRITE (UNIT = nwei8o2a) REAL (wo2a (1_il:jmo2a, 1_il:jpan), KIND=rk_out )
         ENDIF
      ENDIF
      IF (l_wei_i4) THEN 
         WRITE (UNIT = nout, FMT = *) 'Ecriture OCEMASK i4r8 : non masque'
         WRITE (UNIT = nwei4o2a) 'OCEMASK'//clnum
         WRITE (UNIT = nout, FMT = *) 'Ecriture OCEMASK i4r8 : masque'
         WRITE (UNIT = nwei4o2a) REAL (o2amask (1_il:jpan), KIND=rk_out )
      END IF
      IF (l_wei_i8) THEN
         WRITE (UNIT = nout, FMT = *) 'Ecriture OCEMASK i8r8 : non masque'
         WRITE (UNIT = nwei8o2a) 'OCEMASK'//clnum
         WRITE (UNIT = nout, FMT = *) 'Ecriture i8r8 : masque'
         WRITE (UNIT = nwei8o2a) REAL (o2amask (1_il:jpan), KIND=rk_out )
         WRITE (UNIT = nout, FMT = *) 'Ecriture en binaire IEEE : fini '
      ENDIF
      !!
      !! Output diagnostics in NetCDF file
      !!
      !CALL ipsldbg (new_status=.TRUE. )
      WRITE (unit = nout, fmt = *) 'Ecriture o2a, diagnostiques ', TRIM(cldiag_o2a)
      WRITE (unit = nout, fmt = *) 'Retournement nord/sud des champs : ', xalatt(1), xalatt(jpan), TRIM(o2a_orien)

      IF (      TRIM(o2a_orien) .EQ. "nord_en_bas") THEN 
         WRITE (unit = nout, fmt = *) "Cas nord_en_bas"
         IF ( xalatt(1) .GT. xalatt(jpan) ) THEN 
            l_nor2sou = .FALSE.
            ja_deb =    1 ; ja_fin = jpaj ; ja_inc =  1
         ELSE
            l_nor2sou = .TRUE.
            ja_deb = jpaj ; ja_fin =    1 ; ja_inc = -1
         END IF
      ELSE IF ( TRIM(o2a_orien) .EQ. "nord_en_haut") THEN 
         WRITE (unit = nout, fmt = *) "Cas nord_en_haut"
         IF ( xalatt(1) .GT. xalatt(jpan) ) THEN 
            l_nor2sou = .TRUE.
            ja_deb = jpaj ; ja_fin =    1 ; ja_inc = -1
         ELSE
            l_nor2sou = .FALSE.
            ja_deb =    1 ; ja_fin = jpaj ; ja_inc =  1
         END IF
      ELSE
         WRITE (unit = nout, fmt = *) "Cas inconnu"
         l_nor2sou = .FALSE.
         ja_deb =    1 ; ja_fin = jpaj ; ja_inc =  1
      END IF

      WRITE (unit = nout, fmt = *) 'Retournement nord/sud des champs : ', l_nor2sou, ja_deb, ja_fin, ja_inc

      CALL fliocrfd (TRIM(cldiag_o2a) // TRIM(c_suffix), (/'x   ', 'y   ', 'n   ', 'edge'/), (/jpai, jpaj, jmo2a, jpae /),&
         & nco2a, mode=c_FlioMode)
      CALL fliopstc (nco2a, &
         & x_axis_2d = atm_reshape(xalont), &
         & y_axis_2d = atm_reshape(xalatt)  )
      CALL flioputa (nco2a, '?', 'Comment', TRIM(c_comment) )
      CALL fliodefv (nco2a, 'Weights_Max'//clnum, (/1_il,2_il/), v_t=flio_r4, standard_name='Oce -> Atm max weights')
      CALL flioputa (nco2a, 'Weights_Max'//clnum, 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'Neighbors'//clnum  , (/1_il,2_il/), v_t=flio_i4, standard_name='Oce -> Atm Neighbors')
      CALL flioputa (nco2a, 'Neighbors'//clnum, 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'Weights_Sum'//clnum, (/1_il,2_il/), v_t=flio_r4, standard_name='Oce -> Atm sum weights')
      CALL flioputa (nco2a, 'Weights_Sum'//clnum, 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'Mask', (/1_il,2_il/), v_t=flio_i4, standard_name='Oce -> Atm mask')
      CALL flioputa (nco2a, 'Mask', 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'OceMask', (/1_il,2_il/), v_t=flio_r4, standard_name='Oce ->Atm OceMask')
      CALL flioputa (nco2a, 'OceMask', 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'OceMask_int', (/1_il,2_il/), v_t=flio_i4, standard_name='Oce ->Atm OceMask')
      CALL fliodefv (nco2a, 'OceMask_ext', (/1_il,2_il/), v_t=flio_i4, standard_name='Oce ->Atm OceMask')
      CALL fliodefv (nco2a, 'LonEdge', (/1,2,4/), v_t=flio_r4)
      CALL flioputa (nco2a, 'LonEdge', 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'LatEdge', (/1,2,4/), v_t=flio_r4) 
      CALL flioputa (nco2a, 'LatEdge', 'missing_value', 0.0_rl)
      IF (l_fd) THEN
         WRITE (nout,*) 'Fichier Diagnostics '
         CALL fliodefv (nco2a, 'Weights'//clnum, (/1,2,3/), v_t=flio_r4, standard_name='Oce -> Atm Weights')
         CALL flioputa (nco2a, 'Weights'//clnum, 'missing_value', 0.0_rl)
         CALL fliodefv (nco2a, 'Adresses'//clnum,(/1,2,3/), v_t=flio_i4, standard_name='Oce -> Atm Adresses')
         CALL flioputa (nco2a, 'Adresses'//clnum, 'missing_value', 0.0_rl)
         CALL fliodefv (nco2a, 'Index_I'//clnum, (/1,2,3/), v_t=flio_i4 )
         CALL flioputa (nco2a, 'Index_I'//clnum, 'missing_value', 0.0_rl)
         CALL fliodefv (nco2a, 'Index_J'//clnum, (/1_il,2_il,3_il/), v_t=flio_i4 )
         CALL flioputa (nco2a, 'Index_J'//clnum, 'missing_value', 0.0_rl)
      ELSE
          WRITE (nout,*) 'Pas de fichier diagnostics '
      END IF
      CALL fliodefv (nco2a, 'MaskPer', (/1_il,2_il/), v_t=flio_i4 )
      CALL flioputa (nco2a, 'MaskPer', 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'Surface', (/1,2/))
      CALL flioputa (nco2a, 'Surface', 'missing_value', 0.0_rl)
      CALL fliodefv (nco2a, 'SurfacePol', (/1,2/))
      CALL flioputa (nco2a, 'SurfacePol', 'missing_value', 0.0_rl)
      !!
      CALL flioputv (nco2a, 'Weights_Max'//clnum, atm_reshape (MAXVAL(wo2a,DIM=1)) )
      CALL flioputv (nco2a, 'Neighbors'//clnum,   atm_reshape (MAX(0_il, nva))     )
      CALL flioputv (nco2a, 'Weights_Sum'//clnum, atm_reshape (SUM   (wo2a,DIM=1)) )
      CALL flioputv (nco2a, 'Mask',               atm_reshape (1_il-iamskt)        )
      CALL flioputv (nco2a, 'OceMask',            atm_reshape (o2amask)            )
      CALL flioputv (nco2a, 'OceMask_int',        atm_reshape (o2amask_i_int)      )
      CALL flioputv (nco2a, 'OceMask_ext',        atm_reshape (o2amask_i_ext)      )
      CALL flioputv (nco2a, 'LonEdge',            atm_reshape (xa_ed)              )
      CALL flioputv (nco2a, 'LatEdge',            atm_reshape (ya_ed)              )
      CALL flioputv (nco2a, 'Surface',            atm_reshape (xasrft)             )
      CALL flioputv (nco2a, 'SurfacePol',         atm_reshape (xasrft_pol)         )

      IF (l_fd) THEN
         ALLOCATE (w_3d(jpai, jpaj, jmo2a), STAT=ierr) 
         CALL chk_allo (ierr, 'w_3d(jpai,jpaj,jmo2a)', lreset=.TRUE., crout='wri_wei_o2a')
         ALLOCATE (k_3d(jpai, jpaj, jmo2a), STAT=ierr)
         CALL chk_allo (ierr, 'k_3d(jpai,jpaj,jmo2a)')

         DO jn = 1_il, jmo2a
            DO jaj = 1_il, jpaj
               DO jai = 1_il, jpai
                  w_3d (jai, jaj, jn) = REAL(wo2a (jn, m1a(jai, jaj)), KIND=rl)
                  k_3d (jai, jaj, jn) = INT (ko2a (jn, m1a(jai, jaj)), KIND=il)
               END DO
            END DO
         END DO
         IF (limit_stack) THEN 
            DO jn = 1_il, jmo2a
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nco2a, 'Weights'//clnum, w_3d(:,ja_deb:ja_fin:ja_inc,jn), start=(/1_il,1_il,jn/) )
            END DO
         ELSE
            CALL flioputv (nco2a, 'Weights'//clnum, w_3d(:,ja_deb:ja_fin:ja_inc,:) )
         ENDIF
         !!
         IF  (limit_stack) THEN
            DO jn = 1_il, jmo2a
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nco2a, 'Adresses'//clnum, k_3d(:,ja_deb:ja_fin:ja_inc,jn), start=(/1_il,1_il,jn/) )
            ENDDO
         ELSE
            CALL flioputv (nco2a, 'Adresses'//clnum, k_3d(:,ja_deb:ja_fin:ja_inc,:))
         ENDIF
         DEALLOCATE (w_3d)
         !
         k_3d = 0_il
         DO jn = 1_il, jmo2a
            DO jaj = 1_il, jpaj
               DO jai = 1_il, jpai
                  k_3d (jai, jaj, jn) = m2oi(ko2a(jn, m1a(jai, jaj)))
               END DO
            END DO
         END DO
         IF (limit_stack) THEN 
            DO jn = 1_il, jmo2a
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nco2a, 'Index_I'//clnum, k_3d(:,ja_deb:ja_fin:ja_inc,jn), start=(/1_il,1_il,jn/) )
            ENDDO
         ELSE
            CALL flioputv (nco2a, 'Index_I'//clnum, k_3d(:,ja_deb:ja_fin:ja_inc,1:jmo2a)) 
         ENDIF
         k_3d = 0_il
         DO jn = 1_il, jmo2a
            DO jaj = 1_il, jpaj
               DO jai = 1_il, jpai
                  k_3d (jai, jaj, jn) = m2oj(ko2a(jn, m1a(jai, jaj)))
               END DO
            END DO
         END DO
         IF (limit_stack) THEN 
            DO jn = 1_il, jmo2a
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nco2a, 'Index_J'//clnum, k_3d(:,ja_deb:ja_fin:ja_inc,jn), start=(/1_il,1_il,jn/) )
            ENDDO
         ELSE
            CALL flioputv (nco2a, 'Index_J'//clnum, k_3d(:,ja_deb:ja_fin:ja_inc,1:jmo2a)) 
         ENDIF
         DEALLOCATE (k_3d)
      ENDIF
      !!
      WRITE (unit=nout,fmt=*) 'Fin ecriture diagnostiques o2a '
      CALL flioclo (nco2a)
      !!
      IF ( l_wei_oasis_3 ) THEN
         !! Ecriture des poids au format NetCDF de OASIS, format OASIS 3
         !! O -> A
         WRITE (unit=nout,fmt=*) 'Ecriture poids o2a NetCDF OASIS 3'
         CALL fliocrfd (TRIM(clw_o2a) // TRIM(c_suffix) // '.nc' , (/'jmo2a', 'jpan '/), (/jmo2a, jpan/), il_ncid, mode=c_FlioMode )
         CALL flioputa (il_ncid, '?', 'Comment', TRIM(c_comment) )
         CALL fliodefv (il_ncid, 'WEIGHTS'//clnum, (/1, 2/), v_t=flio_r )
         CALL fliodefv (il_ncid, 'ADRESSE'//clnum, (/1, 2/), v_t=flio_i )
         ! Write WEIGHTS
         IF (limit_stack) THEN 
            DO jn = 1, jmo2a
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (il_ncid, 'WEIGHTS'//clnum, wo2a(jn,:), start=(/jn, 1/), count=(/1,jpan/) )
            END DO
         ELSE
            CALL flioputv (il_ncid, 'WEIGHTS'//clnum, wo2a(1:jmo2a,:))
         ENDIF
         ! Write ADRESSE
         IF (limit_stack) THEN 
            DO jn = 1, jmo2a
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (il_ncid, 'ADRESSE'//clnum, ko2a(jn,:), start=(/jn, 1/), count=(/1,jpan/) )
            END DO
         ELSE
            CALL flioputv (il_ncid, 'ADRESSE'//clnum, ko2a(1:jmo2a,:)) 
         ENDIF
         WRITE (unit=nout, fmt=*) 'Fin ecriture poids o2a NetCDF'
         CALL flioclo (il_ncid)
         !!
         WRITE (unit=nout,fmt=*) 'Ecriture poids o2a NetCDF, format OASIS MCT'
         num_links = jpan * jmo2a
         num_wgts  = 1
         ALLOCATE (w_mct(num_wgts,num_links), STAT=ierr) 
         CALL chk_allo (ierr, 'w_mct(num_wgts,num_links)', lreset=.TRUE., crout='wri_wei_o2a')
         ALLOCATE (k_src(num_links), STAT=ierr) 
         CALL chk_allo (ierr, 'k_src(num_links)', lreset=.TRUE., crout='wri_wei_o2a')
         ALLOCATE (k_dst(num_links), STAT=ierr) 
         CALL chk_allo (ierr, 'k_dst(num_links)', lreset=.TRUE., crout='wri_wei_o2a')
         !
         j_link = 0
         DO ja = 1, jpan
            DO jn = 1, jmo2a
               IF (  ko2a (jn, ja) /= 0 ) THEN
                  j_link = j_link + 1
                  k_dst (j_link)   = ja
                  k_src (j_link)   = ko2a (jn, ja)
                  w_mct (1,j_link) = wo2a (jn, ja) 
               END IF
            END DO
         END DO
         num_links = j_link

      END IF
      
      IF ( l_wei_oasis_mct ) THEN
         !! Ecriture des poids au format NetCDF de OASIS, format OASIS MCT
         
         CALL fliocrfd (TRIM(clw_o2a_mct) // TRIM(c_suffix), &
            & (/'src_grid_size   ', 'dst_grid_size   ', 'src_grid_corners', 'dst_grid_corners'  , &
            &   'src_grid_rank   ', 'dst_grid_rank   ', 'num_links       ', 'num_wgts        '/), &
            & (/ jpon             , jpan              ,  4                ,  4                  , &
            &    2                ,  2                ,  num_links        ,  num_wgts         /), &
            & il_ncid, mode=c_FlioMode) 
         CALL flioputa (il_ncid, "?", "title"         , TRIM(clw_o2a_mct) )
         CALL flioputa (il_ncid, '?', 'Comment', TRIM(c_comment) )
         CALL flioputa (il_ncid, "?", "normalization" , "none" )
         CALL flioputa (il_ncid, "?", "map_method"    , "Conservative Remapping" )
         CALL DATE_AND_TIME (c_date, c_time, c_zone )
         CALL flioputa (il_ncid, "?", "history"       , "Created: "//c_date(1:4)//"-"//c_date(5:6)//"-"//c_date(7:8) &
            & //" "//c_time(1:2)//"h"//c_time(3:4)//" GMT"//TRIM(c_zone) )
         CALL flioputa (il_ncid, "?", "conventions"   , "SCRIP" )
         CALL flioputa (il_ncid, "?", "method"        , "MOSAIC" )
         CALL flioputa (il_ncid, "?", "source_grid"   , "curvilinear" )
         CALL flioputa (il_ncid, "?", "dest_grid"     , "curvilinear" )
         CALL flioputa (il_ncid, "?", "Institution"   , "IPSL" )
         CALL flioputa (il_ncid, "?", "Model"         , "IPSL CM6" )
         CALL flioputa (il_ncid, "?", "Max_nei_num"   , jmo2a )
         CALL GET_ENVIRONMENT_VARIABLE ( NAME="HOSTNAME" , VALUE=c_tmp, TRIM_NAME=.TRUE., STATUS=i_stat)
         IF ( i_stat == 0 ) THEN
            CALL flioputa (il_ncid, "?", "HOSTNAME"     , TRIM(c_tmp) )
         ELSE 
            WRITE (nout,*) 'Environment variable not found : $HOSTNAME'
         END IF
         CALL GET_ENVIRONMENT_VARIABLE ( NAME="LOGNAME" , VALUE=c_tmp, TRIM_NAME=.TRUE., STATUS=i_stat)
         IF ( i_stat == 0 ) THEN
            CALL flioputa (il_ncid, "?", "LOGNAME"      , TRIM(c_tmp) )
         ELSE 
            WRITE (nout,*) 'Environment variable not found : $LOGNAME'
         END IF
         !
         CALL fliodefv (il_ncid, 'src_grid_dims'      , (/5/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'dst_grid_dims'      , (/6/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'src_grid_center_lat', (/1/)  , v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'src_grid_center_lon', (/1/)  , v_t=flio_r , units = "degrees_east" )
         CALL fliodefv (il_ncid, 'dst_grid_center_lat', (/2/)  , v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'dst_grid_center_lon', (/2/)  , v_t=flio_r , units = "degrees_east" )
         CALL fliodefv (il_ncid, 'src_grid_corner_lat', (/3,1/), v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'src_grid_corner_lon', (/3,1/), v_t=flio_r , units = "degrees_east" )
         CALL fliodefv (il_ncid, 'dst_grid_corner_lat', (/4,2/), v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'dst_grid_corner_lon', (/4,2/), v_t=flio_r , units = "degrees_east" )
         CALL fliodefv (il_ncid, 'src_grid_imask'     , (/1/)  , v_t=flio_i , units = "unitless"      )
         CALL fliodefv (il_ncid, 'dst_grid_imask'     , (/2/)  , v_t=flio_i , units = "unitless"      )
         CALL fliodefv (il_ncid, 'src_grid_area'      , (/1/)  , v_t=flio_r , units = "m^2" )
         CALL fliodefv (il_ncid, 'dst_grid_area'      , (/2/)  , v_t=flio_r , units = "m^2" )
         CALL fliodefv (il_ncid, 'src_grid_frac'      , (/1/)  , v_t=flio_r , units = "unitless"      )
         CALL fliodefv (il_ncid, 'dst_grid_frac'      , (/2/)  , v_t=flio_r , units = "unitless"      )
         CALL fliodefv (il_ncid, 'dst_address'        , (/7/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'src_address'        , (/7/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'remap_matrix'       , (/8,7/), v_t=flio_r )
         !   
         CALL flioputa (il_ncid, 'src_grid_imask', 'land_value', 0)
         CALL flioputa (il_ncid, 'src_grid_imask', 'sea_value' , 1)
         CALL flioputa (il_ncid, 'dst_grid_imask', 'land_value', 0)
         CALL flioputa (il_ncid, 'dst_grid_imask', 'sea_value' , 1)
         !
         CALL flioputv (il_ncid, 'src_grid_dims'      , (/ jpoi, jpoj /) ) 
         CALL flioputv (il_ncid, 'dst_grid_dims'      , (/ jpai, jpaj /) )
         CALL flioputv (il_ncid, 'src_grid_center_lat',         xolatt )
         CALL flioputv (il_ncid, 'src_grid_center_lon', lon_180(xolont))
         CALL flioputv (il_ncid, 'dst_grid_center_lat',         xalatt )
         CALL flioputv (il_ncid, 'dst_grid_center_lon', lon_180(xalont))
         
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         yo_ed (:,1)  , start=(/1,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         yo_ed (:,3)  , start=(/2,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         yo_ed (:,5)  , start=(/3,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         yo_ed (:,8)  , start=(/4,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xo_ed (:, 1)), start=(/1,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xo_ed (:, 3)), start=(/2,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xo_ed (:, 5)), start=(/3,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xo_ed (:, 8)), start=(/4,1/), count=(/1,jpon/) )
         
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         ya_ed (:, 1) , start=(/1,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         ya_ed (:, 3) , start=(/2,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         ya_ed (:, 5) , start=(/3,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         ya_ed (:, 8) , start=(/4,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xa_ed (:, 1)), start=(/1,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xa_ed (:, 3)), start=(/2,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xa_ed (:, 5)), start=(/3,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xa_ed (:, 8)), start=(/4,1/), count=(/1,jpan/) )
         
         CALL flioputv (il_ncid, 'src_grid_imask'     , (1-iomskt)*(1-iomskp) )
         CALL flioputv (il_ncid, 'dst_grid_imask'     , 1-o2amask_i_int*(1-iomskp) )
         CALL flioputv (il_ncid, 'src_grid_area'      , xosrft )
         CALL flioputv (il_ncid, 'dst_grid_area'      , xasrft )
         
         CALL flioputv (il_ncid, 'dst_grid_frac'      , o2amask )
         
         
         IF (l_src_grid_frac) THEN
            CALL flioputv (il_ncid, 'src_grid_frac'      , REAL ((1-iomskt)*(1-iomskp),KIND=rl) )
         ELSE
            CALL flioputv (il_ncid, 'src_grid_frac'      , REAL (1-0*iomskt,KIND=rl) )
         ENDIF
         IF (l_dst_grid_frac) THEN
            CALL flioputv (il_ncid, 'dst_grid_frac'      , o2amask )
         ELSE
            CALL flioputv (il_ncid, 'dst_grid_frac'      , 1.0_rl+0.0_rl*o2amask )
         END IF
         
         CALL flioputv (il_ncid, 'dst_address'        , k_dst (1:num_links) )
         CALL flioputv (il_ncid, 'src_address'        , k_src (1:num_links) )
         CALL flioputv (il_ncid, 'remap_matrix '      , w_mct (1:num_wgts,1:num_links) )
         
         !!
         CALL flioclo (il_ncid)
      END IF
      
      RETURN
      
   END SUBROUTINE wri_weights_o2a
   !!
   SUBROUTINE wri_weights_a2o (cldiag_a2o, clw_a2o, clw_a2o_mct, clnum, l_fulldiag, lo_src_grid_frac, lo_dst_grid_frac, &
                               co_omsk, co_amsk )
      !> Write atmosphere -> ocean weights and adresses
      IMPLICIT NONE
      !!
      CHARACTER (len=1), INTENT (in) :: clnum
      CHARACTER (len=*) :: cldiag_a2o, clw_a2o, clw_a2o_mct
      LOGICAL, INTENT (in), OPTIONAL :: l_fulldiag, lo_src_grid_frac, lo_dst_grid_frac
      CHARACTER (len=*), INTENT (in), OPTIONAL :: co_omsk, co_amsk
      INTEGER (KIND=il) :: nca2o, ierr, num_links, num_wgts 
      INTEGER (KIND=il) :: il_ncid
      INTEGER (KIND=il) :: joi, joj, ja, jo, j_link
      INTEGER (KIND=il) :: jn, jw, j1, j2, i_var, i_stat, isize_max, isize, ideb, ifin
      LOGICAL :: l_fd, l_src_grid_frac, l_dst_grid_frac
      CHARACTER (len=20) :: c_omsk, c_amsk
      CHARACTER (len=180) :: c_date, c_time, c_zone, c_tmp
      !!
      REAL    (KIND=rl), DIMENSION (:,:,:), ALLOCATABLE :: w_3d
      INTEGER (KIND=il), DIMENSION (:,:,:), ALLOCATABLE :: k_3d
      !!
      REAL    (kind=rl), DIMENSION (:,:), ALLOCATABLE :: w_mct
      INTEGER (kind=il), DIMENSION (:)  , ALLOCATABLE :: k_src, k_dst
      !
      l_fd = .TRUE.
      IF (PRESENT (l_fulldiag)) THEN 
         l_fd = l_fulldiag
      ELSE
         !! Estimation de la taille des variables, limitation si > 2GB
         i_var = jpoi * jpoj * jmo2a
         IF ( i_var >= 2E9_il ) l_fd = .TRUE.
      END IF

      IF ( PRESENT (lo_src_grid_frac) ) THEN
         l_src_grid_frac = lo_src_grid_frac
      ELSE
         l_src_grid_frac = .TRUE.
      END IF
      WRITE (nout,*) 'wri_wei_a2o : l_src_grid_frac  : ', l_src_grid_frac 

      IF ( PRESENT (lo_dst_grid_frac) ) THEN
         l_dst_grid_frac = lo_dst_grid_frac
      ELSE
         l_dst_grid_frac = .TRUE.
      END IF
      WRITE (nout,*) 'wri_wei_a2o : l_dst_grid_frac : ', l_dst_grid_frac

      IF ( PRESENT (co_omsk) ) THEN
         c_omsk = TRIM(co_omsk)
      ELSE
         c_omsk = 'perio'
      END IF
      WRITE (nout,*) 'wri_wei_a2o : c_omsk : ', TRIM ( c_omsk)

      IF ( PRESENT (co_amsk) ) THEN
         c_amsk = TRIM( co_amsk) 
      ELSE
         c_amsk = 'int'
      END IF
      WRITE (nout,*) 'wri_wei_a2o : c_amsk : ', TRIM(c_amsk)

      !IF ( LEN_TRIM(c_suffix) /= 0) c_suffix = '_' // TRIM(c_suffix)
      !!      
      !! Dataset atmosphere --> ocean
      !!
      clweight =  "WEIGHTS" // clnum ; cladress = "ADRESSE" // clnum
      WRITE (UNIT = nout, fmt = *) " atm -> oce ", cladress, " ", clweight, " Number of neighbors : ", jma2o
      IF (c_oasis == '2.2' ) THEN
         IF (l_wei_i4 .OR. l_wei_i8) WRITE (unit = nout, fmt = *) 'Ecriture a2o, fichiers binaires IEEE'
         IF (l_wei_i4) THEN
            WRITE (UNIT = nwei4a2o) cladress
            WRITE (UNIT = nwei4a2o) ((INT (ka2o (jn, jo) , KIND=i_4), jn = 1_il, jma2o), jo = 1_il, jpon)
            WRITE (UNIT = nwei4a2o) clweight
            WRITE (UNIT = nwei4a2o) ((REAL (wa2o (jn, jo), KIND=rk_out), jn = 1_il, jma2o), jo = 1_il, jpon)
         END IF
         IF (l_wei_i8) THEN
            WRITE (UNIT = nwei8a2o) cladress
            WRITE (UNIT = nwei8a2o) ((INT  (ka2o (jn, jo), KIND=i_8), jn = 1_il, jma2o), jo = 1_il, jpon)
            WRITE (UNIT = nwei8a2o) clweight
            WRITE (UNIT = nwei8a2o) ((REAL (wa2o (jn, jo), KIND=rk_out), jn = 1_il, jma2o), jo = 1_il, jpon)
         END IF
      ENDIF

      ! CALL ipsldbg (new_status=.TRUE. )
      WRITE (unit = nout, fmt = *) 'Ecriture a2o, diagnostiques ', TRIM(cldiag_a2o)
      CALL fliocrfd (TRIM(cldiag_a2o) // TRIM(c_suffix), (/'x   ', 'y   ', 'n   ', 'edge'/), (/jpoi, jpoj, jma2o, jpoe/), &
         &nca2o, mode=c_FlioMode)
      CALL fliopstc (nca2o, & 
         & x_axis_2d = RESHAPE (xolont, (/jpoi, jpoj/)), &
         & y_axis_2d = RESHAPE (xolatt, (/jpoi, jpoj/)))
      CALL flioputa (nca2o, '?', 'Comment', TRIM(c_comment) )
      CALL fliodefv (nca2o, 'Weights_Max'//clnum, (/1_il,2_il/), v_t=flio_r4, standard_name='Oce -> Atm max weights')
      CALL flioputa (nca2o, 'Weights_Max'//clnum, 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'Neighbors'//clnum  , (/1_il,2_il/), v_t=flio_i4, standard_name='Oce -> Atm Neighbors')
      CALL flioputa (nca2o, 'Neighbors'//clnum, 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'Weights_Sum'//clnum, (/1_il,2_il/), v_t=flio_r4, standard_name='Oce -> Atm sum weights')
      CALL flioputa (nca2o, 'Weights_Sum'//clnum, 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'Mask', (/1_il,2_il/), v_t=flio_i4, standard_name='Oce -> Atm mask')
      CALL flioputa (nca2o, 'Mask', 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'AtmMask', (/1_il,2_il/), v_t=flio_r4, standard_name='Oce ->Atm OceMask')
      CALL flioputa (nca2o, 'AtmMask', 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'LonEdge', (/1_il,2_il,4_il/), v_t=flio_r4)
      CALL flioputa (nca2o, 'LonEdge', 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'LatEdge', (/1_il,2_il,4_il/), v_t=flio_r4) 
      CALL flioputa (nca2o, 'LatEdge', 'missing_value', 0.0_rl)
      IF (l_fd) THEN 
         CALL fliodefv (nca2o, 'Weights'//clnum, (/1_il,2_il,3_il/), v_t=flio_r4, standard_name='Oce -> Atm Weights')
         CALL flioputa (nca2o, 'Weights'//clnum, 'missing_value', 0.0_rl)
         CALL fliodefv (nca2o, 'Adresses'//clnum,(/1_il,2_il,3_il/), v_t=flio_i4, standard_name='Oce -> Atm Adresses')
         CALL flioputa (nca2o, 'Adresses'//clnum, 'missing_value', 0.0_rl)
         CALL fliodefv (nca2o, 'Index_I'//clnum, (/1_il,2_il,3_il/), v_t=flio_i4)
         CALL flioputa (nca2o, 'Index_I'//clnum, 'missing_value', 0.0_rl)
         CALL fliodefv (nca2o, 'Index_J'//clnum, (/1_il,2_il,3_il/), v_t=flio_i4)
         CALL flioputa (nca2o, 'Index_J'//clnum, 'missing_value', 0.0_rl)
      END IF
      CALL fliodefv (nca2o, 'MaskPer', (/1_il,2_il/), v_t=flio_i4)
      CALL flioputa (nca2o, 'MaskPer', 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'Surface', (/1_il,2_il/))
      CALL flioputa (nca2o, 'Surface', 'missing_value', 0.0_rl)
      CALL fliodefv (nca2o, 'SurfacePol', (/1_il,2_il/))
      CALL flioputa (nca2o, 'SurfacePol', 'missing_value', 0.0_rl)

      ALLOCATE (w_3d(jpoi, jpoj, 1), STAT=ierr) ; CALL chk_allo (ierr, 'w_3d(jpoi,jpoj,1)')
      w_3d (1:jpoi , 1:jpoj, 1) = RESHAPE (MAXVAL (wa2o,DIM=1), (/jpoi, jpoj/))
      CALL flioputv (nca2o, 'Weights_Max'//clnum, w_3d (1:jpoi, 1:jpoj, 1) )
      !CALL flioputv (nca2o, 'Weights_Max'//clnum, RESHAPE(MAXVAL(wa2o,DIM=1),(/jpoi, jpoj/)) )
      CALL flioputv (nca2o, 'Neighbors'//clnum,   RESHAPE ( MAX (0_il, nvo) ,(/jpoi, jpoj/)) )
      CALL flioputv (nca2o, 'Weights_Sum'//clnum, RESHAPE ( SUM (wa2o,DIM=1),(/jpoi, jpoj/)) )
      CALL flioputv (nca2o, 'Mask',    RESHAPE (1_il-iomskt, (/jpoi, jpoj/)) )
      CALL flioputv (nca2o, 'AtmMask', RESHAPE (a2omask, (/jpoi, jpoj/)) )
      CALL flioputv (nca2o, 'LonEdge', RESHAPE (xo_ed, (/jpoi, jpoj, jpoe/)) )
      CALL flioputv (nca2o, 'LatEdge', RESHAPE (yo_ed, (/jpoi, jpoj, jpoe/)) )
      CALL flioputv (nca2o, 'Surface', RESHAPE (xosrft, (/jpoi,jpoj/)))
      CALL flioputv (nca2o, 'SurfacePol', RESHAPE(xosrft_pol, (/jpoi,jpoj/)))
      DEALLOCATE (w_3d)
      
      IF (l_fd) THEN 
         ALLOCATE (w_3d(jpoi, jpoj, jma2o), STAT=ierr)
         CALL chk_allo (ierr, 'w_3d(jpoi,jpoj,jma2o)', lreset=.TRUE., crout='wri_wei_a2o')
         ALLOCATE (k_3d(jpoi, jpoj, jma2o), STAT=ierr)
         CALL chk_allo (ierr, 'k_3d(jpai,jpaj,jmo2a)')
         
         IF (ierr /= 0 ) THEN
            WRITE(UNIT=nout,fmt=*) 'Erreur allocation k_3d dans wri_wei_a2o : ', ierr
            STOP
         END IF
         DO jn = 1_il, jma2o
            DO joj = 1_il, jpoj
               DO joi = 1_il, jpoi
                  w_3d (joi, joj, jn) = wa2o (jn, m1o(joi, joj))
                  k_3d (joi, joj, jn) = ka2o (jn, m1o(joi, joj))
               END DO
            END DO
         END DO
         !!
         IF (limit_stack) THEN
            DO jn = 1_il, jma2o
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nca2o, 'Weights'//clnum, w_3d(:,:,jn), start=(/1_il, 1_il, jn/) )
            ENDDO
         ELSE
            CALL flioputv (nca2o, 'Weights'//clnum, w_3d(:,:,:) )
         ENDIF
         !!
         IF (limit_stack) THEN
            DO jn = 1_il, jma2o
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nca2o, 'Adresses'//clnum, k_3d(:,:,jn), start=(/1_il, 1_il, jn/) )
            ENDDO
         ELSE
            CALL flioputv (nca2o, 'Adresses'//clnum, k_3d(:,:,1:jma2o)) 
         ENDIF
         !!
         CALL flioputv (nca2o, 'MaskPer',  & 
            RESHAPE( (1_il-iomskt)*(1_il-iomskp), (/jpoi, jpoj/) ) )
         !!
         DEALLOCATE (w_3d)
         !!
         k_3d = 0_il
         DO jn = 1_il, jma2o
            DO joj = 1_il, jpoj
               DO joi = 1_il, jpoi
                  k_3d (joi, joj, jn) = m2ai(ka2o(jn, m1o(joi, joj)))
               END DO
            END DO
         END DO
         IF (limit_stack) THEN
            DO jn = 1_il, jma2o
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nca2o, 'Index_I'//clnum, k_3d(:,:,jn), start=(/1_il,1_il,jn/) )
            ENDDO
         ELSE
            CALL flioputv (nca2o, 'Index_I'//clnum, k_3d(:,:,1:jma2o)) 
         ENDIF
         k_3d = 0_il
         DO jn = 1_il, jma2o
            DO joj = 1_il, jpoj
               DO joi = 1_il, jpoi
                  k_3d (joi, joj, jn) = m2aj(ka2o (jn, m1o(joi, joj)))
               END DO
            END DO
         END DO
         IF (limit_stack) THEN
            DO jn = 1_il, jma2o
               !WRITE (UNIT=nout,FMT='(1I4)') jn
               CALL flioputv (nca2o, 'Index_J'//clnum, k_3d(:,:,jn), start=(/1_il,1_il,jn/) )
            END DO
         ELSE
            CALL flioputv (nca2o, 'Index_J'//clnum, k_3d(:,:,1:jma2o))
         ENDIF
         !
         DEALLOCATE (k_3d)
         !!
      END IF
      !!
      WRITE (unit=nout,fmt=*) 'Fin ecriture diagnostiques a2o '
      CALL flioclo (nca2o)
      !!
      !!
      !! Ecriture des poids au format NetCDF de OASIS 3
      IF ( l_wei_oasis_3 ) THEN
         WRITE (unit=nout,fmt=*) 'Ecriture poids a2o NetCDF, format OASIS 3'
         CALL fliocrfd (TRIM(clw_a2o) // TRIM(c_suffix) // '.nc', (/'jma2o', 'jpon '/), (/jma2o, jpon/), il_ncid, mode=c_FlioMode)
         CALL flioputa (il_ncid, '?', 'Comment', TRIM(c_comment) )
         CALL fliodefv (il_ncid, 'WEIGHTS'//clnum, (/1,2/), v_t=flio_r)
         CALL fliodefv (il_ncid, 'ADRESSE'//clnum, (/1,2/), v_t=flio_i)
         ! Write WEIGHTS
         IF (limit_stack) THEN
            j1 = 1 ; j2 = MIN (slice_size, jma2o)
            DO WHILE (j1 < jma2o)
               WRITE (UNIT=nout,FMT='("Ecriture wa2o, jn : ", 2I6, 5I8)') j1, j2, SIZE (wa2o(j1:j2,:))
               CALL flioputv (il_ncid, 'WEIGHTS'//clnum, wa2o(j1:j2,:), start=(/j1,1_il/), count=(/j2-j1+1,jpon/) )
               j1 = j1 + slice_size ; j2 = MIN (j1+slice_size-1, jma2o)
            END DO
         ELSE
            WRITE (UNIT=nout,FMT='(5I10)') SIZE (wa2o(1:jma2o,:)), SIZE(wa2o(1:jma2o,:),DIM=1), SIZE(wa2o(1:jma2o,:),DIM=2) 
            CALL flioputv (il_ncid, 'WEIGHTS'//clnum, wa2o(1:jma2o,:) )
         ENDIF
         ! Write ADRESSE
         IF (limit_stack) THEN
            j1 = 1 ; j2 = MIN (slice_size, jma2o)
            DO WHILE ( j1 < jma2o)
               WRITE (UNIT=nout,FMT='("Ecriture ka2o, jn : ", 2I6, 5I8)') j1, j2, SIZE (ka2o(j1:j2,:))
               CALL flioputv (il_ncid, 'ADRESSE'//clnum, ka2o(j1:j2,:), start=(/j1,1_il/), count=(/j2-j1+1,jpon/) )
               j1 = j1 + slice_size ; j2 = MIN (j1+slice_size-1, jma2o)
            END DO
         ELSE
            WRITE (UNIT=nout,FMT='(5I10)') SIZE (ka2o(1:jma2o,:)), SIZE (ka2o(1:jma2o,:),DIM=1), SIZE (ka2o(1:jma2o,:),DIM=2) 
            CALL flioputv (il_ncid, 'ADRESSE'//clnum, ka2o(1:jma2o,:) ) 
         ENDIF
         !!
         WRITE (unit=nout, fmt=*) 'Fin ecriture adresses a2o NetCDF, format OASIS 3'
         CALL flioclo (il_ncid)
         !!
         WRITE (unit=nout,fmt=*) 'Ecriture poids a2o NetCDF, format OASIS MCT'
         num_links = jpon * jma2o
         num_wgts  = 1
         ALLOCATE (w_mct(num_wgts,num_links), STAT=ierr) 
         CALL chk_allo (ierr, 'w_mct(num_wgts,num_links)', lreset=.TRUE., crout='wri_wei_a2o')
         ALLOCATE (k_src(num_links), STAT=ierr) 
         CALL chk_allo (ierr, 'k_src(num_links)', lreset=.TRUE., crout='wri_wei_a2o')
         ALLOCATE (k_dst(num_links), STAT=ierr) 
         CALL chk_allo (ierr, 'k_dst(num_links)', lreset=.TRUE., crout='wri_wei_a2o')
         !
         j_link = 0
         DO jo = 1, jpon
            DO jn = 1, jma2o
               IF ( ka2o (jn, jo) /= 0 ) THEN 
                  j_link = j_link + 1
                  k_dst (j_link)   = jo
                  k_src (j_link)   = ka2o (jn, jo)
                  w_mct (1,j_link) = wa2o (jn, jo) 
               END IF
            END DO
         END DO
         num_links = j_link
         !
      END IF
      !
         !! Ecriture des poids au format NetCDF de OASIS MCT
      IF ( l_wei_oasis_mct ) THEN
         CALL fliocrfd (TRIM(clw_a2o_mct) // TRIM(c_suffix), &
            & (/'src_grid_size   ', 'dst_grid_size   ', 'src_grid_corners', 'dst_grid_corners',   &
            &   'src_grid_rank   ', 'dst_grid_rank   ', 'num_links       ', 'num_wgts        '/), &
            & (/ jpan             ,  jpon             ,  4                ,  4                ,   &
            &    2                ,  2                ,  num_links        ,  num_wgts         /), &
            & il_ncid, mode=c_FlioMode) 
         CALL flioputa (il_ncid, "?", "title"         , TRIM(clw_a2o_mct) )
         CALL flioputa (il_ncid, '?', 'Comment', TRIM(c_comment) )
         CALL flioputa (il_ncid, "?", "normalization" , "none" )
         CALL flioputa (il_ncid, "?", "map_method"    , "Conservative Remapping" )
         CALL DATE_AND_TIME (c_date, c_time, c_zone )
         CALL flioputa (il_ncid, "?", "history"       , "Created: "//c_date(1:4)//"-"//c_date(5:6)//"-"// &
            & c_date(7:8)//" "//c_time(1:2)//"h"//c_time(3:4)//" GMT"//TRIM(c_zone) )
         CALL flioputa (il_ncid, "?", "conventions"   , "SCRIP" )
         CALL flioputa (il_ncid, "?", "method"        , "MOSAIC" )
         CALL flioputa (il_ncid, "?", "source_grid"   , "curvilinear" )
         CALL flioputa (il_ncid, "?", "dest_grid"     , "curvilinear" )
         CALL flioputa (il_ncid, "?", "Institution"   , "IPSL" )
         CALL flioputa (il_ncid, "?", "Model"         , "IPSL CM6" )
         CALL flioputa (il_ncid, "?", "Max_nei_num"   , jma2o )
         CALL flioputa (il_ncid, "?", "c_amsk"        , TRIM(c_amsk))
         CALL flioputa (il_ncid, "?", "c_omsk"        , TRIM(c_omsk))
         CALL GET_ENVIRONMENT_VARIABLE ( NAME="HOSTNAME", VALUE=c_tmp, TRIM_NAME=.TRUE., STATUS=i_stat)
         IF ( i_stat == 0 ) THEN
            CALL flioputa (il_ncid, "?", "HOSTNAME"     , TRIM(c_tmp) )
         ELSE 
            WRITE (nout,*) 'Environment variable not found : $HOSTNAME'
         END IF
         CALL GET_ENVIRONMENT_VARIABLE ( NAME="LOGNAME" , VALUE=c_tmp, TRIM_NAME=.TRUE., STATUS=i_stat)
         IF ( i_stat == 0 ) THEN
            CALL flioputa (il_ncid, "?", "LOGNAME"      , TRIM(c_tmp) )
         ELSE 
            WRITE (nout,*) 'Environment variable not found : $LOGNAME'
         END IF
         !
         CALL fliodefv (il_ncid, 'src_grid_dims'      , (/5/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'dst_grid_dims'      , (/6/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'src_grid_center_lat', (/1/)  , v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'src_grid_center_lon', (/1/)  , v_t=flio_r , units = "degrees_east"  )
         CALL fliodefv (il_ncid, 'dst_grid_center_lat', (/2/)  , v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'dst_grid_center_lon', (/2/)  , v_t=flio_r , units = "degrees_east"  )
         CALL fliodefv (il_ncid, 'src_grid_corner_lat', (/3,1/), v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'src_grid_corner_lon', (/3,1/), v_t=flio_r , units = "degrees_east"  )
         CALL fliodefv (il_ncid, 'dst_grid_corner_lat', (/4,2/), v_t=flio_r , units = "degrees_north" )
         CALL fliodefv (il_ncid, 'dst_grid_corner_lon', (/4,2/), v_t=flio_r , units = "degrees_east"  )
         CALL fliodefv (il_ncid, 'src_grid_imask'     , (/1/)  , v_t=flio_i , units = "unitless"      )
         CALL fliodefv (il_ncid, 'dst_grid_imask'     , (/2/)  , v_t=flio_i , units = "unitless"      )
         CALL fliodefv (il_ncid, 'src_grid_area'      , (/1/)  , v_t=flio_r , units = "m^2" )
         CALL fliodefv (il_ncid, 'dst_grid_area'      , (/2/)  , v_t=flio_r , units = "m^2" )
         CALL fliodefv (il_ncid, 'src_grid_frac'      , (/1/)  , v_t=flio_r , units = "unitless"      )
         CALL fliodefv (il_ncid, 'dst_grid_frac'      , (/2/)  , v_t=flio_r , units = "unitless"      )
         CALL fliodefv (il_ncid, 'dst_address'        , (/7/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'src_address'        , (/7/)  , v_t=flio_i )
         CALL fliodefv (il_ncid, 'remap_matrix'       , (/8,7/), v_t=flio_r )
         !
         CALL flioputa (il_ncid, 'src_grid_imask', 'land_value', 0)
         CALL flioputa (il_ncid, 'src_grid_imask', 'sea_value' , 1)
         CALL flioputa (il_ncid, 'dst_grid_imask', 'land_value', 0)
         CALL flioputa (il_ncid, 'dst_grid_imask', 'sea_value' , 1)
         !
         CALL flioputv (il_ncid, 'src_grid_dims'      , (/ jpai, jpaj /) ) 
         CALL flioputv (il_ncid, 'dst_grid_dims'      , (/ jpoi, jpoj /) )
         CALL flioputv (il_ncid, 'src_grid_center_lat',         xalatt )
         CALL flioputv (il_ncid, 'src_grid_center_lon', lon_180(xalont))
         CALL flioputv (il_ncid, 'dst_grid_center_lat',         xolatt )
         CALL flioputv (il_ncid, 'dst_grid_center_lon', lon_180(xolont))
         
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         ya_ed (:,1) , start=(/1,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         ya_ed (:,3) , start=(/2,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         ya_ed (:,5) , start=(/3,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lat',         ya_ed (:,8) , start=(/4,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xa_ed (:,1)), start=(/1,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xa_ed (:,3)), start=(/2,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xa_ed (:,5)), start=(/3,1/), count=(/1,jpan/) )
         CALL flioputv (il_ncid, 'src_grid_corner_lon', lon_180(xa_ed (:,8)), start=(/4,1/), count=(/1,jpan/) )
         
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         yo_ed (:,1) , start=(/1,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         yo_ed (:,3) , start=(/2,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         yo_ed (:,5) , start=(/3,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lat',         yo_ed (:,8) , start=(/4,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xo_ed (:,1)), start=(/1,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xo_ed (:,3)), start=(/2,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xo_ed (:,5)), start=(/3,1/), count=(/1,jpon/) )
         CALL flioputv (il_ncid, 'dst_grid_corner_lon', lon_180(xo_ed (:,8)), start=(/4,1/), count=(/1,jpon/) )

         SELECT CASE ( TRIM(c_amsk) )
         CASE ( 'ext' )
            CALL flioputv (il_ncid, 'src_grid_imask'        , 1-o2amask_i_ext     )
         CASE ( 'int' )
            CALL flioputv (il_ncid, 'src_grid_imask'        , 1-o2amask_i_int     )
         CASE ( 'full')
            CALL flioputv (il_ncid, 'src_grid_imask'        , 0*(1-o2amask_i_int) )
         END SELECT
         
         SELECT CASE ( TRIM (c_omsk) )
         CASE ( 'noperio' )
            CALL flioputv (il_ncid, 'dst_grid_imask'     , (1-iomskt)*(1-iomskp) )
         CASE ( 'perio' )
            CALL flioputv (il_ncid, 'dst_grid_imask'     , (1-iomskt)            )
         END SELECT
         
         CALL flioputv (il_ncid, 'src_grid_area'      , xasrft )
         CALL flioputv (il_ncid, 'dst_grid_area'      , xosrft )
         IF (l_src_grid_frac) THEN
            CALL flioputv (il_ncid, 'src_grid_frac'      , o2amask )
         ELSE
            CALL flioputv (il_ncid, 'src_grid_frac'      , o2amask*0.0_rl+1.0_rl )
         ENDIF
         IF (l_dst_grid_frac) THEN
            SELECT CASE ( TRIM(c_omsk))
            CASE ('noperio' )
               CALL flioputv (il_ncid, 'dst_grid_frac'      , REAL ((1-iomskt)*(1-iomskp),KIND=rl) )
            CASE ( 'perio' )
               CALL flioputv (il_ncid, 'dst_grid_frac'      , REAL ((1-iomskt)           ,KIND=rl) )
            END SELECT
         ELSE
            SELECT CASE ( TRIM(c_omsk))
            CASE ('noperio' )
               CALL flioputv (il_ncid, 'dst_grid_frac'      , REAL ( (1-0*iomskt)*(1-0*iomskp) ,KIND=rl) )
            CASE ( 'perio' )
               CALL flioputv (il_ncid, 'dst_grid_frac'      , REAL ( (1-0*iomskt)              ,KIND=rl) )
            END SELECT
         END IF
         
         CALL flioputv (il_ncid, 'dst_address'        , k_dst (1:num_links) )
         CALL flioputv (il_ncid, 'src_address'        , k_src (1:num_links) )
         CALL flioputv (il_ncid, 'remap_matrix '      , w_mct (1:num_wgts,1:num_links) )
         
         !!
         CALL flioclo (il_ncid)
      END IF
      !!
      !!
      RETURN
   END SUBROUTINE wri_weights_a2o
END MODULE mod_wri_wei
