MODULE u_polyg
   !---------------------------------------------------------------------
   !- utilitaires relatifs aux polygones
   !---------------------------------------------------------------------
   USE poly_types
   USE m_g2d
   USE mt_polyg
   !-
   IMPLICIT NONE
   !-
   !$ real,parameter :: v_eps = 1.e-4
   REAL (kind=rp) :: v_eps = 100.0_rp *EPSILON(v_eps)
   !---------------------------------------------------------------------
CONTAINS
   !=
   SUBROUTINE p_dec   (pol,npd,pd)
      !---------------------------------------------------------------------
      !- decomposition d'un polygone quelconque en elements convexes
      !---------------------------------------------------------------------
      TYPE(polyg) :: pol
      INTEGER :: npd
      TYPE(polyg),DIMENSION(:) :: pd
      !-
      TYPE(polyg) :: pv
      INTEGER :: nppol
      INTEGER,DIMENSION(pol%n) :: kpt
      INTEGER,DIMENSION(pol%n) :: tabpt
      REAL (kind=rp), DIMENSION(pol%n)    :: tabcr
      !- elements relatifs aux croisements
      TYPE :: croisement
         REAL (kind=rp)   :: c1,c2
         INTEGER :: k1,k2,kp
      END TYPE croisement
      INTEGER :: nvcr
      TYPE(croisement),DIMENSION(3*pol%n) :: tvcr
      !- elements relatifs aux segments
      TYPE :: segment
         INTEGER :: ideb,ifin
      END TYPE segment
      INTEGER :: nsgm
      TYPE(segment),DIMENSION(7*pol%n) :: tsgm
      !-
      INTEGER :: npt
      TYPE(c2d),DIMENSION(5*pol%n) :: tpt
      !-
      INTEGER :: n1,n2,n3,ii,ki,kp
      INTEGER :: ipd,ipc,ib,npw
      REAL (kind=rp) :: c1,c2,cb,alfa
      TYPE(c2d) :: d1,d2,ptn
      INTRINSIC huge
      !---------------------------------------------------------------------
      npd = 0; pv = pol;
      !- reduction du polygone ---------------------------------------------
      CALL p_red   (pv)
      !- obtention des elements constitutifs du polygone -------------------
      nppol = 0; npt = 0;
      IF (pv%n > 2) THEN
         DO n1=1,pv%n
            nppol = nppol+1; ptn = pv%pt(n1);
            CALL p_apt   (ptn,npt,tpt,kp)
            kpt(nppol) = kp;
         ENDDO
      ENDIF
      !- recherche des croisements -----------------------------------------
      nvcr = 0;
      DO n1=1,nppol-2
         d1 = tpt(kpt(n1+1))-tpt(kpt(n1));
         DO n2=n1+2,nppol+MIN(n1-2,0)
            d2 = tpt(kpt(MOD(n2,nppol)+1))-tpt(kpt(n2));
            CALL p_i2d (tpt(kpt(n1)),d1,tpt(kpt(n2)),d2,c1,c2,ii)
            IF (ii > 0) THEN
               IF (     (     (c1 >  v_eps).AND.(c1 < (1.-v_eps))    &
                  .AND.(c2 > -v_eps).AND.(c2 < (1.+v_eps)) )  &
                  .OR.(     (c1 > -v_eps).AND.(c1 < (1.+v_eps))    &
                  .AND.(c2 >  v_eps).AND.(c2 < (1.-v_eps)) )  ) THEN
                  nvcr = nvcr+1;
                  ptn = tpt(kpt(n2))+c2*d2
                  CALL p_apt   (ptn,npt,tpt,kp)
                  tvcr(nvcr) = croisement(c1,c2,n1,n2,kp)
               ENDIF
            ENDIF
         ENDDO
      ENDDO
      !- creation des segments avec inclusion des croisements --------------
      nsgm = 0;
      DO n1=1,nppol
         !--- creation des points
         ki = 1; tabcr(ki) = 0.; tabpt(ki) = kpt(n1);
         DO n2=1,nvcr
            IF      (n1 == tvcr(n2)%k1) THEN
               ki = ki+1; tabcr(ki) = tvcr(n2)%c1; tabpt(ki) = tvcr(n2)%kp;
            ELSE IF (n1 == tvcr(n2)%k2) THEN
               ki = ki+1; tabcr(ki) = tvcr(n2)%c2; tabpt(ki) = tvcr(n2)%kp;
            ENDIF
         ENDDO
         ki = ki+1; tabcr(ki) = 1.; tabpt(ki) = kpt(MOD(n1,nppol)+1);
         !--- ordonnancement des points
         DO n3=3,ki-1
            DO n2=2,n3-1
               IF (tabcr(n3) < tabcr(n2)) THEN
                  tabcr(n2:n3) = (/ tabcr(n3), tabcr(n2:n3-1) /)
                  tabpt(n2:n3) = (/ tabpt(n3), tabpt(n2:n3-1) /)
                  EXIT
               ENDIF
            ENDDO
         ENDDO
         !--- creation des segments
         DO n2=1,ki-1
            IF (ABS(tabcr(n2+1)-tabcr(n2)) > v_eps) THEN
               nsgm = nsgm+1;
               alfa   = 0.5_rp*(tabcr(n2)+tabcr(n2+1))
               IF (p_qos(nppol,kpt,tpt,n1,alfa)) THEN
                  tsgm(nsgm) = segment(tabpt(n2),tabpt(n2+1));
               ELSE
                  tsgm(nsgm) = segment(tabpt(n2+1),tabpt(n2));
               ENDIF
            ENDIF
         ENDDO
      ENDDO
      !- creation des polygones sans croisement par chainage des segments
      l_p: &
         DO n1=1,nsgm
      !--- recherche du prochain segment disponible
      IF (tsgm(n1)%ideb <= 0)   CYCLE l_p
      !--- creation d'un nouveau polygone
      ipd = tsgm(n1)%ideb; ipc = tsgm(n1)%ifin;
      tsgm(n1)%ideb = -ipd;
      pv%n = 2; pv%pt(1:2) = (/ tpt(ipd),tpt(ipc) /);
      DO
         !----- recherche du segment suivant
         ib = 0; cb = HUGE(cb);
         DO n2=n1+1,nsgm
            IF (tsgm(n2)%ideb == ipc) THEN
               c1 =   (tpt(tsgm(n2)%ifin)-pv%pt(pv%n)) &
                  .a.(pv%pt(pv%n-1)-pv%pt(pv%n))
               IF (ABS(c1) > v_eps) THEN
                  IF (    (ib == 0) &
                     .OR.((cb < 0).AND.(c1 > cb)) &
                     .OR.((cb > 0).AND.(c1 > 0).AND.(c1 < cb)) ) THEN
                     ib = n2; cb = c1;
                  ENDIF
               ENDIF
            ENDIF
         ENDDO
         IF (ib == 0) THEN
            WRITE (unit=*,fmt='(" chainage interrompu !!!")');
            CYCLE l_p;
         ENDIF
         !-----
         tsgm(ib)%ideb = -ipc; ipc = tsgm(ib)%ifin;
         IF (ipc /= ipd) THEN
            !------- extension du polygone
            pv%n = pv%n+1; pv%pt(pv%n) = tpt(ipc);
         ELSE
            !------- traitement des concavites
            CALL p_dcv   (pv,npw,pd(npd+1:))
            npd = npd+npw
            EXIT
         ENDIF
      ENDDO
   ENDDO l_p
   !----------------------
END SUBROUTINE p_dec
!=
SUBROUTINE p_dcv   (pg,npd,pd)
   !---------------------------------------------------------------------
   !- decomposition d'un polygone sans croisement en polygones convexes
   !---------------------------------------------------------------------
   TYPE(polyg) :: pg
   INTEGER :: npd
   TYPE(polyg),DIMENSION(:) :: pd
   !-
   TYPE(polyg) :: pw
   TYPE(polyg),DIMENSION((pg%n+1)/2) :: pp
   INTEGER :: jc,jp,js,np
   INTEGER :: ks,k0,k1,k2
   INTEGER :: jw,ii
   REAL (kind=rp) :: c1,c2,cb
   TYPE(c2d) :: d1,d2,ptn
   INTRINSIC huge
   !---------------------------------------------------------------------
   npd = 0;
   !- memorisation et reduction du polygone
   np = 1; pp(1)%pt(1:pg%n) = pg%pt(1:pg%n); pp(1)%n = pg%n;
   CALL p_red   (pp(1))
   !- recherche et des traitements des concavites
   DO
      IF (np <= 0) EXIT
      jc = 0;
      DO
         pw = pp(np)
         jc = jc+1;
         IF ( (pw%n <= 3).OR.(jc > pw%n) ) EXIT
         jp = MOD(jc-2+pw%n,pw%n)+1; js = MOD(jc,pw%n)+1;
         IF ( ((pw%pt(js)-pw%pt(jc)).a.(pw%pt(jp)-pw%pt(jc))) &
            < -v_eps) THEN
            !------- traitement d'un rebroussement
            !------- partage du polygone pw par le segment (jc,jc+1)
            d1 = pw%pt(MOD(jc,pw%n)+1)-pw%pt(jc);
            !------- recherche de l'intersection la plus proche
            ks = 0; cb = HUGE(cb);
            DO jw=2,pw%n-2
               k0 = MOD(jc+jw-1,pw%n)+1
               d2 = pw%pt(MOD(k0,pw%n)+1)-pw%pt(k0)
               CALL p_i2d (pw%pt(jc),d1,pw%pt(k0),d2,c1,c2,ii)
               IF (ii > 0) THEN
                  IF ( (c2 > -v_eps).AND.(c2 < (1.+v_eps)) ) THEN
                     IF (c1 < v_eps) THEN
                        IF (ABS(c1) < ABS(cb)) THEN
                           ks = k0; ptn = pw%pt(k0)+c2*d2; cb = c1;
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
            IF (ks /= 0) THEN
               !--------- polygones resultants
               k1 = MIN(jc,ks); k2 = MAX(jc,ks);
               pp(np)%n = k2-k1+1;
               pp(np)%pt(1:pp(np)%n) = (/ ptn, pw%pt(k1+1:k2) /);
               CALL p_red (pp(np));
               np = np+1;
               pp(np)%n = k1+1+pw%n-k2;
               pp(np)%pt(1:pp(np)%n) = &
                  (/ pw%pt(1:k1), ptn, pw%pt(k2+1:pw%n) /);
               CALL p_red (pp(np));
               jc = 0;
            ELSE
               WRITE (unit=*,fmt='(" echec !!!")'); STOP
            ENDIF
         ENDIF
      ENDDO
      npd = npd+1; pd(npd) = pw; np = np-1;
   ENDDO
   !----------------------
END SUBROUTINE p_dcv
!=
SUBROUTINE p_apt (point,npt,tpt,kp)
   !---------------------------------------------------------------------
   !- recherche de l'index d'un point dans un tableau
   !- creation de l'element si il n'existe pas
   !---------------------------------------------------------------------
   TYPE(c2d) :: point
   INTEGER :: npt
   TYPE(c2d),DIMENSION(*) :: tpt
   INTEGER :: kp
   !-
   INTEGER :: i
   TYPE(c2d) :: ddp
   !---------------------------------------------------------------------
   kp = 0
   DO i=1,npt
      ddp = point-tpt(i)
      IF ((ddp.s.ddp) < v_eps) THEN
         kp = i
         EXIT
      ENDIF
   ENDDO
   IF (kp == 0) THEN
      npt = npt+1; kp = npt; tpt(kp) = point;
   ENDIF
   !----------------------
END SUBROUTINE p_apt
!=
SUBROUTINE p_i2d (p1,d1,p2,d2,c1,c2,ii)
   !---------------------------------------------------------------------
   !- recherche de l'intersection de deux droites
   !-
   !-        1  droites secantes
   !- ii =   0  droites paralleles distinctes
   !-       -1  droites confondues
   !---------------------------------------------------------------------
   TYPE(c2d) :: p1,d1,p2,d2
   REAL (kind=rp) :: c1,c2
   INTEGER :: ii
   !-
   REAL (kind=rp) :: det
   TYPE(c2d) :: vp,vn
   !---------------------------------------------------------------------
   det = d1.v.d2; vp = p2-p1;
   IF (ABS(det) > v_eps) THEN
      ii = 1;
      c1 = (vp.v.d2)/det; c2 = (vp.v.d1)/det;
   ELSE
      ii =  0;
      vn = c2d(-d1%y,d1%x);
      det = vn.v.d2;
      IF (ABS(det) > v_eps) THEN
         c1 = (vp.v.d2)/det; c2 = (vp.v.vn)/det;
         IF (ABS(c1) < v_eps) THEN
            ii = -1;
         ENDIF
      ENDIF
   ENDIF
   !----------------------
END SUBROUTINE p_i2d
!=
SUBROUTINE p_orp   (pw)
   !---------------------------------------------------------------------
   !- orientation d'un polygone (sans croisement)
   !---------------------------------------------------------------------
   !- comptage des intersections de la demi-mediatrice orientee
   !- du segment (pw%n,1) avec le polygone.
   !- si ce nombre est pair, la demi-mediatrice est exterieure,
   !- et on inverse le sens du polygone.
   !---------------------------------------------------------------------
   TYPE(polyg) :: pw
   !-
   INTEGER :: i,ii,nic,issp,issc
   REAL (kind=rp) :: c1,c2
   TYPE(c2d) :: pm,vt,vn,vw

!$$$   integer :: ntoto = 0
   !---------------------------------------------------------------------
   pm = 0.5_rp*(pw%pt(pw%n)+pw%pt(1));
   vt = pw%pt(1)-pw%pt(pw%n);
   vn = c2d(-vt%y,vt%x);
   !-
   nic = 0; issp = 0;
   DO i=1,pw%n-1
      vw  = pw%pt(i+1)-pw%pt(i);
      CALL p_i2d   (pm,vn,pw%pt(i),vw,c1,c2,ii)
      IF (ii > 0) THEN
!$$$$$ if (c1 > 0.) then
         IF (c1 > -v_eps) THEN
            IF      ( (c2 >  v_eps).AND.(c2 < (1.-v_eps)) ) THEN
               !--------- la normale passe par l'interieur du cote (i,i+1)
               nic = nic+1
               issp = 0;
            ELSE IF ( (c2 > -v_eps).AND.(c2 < (1.+v_eps)) ) THEN
               !--------- la normale passe par le sommet i
               issc = NINT(SIGN(1.,vw.s.vt));
               IF ((issc*issp) > 0) THEN
                  nic = nic+1
               ENDIF
               issp = issc;
            ENDIF
         ENDIF
      ENDIF
   ENDDO
   !-
   IF (MOD(nic,2) == 0) THEN
!$$$     write  (unit=*,fmt='(" inversion du sens du polygone")');
!$$$     ntoto = ntoto + 1
!$$$     write  (unit=*,fmt=*) pw%pt(1:pw%n:1)
!$$$     if ( ntoto > 4 ) stop
      pw%pt(1:pw%n:1) = pw%pt(pw%n:1:-1);
!$$$   else
!$$$     write  (unit=*,fmt='(" sens correct")');
   ENDIF
   !----------------------
END SUBROUTINE p_orp
!=
LOGICAL FUNCTION p_qos   (nppol,kpt,tpt,isc,alfa)
   !---------------------------------------------------------------------
   !- test d'orientation d'un segment de polygone
   !---------------------------------------------------------------------
   !- comptage des intersections de la demi-mediatrice orientee
   !- du segment (isc,isc+alpha*longueur(isc)) avec le polygone.
   !- si ce nombre est pair, la demi-mediatrice est exterieure,
   !- et le polygone est mal oriente.
   !---------------------------------------------------------------------
   INTEGER :: nppol
   INTEGER,DIMENSION(*) :: kpt
   TYPE(c2d),DIMENSION(*) :: tpt
   INTEGER :: isc
   REAL (kind=rp) :: alfa
   !-
   INTEGER :: i,ii,nic,issp,issc,isd
   REAL (kind=rp) :: c1,c2
   TYPE(c2d) :: ptm,pts,normale,segment
   !---------------------------------------------------------------------
   ptm = tpt(kpt(isc))+alfa*(tpt(kpt(MOD(isc,nppol)+1))-tpt(kpt(isc)));
   segment = ptm-tpt(kpt(isc));
   normale%x = -segment%y; normale%y = segment%x;
   !-
   nic = 0; issp = 0;
   DO i=0,nppol-2
      isd = MOD(isc+i,nppol)+1
      pts  = tpt(kpt(MOD(isd,nppol)+1))-tpt(kpt(isd));
      CALL p_i2d   (ptm,normale,tpt(kpt(isd)),pts,c1,c2,ii)
      IF (ii > 0) THEN
         IF (c1 > 0.) THEN
            IF      ( (c2 > v_eps).AND.(c2 < (1.-v_eps)) ) THEN
               !--------- la normale passe par l'interieur du cote (isd,isd+1)
               nic = nic+1
               issp = 0;
            ELSE IF ( (c2 > -v_eps).AND.(c2 < (1.+v_eps)) ) THEN
               !--------- la normale passe par le sommet isd
               issc = NINT(SIGN(1.,pts.s.segment));
               IF ((issc*issp) > 0) THEN
                  nic = nic+1
               ENDIF
               issp = issc;
            ENDIF
         ENDIF
      ENDIF
   ENDDO
   !-
   p_qos = (MOD(nic,2) /= 0)
   !--------------------
END FUNCTION p_qos
!=
SUBROUTINE p_pcp   (p1,p2,p3)
   !---------------------------------------------------------------------
   !- polygone commun a 2 polygones convexes orientes
   !---------------------------------------------------------------------
   TYPE(polyg) :: p1,p2,p3
   !-
   TYPE(polyg),DIMENSION(2) :: pw
   INTEGER :: j1d,j1f,j2d,j2f
   INTEGER :: kc,kr
   INTEGER :: ipd,ipf
   TYPE(c2d) :: v1,v2
   REAL (kind=rp) :: ww
   !---------------------------------------------------------------------
   !- copie du polygone p1 dans le polygone candidat
   kc = 1; pw(kc)%n = p1%n; pw(kc)%pt(1:pw(kc)%n) = p1%pt(1:pw(kc)%n);
   !- decoupage du polygone candidat par chaque cote de p2
   j2d = p2%n;
   l_r: &
      DO j2f=1,p2%n
   !--- definition du polygone resultat
   kr = 3-kc; pw(kr)%n = 0;
   !--- definition du vecteur decoupant
   v2 = p2%pt(j2f)-p2%pt(j2d);
   !--- traitement des cotes successifs du polygone candidat
   j1d = pw(kc)%n;
   !--- position relative (origine segment candidat)/(segment decoupant)
   ipd = n_sgn(v2.v.(pw(kc)%pt(j1d)-p2%pt(j2d)))
   DO j1f=1,pw(kc)%n
      !----- definition du vecteur candidat
      v1 = pw(kc)%pt(j1f)-pw(kc)%pt(j1d);
      !----- position relative (fin segment candidat)/(segment decoupant)
      ipf = n_sgn(v2.v.(pw(kc)%pt(j1f)-p2%pt(j2d)))
      !----- si l'origine appartient au cote decoupant ou est du bon cote,
      !----- on l'insere dans le nouveau polygone candidat
      IF (ipd >= 0) THEN
         pw(kr)%n = pw(kr)%n+1
         pw(kr)%pt(pw(kr)%n) = pw(kc)%pt(j1d)
      ENDIF
      !----- si le segment candidat est coupe par le segment decoupant,
      !----- on introduit l'intersection dans le polygone resultat
      IF ((ipd*ipf) < 0) THEN
         pw(kr)%n = pw(kr)%n+1
         ww  = (v2.v.(pw(kc)%pt(j1d)-p2%pt(j2d)))/(v1.v.v2)
         pw(kr)%pt(pw(kr)%n) = pw(kc)%pt(j1d)+ww*v1
      ENDIF
      !----- l'extremite du segment sera l'origine du segment suivant
      ipd = ipf; j1d = j1f;
   ENDDO
   !--- si le polygone resultat est degenere, il n'y a pas de recouvrement
   IF (pw(kr)%n < 3)   EXIT l_r
   !--- le polygone resultat devient polygone candidat
   kc = kr;
   !--- passage au cote suivant de p2
   j2d = j2f;
ENDDO l_r
!- fin de decoupage : rangement du resultat dans p3
IF (pw(kr)%n < 3) THEN
   p3%n = 0
ELSE
   p3%n = pw(kr)%n
   p3%pt(1:p3%n) = pw(kr)%pt(1:pw(kr)%n)
ENDIF
!----------------------
END SUBROUTINE p_pcp
!=
SUBROUTINE p_red (p0)
   !---------------------------------------------------------------------
   !- reduction d'un polygone
   !- suppression des doublons et alignements
   !---------------------------------------------------------------------
TYPE(polyg) :: p0
!-
INTEGER :: ip,ic,is
TYPE(c2d) :: dp,ds
!---------------------------------------------------------------------
ic = 1;
DO
   IF ( (p0%n < 3).OR.(ic > p0%n) ) EXIT
   ip = MOD(ic-2+p0%n,p0%n)+1; is = MOD(ic,p0%n)+1;
   dp = p0%pt(ic)-p0%pt(ip); ds = p0%pt(is)-p0%pt(ic);
!$$$ if      ((dp.s.dp) < v_eps) then
!$$$   p0%pt(ip:p0%n-1) = p0%pt(ip+1:p0%n); p0%n = p0%n-1;
!$$$ else if ((ds.s.ds) < v_eps) then
!$$$   p0%pt(is:p0%n-1) = p0%pt(is+1:p0%n); p0%n = p0%n-1;
!$$$ else if (abs(dp.v.ds) < v_eps) then
   IF (ABS(dp.v.ds) < v_eps) THEN
      p0%pt(ic:p0%n-1) = p0%pt(ic+1:p0%n); p0%n = p0%n-1;
      ic = MAX(ic-1,1);
   ELSE
      ic = ic+1;
   ENDIF
ENDDO
!----------------------
END SUBROUTINE p_red
!=
INTEGER FUNCTION n_sgn (v)
   !---------------------------------------------------------------------
   REAL (kind=rp):: v
   !---------------------------------------------------------------------
   IF      (v >  v_eps) THEN
      n_sgn =  1
   ELSE IF (v < -v_eps) THEN
      n_sgn = -1
   ELSE
      n_sgn =  0
   ENDIF
   !--------------------
END FUNCTION n_sgn
!=
FUNCTION p_srf   (pw)
   !---------------------------------------------------------------------
   !- calcul de la surface d'un polygone (sans croisement)
   !---------------------------------------------------------------------
   !- sommation des surfaces des triangles orientes composant le polygone
   !---------------------------------------------------------------------
   REAL (kind=rp) :: p_srf
   TYPE(polyg) :: pw
   !-
   INTEGER :: j1,j2,j3,jp
   !---------------------------------------------------------------------
   p_srf = 0.0_rp ; j1 = 1;
   DO jp=1,pw%n-2
      j2 = jp+1; j3 = jp+2;
      p_srf = p_srf+((pw%pt(j2)-pw%pt(j1)).v.(pw%pt(j3)-pw%pt(j1)))
   ENDDO
   p_srf = 0.5_rp * ABS(p_srf)
   !--------------------
END FUNCTION p_srf
!=
INTEGER FUNCTION p_tap   (pq,pw)
   !---------------------------------------------------------------------
   !- test d'appartenance d'un point a un polygone
   !---------------------------------------------------------------------
   !- calcul du nombre d'intersections de la demi-droite perpendiculaire
   !- au premier cote avec les autres cotes.
   !- si ce nombre est pair, le point appartient au polygone.
   !---------------------------------------------------------------------
   TYPE(c2d) :: pq
   TYPE(polyg) :: pw
   !-
   INTEGER :: jp,ii,nic,issp,issc
   REAL (kind=rp)   :: c1,c2
   TYPE(c2d) :: vq,vs
   !---------------------------------------------------------------------
   nic = 0; issp = 0;
vq  = c2d(pw%pt(1)%y-pw%pt(2)%y,pw%pt(2)%x-pw%pt(1)%x)
DO jp=1,pw%n
   vs   = pw%pt(MOD(jp,pw%n)+1)-pw%pt(jp)
   CALL p_i2d   (pq,vq,pw%pt(jp),vs,c1,c2,ii)
   IF (ii > 0) THEN
      !----- la demi-droite coupe un cote
      IF (c1 > v_eps) THEN
         !------- le point est externe a la droite portant le cote
         IF      ( (c2 > v_eps).AND.(c2 < (1.-v_eps)) ) THEN
            !--------- la demi-droite passe par l'interieur du cote
            nic = nic+1
            issp = 0;
         ELSE IF ( (c2 > -v_eps).AND.(c2 < (1.+v_eps)) ) THEN
            !--------- la demi-droite passe par un sommet
            issc = NINT(SIGN(1.,vq.s.vs));
            IF ((issc*issp) < 0) THEN
               nic = nic+1
               issp = 0;
            ENDIF
            issp = issc;
         ENDIF
      ELSE IF (ABS(c1) < v_eps) THEN
         !------- le point est sur la droite portant le cote
         IF ( (c2 > -v_eps).AND.(c2 < (1.+v_eps)) ) THEN
            !--------- le point appartient au cote
            nic = -1
            EXIT
         ENDIF
      ENDIF
   ELSE IF (ii < 0) THEN
      !----- la demi-droite est colineaire a un cote
      IF ( (c2 > -v_eps).AND.(c2 < (1.+v_eps)) ) THEN
         nic = -1
         EXIT
      ENDIF
   ENDIF
ENDDO
!-
p_tap = MOD(nic,2)
!--------------------
END FUNCTION p_tap
!=
!-----------------
END MODULE u_polyg
